@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from ... import globals
 import re
 def determine_domain(path) -> str:
     codes = open(path, 'r').read()
@@ -23,5 +22,11 @@ def determine_domain(path) -> str:
             return 'transformers_trainer'
         else:
             return 'transformers_no_trainer'
+    elif 'onnx.load(' in codes:
+        return 'onnx'
+    elif 'keras.Sequential' in codes:
+        return 'keras_script'
+    elif 'from tensorflow import' in codes or 'import tensorflow' in codes:
+        return 'tensorflow_keras_model'
     else:
         return 'random model'
