@@ -52,7 +52,7 @@ def enable(
     save_patch_path="",
     patch_suffix=".diff",
     remove_copy=True,
-    consider_imports=True,
+    consider_imports=False,
     patch_imports=False,
     logging_level="info",
     run_bench=False,
@@ -66,7 +66,7 @@ def enable(
     test_code_line=False, # print code line info for debug use
     cache_load_transformers=True,
     optimum_quant_config="", # only for HF optimum optimizations, yaml or hub path
-    use_inc=False,
+    use_inc=True,
     use_modular=False,
     modular_item="",
 ):
@@ -196,6 +196,7 @@ def enable(
         "pytorch_cuda_to_cpu",
         "pytorch_lightning_bf16_cpu",
         "tensorflow_mixed_precision",
+        "tensorflow_inc",
         "change_trainer_to_nlptrainer",
     ]
     
@@ -374,7 +375,7 @@ def enable(
                 if "tensorflow_mixed_precision" in features:
                     from .coders.tensorflow.amp import TensorFlowKerasAMP
                     list_transformed_code[i] = TensorFlowKerasAMP(list_transformed_code[i]).transform()
-                if "tensorflow_inc" in features:
+                if feature == "tensorflow_inc":
                     from .coders.tensorflow.inc import TensorFlowKerasINC
                     list_transformed_code[i] = TensorFlowKerasINC(list_transformed_code[i]).transform()
                 # Change Trainer to NLPTrainer (only for intel_extension_for_pytorch)
@@ -751,7 +752,7 @@ def superbench(
     ncore_per_instance=-1,  # only for "self_defined" mode
     ninstances=-1,  # only for "self_defined" mode
     bench_batch_size=-1,  # only for "self_defined" mode
-    use_inc=False,
+    use_inc=True,
     auto_quant=False,
 ):
 
@@ -1280,7 +1281,7 @@ def auto_quant(
     ncore_per_instance=-1,  # only for "self_defined" mode
     ninstances=-1,  # only for "self_defined" mode
     bench_batch_size=-1,  # only for "self_defined" mode
-    use_inc=False,
+    use_inc=True,
 ):
     return superbench(
         code,
