@@ -15,15 +15,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .quantization import Quantization
-from .pruning import Pruning
-from .benchmark import Benchmark
+"""Intel® Neural Compressor: An open-source Python library supporting popular model compression techniques."""
+from .benchmark import _Benchmark
 from .version import __version__
 from .contrib import *
-from .model.model import export
 # we need to set a global 'NA' backend, or Model can't be used
-from .utils.utility import set_backend
+from .utils.utility import set_random_seed, set_tensorboard, set_workspace
 from .utils import options
 from .conf.config import conf
-
-set_backend('NA')
+from .conf.pythonic_config import config
+from .config import DistillationConfig, PostTrainingQuantConfig, \
+                    WeightPruningConfig, QuantizationAwareTrainingConfig, \
+                    MixedPrecisionConfig
