@@ -28,17 +28,23 @@ from ..utils.utility import LazyImport, CpuInfo, singleton, Dequantize, dump_ela
 from ..utils.utility import Statistics, GLOBAL_STATE, MODE, version1_lt_version2
 from ..utils import logger
 from ..conf.dotdict import deep_get
-from ..experimental.data.dataloaders.base_dataloader import BaseDataLoader
+from ..data.dataloaders.base_dataloader import BaseDataLoader
 tf = LazyImport('tensorflow')
 
 def _add_supported_quantized_objects(custom_objects):
   """Map all the quantized objects."""
   from neural_compressor.adaptor.keras_utils.quantizer import Quantize, DeQuantize
-  from neural_compressor.adaptor.keras_utils.quantizer import FakeQuant, QConv2D, QDense
+  from neural_compressor.adaptor.keras_utils.quantizer import FakeQuant
+  from neural_compressor.adaptor.keras_utils.conv2d import QConv2D
+  from neural_compressor.adaptor.keras_utils.depthwise_conv2d import QDepthwiseConv2D
+  from neural_compressor.adaptor.keras_utils.separable_conv2d import QSeparableConv2D
+  from neural_compressor.adaptor.keras_utils.dense import QDense
   custom_objects["Quantize"] = Quantize
   custom_objects["DeQuantize"] = DeQuantize
   custom_objects["FakeQuant"] = FakeQuant
   custom_objects["QConv2D"] = QConv2D
+  custom_objects["QDepthwiseConv2D"] = QDepthwiseConv2D
+  custom_objects["QSeparableConv2D"] = QSeparableConv2D
   custom_objects["QDense"] = QDense
   return custom_objects
 
@@ -56,7 +62,9 @@ class KerasAdaptor(Adaptor):
         #self.work_dir = os.path.abspath(self.framework_specific_info['workspace_path'])
         self.recipes = deep_get(self.framework_specific_info, 'recipes', {})
         #os.makedirs(self.work_dir, exist_ok=True)
+        self.supported_op = ['Conv2D', 'Dense', 'SeparableConv2D', 'DepthwiseConv2D']
 
+        self.pre_optimized_object = None
         self.pre_optimized_model = None
         self.pre_optimizer_handle = None
         self.fp32_ops = []
@@ -69,6 +77,8 @@ class KerasAdaptor(Adaptor):
         self.callbacks = []
         self.optype_statistics = None
 
+        self.conv_format = {}
+
     def tuning_cfg_to_fw(self, tuning_cfg):
         self.quantize_config['calib_iteration'] = tuning_cfg['calib_iteration']
         self.quantize_config['device'] = self.device
@@ -106,6 +116,145 @@ class KerasAdaptor(Adaptor):
                                                                weight_bit)
         self.fp32_ops = fp32_ops
 
+    def _pre_optimize(self, model):
+        model = self._check_quantize_format(model)
+        model = self._fuse_bn(model)
+        return model
+
+    def _check_quantize_format(self, model):
+        json_model = copy.deepcopy(json.loads(model.to_json()))
+        config = json_model["config"]
+        fp32_layers = config["layers"]
+        name_op_map = {}
+
+        for idx, layer in enumerate(copy.deepcopy(fp32_layers)):
+            name_op_map[layer['config']['name']] = layer
+
+        for idx, layer in enumerate(copy.deepcopy(fp32_layers)):
+            layer_config = layer['config']
+            if layer['class_name'] in self.supported_op:
+                if 'inbound_nodes' in layer:
+                    check_layer = name_op_map[layer['inbound_nodes'][0][0][0]]
+                else:
+                    check_layer = fp32_layers[idx - 1]
+                if check_layer['class_name'] in ['Activation'] and \
+                        check_layer['config']['activation'] in ['relu']:
+                    self.conv_format[layer['config']['name']] = 'u8'
+                else:
+                    self.conv_format[layer['config']['name']] = 's8'
+        return model
+
+    def _fuse_bn(self, model):
+        json_model = copy.deepcopy(json.loads(model.to_json()))
+        config = json_model["config"]
+        fp32_layers = config["layers"]
+        def fuse_conv_bn(conv_weight, bn_weight, conv_type='Conv2D', eps=1.0e-5):
+            assert conv_type in ['Conv2D', 'DepthwiseConv2D', 'SeparableConv2D'], \
+                    'only support Conv2D, DepthwiseConv2D, SeparableConv2D...'
+            if len(bn_weight) > 3:
+                if conv_type == 'DepthwiseConv2D':
+                    gamma = bn_weight[0].reshape(1, 1, bn_weight[0].shape[0], 1) 
+                    var = bn_weight[3].reshape(1, 1, bn_weight[3].shape[0], 1)
+                else:
+                    gamma = bn_weight[0].reshape(1, 1, 1, bn_weight[0].shape[0])
+                    var = bn_weight[3].reshape(1, 1, 1, bn_weight[3].shape[0])
+                beta = bn_weight[1]
+                mean = bn_weight[2]
+            else:
+                gamma = 1.
+                beta = bn_weight[0]
+                mean = bn_weight[1]
+                if conv_type == 'DepthwiseConv2D':
+                    var = bn_weight[2].reshape(1, 1, bn_weight[2].shape[0], 1)
+                else:
+                    var = bn_weight[2].reshape(1, 1, 1, bn_weight[2].shape[0])
+
+            if len(conv_weight) == 1:
+                weight = conv_weight[0]
+                bias = np.zeros_like(beta)
+            elif len(conv_weight) == 2 and conv_type == 'SeparableConv2D':
+                depth_weight = conv_weight[0]
+                weight = conv_weight[1]
+                bias = np.zeros_like(beta)
+            elif len(conv_weight) == 2 and conv_type != 'SeparableConv2D':
+                weight = conv_weight[0]
+                bias = conv_weight[1]
+            elif len(conv_weight) == 3:
+                depth_weight = conv_weight[0]
+                weight = conv_weight[1]
+                bias = conv_weight[2]
+            scale_value = gamma / np.sqrt(var + eps)
+            weight = weight * scale_value
+            bias = beta + (bias - mean) * scale_value.reshape(-1)
+            bias = bias.reshape(-1)
+            return [depth_weight, weight, bias] if conv_type == 'SeparableConv2D' \
+                else [weight, bias] 
+
+        node_map = {}
+        for idx, layer in enumerate(copy.deepcopy(fp32_layers)):
+            layer_config = layer['config']
+            if 'inbound_nodes' in layer:
+                node_map[layer['name']] = layer
+
+        fuse_layers = []
+        fold_conv = []
+        for idx, layer in enumerate(copy.deepcopy(fp32_layers)):
+            layer_config = layer['config']
+            if 'inbound_nodes' in layer: 
+                if layer['class_name'] in ['BatchNormalization']:
+                    bn_inbound_node = node_map[layer_config['name']]['inbound_nodes'][0][0]
+                    if bn_inbound_node[0] in self.conv_weights.keys():
+                        conv_weight = self.conv_weights[bn_inbound_node[0]]
+                        conv_layer = node_map[bn_inbound_node[0]]
+                        bn_weight = self.bn_weights[layer_config['name']]
+                        self.layer_weights[bn_inbound_node[0]] = fuse_conv_bn(
+                            conv_weight, bn_weight, conv_layer['class_name'], layer['config']['epsilon'])
+                        fold_conv.append(bn_inbound_node[0])
+                    else:
+                        fuse_layers.append(layer)
+                elif len(layer['inbound_nodes']):
+                    new_bound_nodes = []
+                    # OpLambda node will have different bound node
+                    if layer['class_name'] in ['TFOpLambda', 'SlicingOpLambda']:
+                        fuse_layers.append(layer)
+                    else:
+                        for bound_node in layer['inbound_nodes'][0]:
+                            if bound_node[0] in self.bn_weights.keys():
+                                bn_inbound_node = node_map[bound_node[0]]['inbound_nodes'][0][0]
+                                if bn_inbound_node[0] in self.conv_weights.keys():
+                                    new_bound_nodes.append(bn_inbound_node)
+                                else:
+                                    new_bound_nodes.append(bound_node)
+                            else:
+                                new_bound_nodes.append(bound_node)
+                        layer['inbound_nodes'] = [new_bound_nodes]
+                        fuse_layers.append(layer)
+                else:
+                    fuse_layers.append(layer)
+            else:
+                if idx > 0 and layer['class_name'] in ['BatchNormalization'] and \
+                    fp32_layers[idx - 1]['class_name'] in ['Conv2D']:
+                    conv_name = fp32_layers[idx - 1]['config']['name']
+                    conv_weight = self.conv_weights[conv_name]
+                    bn_weight = self.bn_weights[layer_config['name']]
+                    conv_type = fp32_layers[idx - 1]['class_name']
+                    self.layer_weights[conv_name] = fuse_conv_bn(
+                        conv_weight, bn_weight, conv_type, layer['config']['epsilon'])
+                    fold_conv.append(conv_name)
+                else:
+                    fuse_layers.append(layer)
+
+        # bn folding will have a shift bias
+        for idx, layer in enumerate(fuse_layers):
+            layer_config = layer['config']
+            if layer['class_name'] in ['Conv2D', 'DepthwiseConv2D', 'SeparableConv2D'] and \
+                    layer_config['name'] in fold_conv:
+                layer_config['use_bias'] = True
+            
+        json_model['config']['layers'] = fuse_layers
+        fused_model = self._restore_model_from_json(json_model)
+        return fused_model
+
     @dump_elapsed_time("Pass quantize model")
     def quantize(self, tune_cfg, model, dataloader, q_func=None):
         '''Execute the quantize process on the specified model.
@@ -145,23 +294,32 @@ class KerasAdaptor(Adaptor):
                     "So the real sampling size is {}.".
                     format(calib_sampling_size, dataloader.batch_size,
                            dataloader.batch_size * iter))
+
         q_layers = []
-        for idx, layer in enumerate(self.fp32_layers):
+        self.inbound_nodes_map = {}
+        for idx, layer in enumerate(copy.deepcopy(self.fp32_layers)):
           layer_config = layer["config"]
-          if layer["class_name"] in ["Conv2D", "Dense"] and \
+          if layer["class_name"] in self.supported_op and \
             layer['config']['name'] in self.quantize_config['op_wise_config']:
               op_config = self.quantize_config['op_wise_config'][layer['config']['name']]
               mode = 'per_channel' if op_config[0] else 'per_tensor'
-              #(TODO) support asym/sym
-              fake_quant_name = 'fake_quant_' + str(idx)
-              q_layers.append({'class_name': 'FakeQuant', 
-                  'config': {'mode': 'per_tensor', 'name': fake_quant_name}})
+              fake_q_name = 'fake_quant_' + str(idx)
+              fake_q_layer = {'class_name': 'FakeQuant', 
+                              'name': fake_q_name,
+                              'T': self.conv_format[layer['config']['name']],
+                              'config': {'mode': 'per_tensor', 'name': fake_q_name}, 
+                              }
+              if 'inbound_nodes' in layer:
+                  fake_q_layer['inbound_nodes'] = layer['inbound_nodes']
+                  layer['inbound_nodes'] = [[[fake_q_name, 0, 0, {}]]]
+                  self.inbound_nodes_map[fake_q_name] = layer
+              
+              q_layers.append(fake_q_layer)
               q_layers.append(layer)
           else:
               q_layers.append(layer)
-
-        keras_object = model._model_object
-        json_model = copy.deepcopy(json.loads(keras_object.to_json()))
+        
+        json_model = copy.deepcopy(json.loads(self.pre_optimized_object.to_json()))
         json_model['config']['layers'] = q_layers
         quantized_model = self._restore_model_from_json(json_model)
 
@@ -199,40 +357,85 @@ class KerasAdaptor(Adaptor):
         config = json_model["config"]
         layers = config["layers"]
         q_layers = []
-        for layer in layers:
+        # quantize_mode = self._check_quantize_mode(json_model)
+        inbound_reverse_map = {}
+        for idx,  layer in enumerate(layers):
             layer_config = copy.deepcopy(layer['config'])
             if layer['class_name'] == 'FakeQuant':
                 min_value = min(results[layer['config']['name']]['min'])
                 max_value = max(results[layer['config']['name']]['max'])
-                q_layers.append({'class_name': 'Quantize',
-                                 'config': {'min_range': min_value,
-                                            'max_range': max_value,
-                                           }})
-                q_layers.append({'class_name': 'DeQuantize',
-                                 'config': {'min_range': min_value,
-                                            'max_range': max_value,
-                                           }})
-            elif layer['class_name'] == 'Conv2D' or layer['class_name'] == 'Dense':
+                quantize_layer = {'class_name': 'Quantize',
+                                  'name': 'quantize_' + str(idx), 
+                                  'config': {'min_range': min_value,
+                                             'max_range': max_value,
+                                             'T': layer_config['T'],
+                                             'name': 'quantize_' + str(idx), 
+                                            }}
+                dequantize_layer = {'class_name': 'DeQuantize',
+                                    'name': 'dequantize_' + str(idx), 
+                                    'config': {'min_range': min_value,
+                                                'max_range': max_value,
+                                                # 'mode': quantize_mode,
+                                                'name': 'dequantize_' + str(idx), 
+                                               }}
+                if 'inbound_nodes' in layer:
+                    quantize_layer['inbound_nodes'] = layer['inbound_nodes']
+                    dequantize_layer['inbound_nodes'] = [[['quantize_' + str(idx), 0, 0, {}]]]
+                    # find the conv/dense layer from fake quant map and 
+                    # change the conv/dense node inbound to dequantize
+                    layer_name = self.inbound_nodes_map[layer['name']]['name']
+                    inbound_reverse_map[layer_name] = [[['dequantize_' + str(idx), 0, 0, {}]]]
+                    
+                q_layers.append(quantize_layer)
+                q_layers.append(dequantize_layer)
+            elif layer['class_name'] in self.supported_op and \
+                layer['config']['name'] in self.quantize_config['op_wise_config']:
                 # index 0 is weight, index 1 is bias
                 q_layer_name = 'Q' + layer['class_name']
+                # this is for inbounds search
+                q_name  = layer['config']['name']
                 kernel = self.layer_weights[layer['config']['name']][0]
-                layer_config['min_value'] = str(kernel.min())
-                layer_config['max_value'] = str(kernel.max())
-                q_layers.append({'class_name': q_layer_name, 'config': layer_config})
+                dim = list(range(0, kernel.ndim)) 
+                t_dim = [dim.pop(-1)]
+                t_dim.extend(dim)
+                channel_size = kernel.shape[-1]
+                kernel_channel = kernel.transpose(t_dim).reshape(channel_size, -1)
+                layer_config['min_value'] = json.dumps(\
+                        np.min(kernel_channel, axis=1).tolist())
+                layer_config['max_value'] = json.dumps(\
+                        np.max(kernel_channel, axis=1).tolist())
+                layer_config['name'] = q_name
+                q_layer = {'class_name': q_layer_name,
+                           'name': q_name,
+                           'config': layer_config}
+                if 'inbound_nodes' in layer:
+                    q_layer['inbound_nodes'] = inbound_reverse_map[layer['name']]
+                q_layers.append(q_layer)
             else:
-                q_layers.append(layer) 
+                q_layers.append(layer)
 
         json_model['config']['layers'] = q_layers
         quantized_model = self._restore_model_from_json(json_model)
         return quantized_model
 
+
+    #(TODO) choose the properly quantize mode
+    def _check_quantize_mode(self, json_model):
+        config = json_model["config"]
+        layers = config["layers"]
+        for idx,  layer in enumerate(layers):
+            if 'ReLU' in layer['class_name']:
+                return 'MIN_FIRST'
+        return 'SCALED'
+
     def _restore_model_from_json(self, json_model):
         from tensorflow.keras.models import model_from_json
         custom_objects = {}
         # We need to keep a dictionary of custom objects as our quantized library
         # is not recognized by keras.
         custom_objects = _add_supported_quantized_objects(custom_objects)
-        qmodel = model_from_json(json.dumps(json_model), custom_objects=custom_objects)
+        json_model_file = json.dumps(json_model)
+        qmodel = model_from_json(json_model_file, custom_objects=custom_objects)
         qmodel = self._set_weights(qmodel, self.layer_weights)
         return qmodel
 
@@ -295,7 +498,10 @@ class KerasAdaptor(Adaptor):
                         metric.update(predictions, labels)
             if idx + 1 == iteration:
                 break
-        return results
+
+        acc = 0 if metrics is None else [metric.result() for metric in metrics]
+
+        return acc if not isinstance(acc, list) or len(acc) > 1 else acc[0]
 
     def query_fw_capability(self, model):
         '''The function is used to return framework tuning capability.
@@ -303,25 +509,36 @@ class KerasAdaptor(Adaptor):
            Args:
                model (object): The model to query quantization tuning capability.
         '''
-        self.pre_optimized_model = model
         fp32_config = {'weight': {'dtype': 'fp32'}, 'activation': {'dtype': 'fp32'}}
         int8_type = self.query_handler.get_op_types_by_precision(precision='int8')
         op_capability = self.query_handler.get_quantization_capability()
         conv_config = copy.deepcopy(op_capability['int8']['Conv2D'])
+        conv_config = copy.deepcopy(op_capability['int8']['SeparableConv2D'])
+        conv_config = copy.deepcopy(op_capability['int8']['DepthwiseConv2D'])
         dense_config = copy.deepcopy(op_capability['int8']['Dense'])
         other_config = copy.deepcopy(op_capability['int8']['default'])
 
-        # get the layers info
+        # # get fp32 layer weights
         keras_object = model._model_object
-        json_model = copy.deepcopy(json.loads(keras_object.to_json()))
-        config = json_model["config"]
-        self.fp32_layers = config["layers"]
-
-        # get fp32 layer weights
+        self.conv_weights = {}
+        self.bn_weights = {}
         self.layer_weights = {}
         for layer in keras_object.layers:
             if layer.get_weights():
+                if isinstance(layer, tf.keras.layers.Conv2D) or \
+                   isinstance(layer, tf.keras.layers.DepthwiseConv2D) or \
+                   isinstance(layer, tf.keras.layers.SeparableConv2D):
+                    self.conv_weights[layer.name] = copy.deepcopy(layer.get_weights())
+                elif isinstance(layer, tf.keras.layers.BatchNormalization):
+                    self.bn_weights[layer.name] = copy.deepcopy(layer.get_weights())
                 self.layer_weights[layer.name] = copy.deepcopy(layer.get_weights())
+        self.pre_optimized_object = self._pre_optimize(keras_object)
+
+        from neural_compressor.model.keras_model import KerasModel
+        self.pre_optimized_model = KerasModel(self.pre_optimized_object)
+        json_model = copy.deepcopy(json.loads(self.pre_optimized_object.to_json()))
+        config = json_model["config"]
+        self.fp32_layers = config["layers"]
 
         quantizable_op_details = OrderedDict()
         for details in self.fp32_layers:
@@ -389,7 +606,35 @@ class KerasAdaptor(Adaptor):
                  ]
                }
         '''
-        pass
+        assert inspect_type in ['weight', 'activation', 'all'], \
+                'Inspect type only support weight, activation or all'
+        from keras import backend as K # pylint: disable=E0401
+        tensor_out = {}
+        inp = model.input
+        outputs = [(layer.name, layer.output) for layer in model.layers]
+        outputs = [(name, out) for (name, out) in outputs if name in op_list] if len(op_list) else outputs
+        if inspect_type == 'weight' or inspect_type == 'all':
+            weights = [(layer.name, layer.get_weights()) for layer in model.layers if layer.get_weights()]
+            weights = [(name, wei) for (name, wei) in weights if name in op_list] if len(op_list) else weights
+            tensor_out['weight'] = weights
+
+        functors = [(name, K.function([inp], [out])) for (name, out) in outputs]
+        iterations = max(iteration_list) if iteration_list is not None else -1
+        observer_dict = {}
+        ret = {}
+        if inspect_type == 'activation' or inspect_type == 'all':
+            activation_list = []
+            for idx, (inputs, labels) in enumerate(dataloader):
+                layer_outs = [(name, func([inputs])) for (name, func) in functors]
+                iter_map = {}
+                for (name, out_list) in layer_outs:
+                    iter_map[name] = dict([(name+':'+str(i), out) for i, out in enumerate(out_list)])
+                activation_list.append(iter_map)
+                if idx == iterations:
+                    break
+            tensor_out['activation'] = [acti for idx, acti in enumerate(activation_list) if idx in iteration_list]
+
+        return tensor_out
 
     def set_tensor(self, model, tensor_dict):
         '''The function is used by tune strategy class for setting tensor back to model.
