@@ -1,3 +1,4 @@
+"""mxnet util module."""
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
@@ -24,7 +25,7 @@ import numpy as np
 from enum import Enum
 from tempfile import TemporaryDirectory
 from neural_compressor.utils.utility import LazyImport
-from neural_compressor.model.model import MXNetModel as NCModel
+from neural_compressor.model.mxnet_model import MXNetModel as NCModel
 
 mx = LazyImport("mxnet")
 
@@ -39,6 +40,7 @@ NULL_OP_NAMES = ['null']
 
 
 class OpType(Enum):
+    """Enum op types."""
     NORMAL = 0
     QUANTIZE = 1
     QUANTIZED = 2
@@ -62,6 +64,12 @@ def isiterable(obj) -> bool:
 
 def ensure_list(x):
     """Ensures that object is a list.
+
+    Args:
+        x : input.
+
+    Returns:
+        list: x if x is list, else [x].
     """
     return x if isinstance(x, (tuple, list)) else [x]
 
@@ -83,6 +91,15 @@ def check_mx_version(version):
 
 
 def combine_capabilities(current, new):
+    """Combine capabilities.
+
+    Args:
+        current (dict): current capabilities.
+        new (dict): new capabilities.
+
+    Returns:
+        dict: contains all capabilities.
+    """
     if isinstance(current, list):
         assert isinstance(new, list)
         return list(set(current) | set(new))
@@ -99,13 +116,13 @@ def combine_capabilities(current, new):
 def make_nc_model(target, sym_model, ctx, input_desc):
     """Converts a symbolic model to an Neural Compressor model.
 
-        Args:
-            target (object): target model type to return.
-            sym_model (tuple): symbol model (symnet, args, auxs).
-            input_desc (list): model input data description.
+    Args:
+        target (object): target model type to return.
+        sym_model (tuple): symbol model (symnet, args, auxs).
+        input_desc (list): model input data description.
 
-        Returns:
-            NCModel: converted neural_compressor model
+    Returns:
+        NCModel: converted neural_compressor model
     """
     assert isinstance(sym_model, tuple) and isinstance(sym_model[0], mx.symbol.Symbol)
 
@@ -117,25 +134,33 @@ def make_nc_model(target, sym_model, ctx, input_desc):
 def fuse(sym_model, ctx):
     """Fuse the supplied model.
 
-        Args:
-            sym_model (tuple): symbol model (symnet, args, auxs).
+    Args:
+        sym_model (tuple): symbol model (symnet, args, auxs).
 
-        Returns:
-            tuple: fused symbol model (symnet, args, auxs).
+    Returns:
+        tuple: fused symbol model (symnet, args, auxs).
     """
     assert isinstance(sym_model, tuple) and isinstance(sym_model[0], mx.symbol.Symbol)
 
     symnet, args, auxs = sym_model
-    backend = get_backend_name(ctx)
-    if backend is not None:
+    framework = get_framework_name(ctx)
+    if framework is not None:
         if check_mx_version('2.0.0'):
-            symnet = symnet.optimize_for(backend)
+            symnet = symnet.optimize_for(framework)
         else:
-            symnet = symnet.get_backend_symbol(backend)
+            symnet = symnet.get_backend_symbol(framework)
     return (symnet, args, auxs)
 
 
-def get_backend_name(ctx):
+def get_framework_name(ctx):
+    """Get the framework name by version.
+
+    Args:
+        ctx (object): mxnet context object.
+
+    Returns:
+        str: framework name.
+    """
     if 'cpu' in ctx.device_type:
         if check_mx_version('2.0.0'):
             return 'ONEDNN_QUANTIZE'
@@ -160,6 +185,16 @@ def prepare_model_data(nc_model, ctx, data_x):
 
 
 def prepare_model(nc_model, ctx, input_desc):
+    """Prepare model.
+
+    Args:
+        nc_model (object): model to prepare.
+        ctx (object): mxnet context object.
+        input_desc (list): input list of mxnet data types.
+
+    Returns:
+        object: mxnet model (symnet, args, auxs).
+    """
     assert isinstance(nc_model, NCModel)
 
     model_x = nc_model.model
@@ -181,6 +216,15 @@ def prepare_model(nc_model, ctx, input_desc):
 
 
 def create_data_example(ctx, input_desc):
+    """Create data example by mxnet input description and ctx.
+
+    Args:
+        ctx (object): mxnet context object.
+        input_desc (list): input list of mxnet data types.
+
+    Returns:
+        list: data example.
+    """
     if mx.is_np_array():
         return [mx.np.zeros(d.shape, ctx=ctx, dtype=d.dtype) for d in input_desc]
     else:
@@ -188,6 +232,14 @@ def create_data_example(ctx, input_desc):
 
 
 def _match_array_semantics(sym_model):
+    """Match array semantics.
+
+    Args:
+        sym_model (tuple): symbol model (symnet, args, auxs).
+
+    Returns:
+        tuple: symbol model (symnet, args, auxs).
+    """
     if check_mx_version('2.0.0') and mx.util.is_np_array():
         symnet, args, auxs = sym_model
         symnet = symnet.as_np_ndarray()
@@ -200,6 +252,16 @@ def _match_array_semantics(sym_model):
 
 
 def prepare_dataloader(nc_model, ctx, data_x):
+    """Prepare dataloader.
+
+    Args:
+        nc_model (object): model to prepare.
+        ctx (object): mxnet context object.
+        data_x (object): mxnet io iterable object or dataloader object.
+
+    Returns:
+        object: dataloader.
+    """
     assert isinstance(nc_model, NCModel)
 
     if isinstance(data_x, DataLoaderWrap):
@@ -236,6 +298,15 @@ def prepare_dataloader(nc_model, ctx, data_x):
 
 
 def ndarray_to_device(ndarray, device):
+    """Ndarray to device.
+
+    Args:
+        ndarray (ndarray): model to prepare.
+        device (object): mxnet device object.
+
+    Returns:
+        ndarray: ndarray on the device.
+    """
     try:
         return ndarray.to_device(device)  # 2.x version
     except AttributeError:
@@ -245,10 +316,11 @@ def ndarray_to_device(ndarray, device):
 def is_model_quantized(sym_model):
     """Checks whether the model is quantized.
 
-        Args:
-            sym_model (tuple): symbol model (symnet, args, auxs).
-        Returns:
-            boolean: True if model is quantized, else False.
+    Args:
+        sym_model (tuple): symbol model (symnet, args, auxs).
+
+    Returns:
+        boolean: True if model is quantized, else False.
     """
     assert isinstance(sym_model, tuple) and isinstance(sym_model[0], mx.symbol.Symbol)
 
@@ -260,6 +332,14 @@ def is_model_quantized(sym_model):
 
 
 def _dequantize_sym_name(sym_name, check_list=None):
+    """Revise sym name by the node prefix or postfix.
+
+    Args:
+        sym_name (str): sym name.
+
+    Returns:
+        tuple: (name, op_type).
+    """
     name = sym_name
     op_type = OpType.NORMAL
     if sym_name.endswith(QUANTIZE_NODE_POSTFIX):
@@ -342,7 +422,6 @@ def quantize_sym_model(sym_model, ctx, qconfig):
     Returns:
         tuple: Symbol model (symnet, args, auxs) and list of tensors for calibration.
     """
-
     assert isinstance(sym_model, tuple) and isinstance(sym_model[0], mx.symbol.Symbol)
 
     symnet, args, auxs = sym_model
@@ -361,10 +440,11 @@ def quantize_sym_model(sym_model, ctx, qconfig):
 
 
 def _tensor_to_node(tensor, nodes):
-    """Map tensor to one of the nodes. This function assumes, that node tensors 
-    (weights, outputs, etc) contain node name in their names.
+    """Map tensor to one of the nodes.
+
+    This function assumes, that node tensors (weights, outputs, etc) contain node name in their names.
     """
-    assert len(nodes) > 0, '`nodes` cannot be empty'
+    assert len(nodes) > 0, "`nodes` cannot be empty"
 
     PATTERNS = {'',
                 '_output[0-9]*$',
@@ -380,11 +460,13 @@ def _tensor_to_node(tensor, nodes):
 
 
 def _qtensor_to_tensor(qtensor, tensors):
-    """Map quantized tensor to its fp32 equivalent. New ops may require updating the patterns.
+    """Map quantized tensor to its fp32 equivalent.
+
+    New ops may require updating the patterns.
     Tensors of quantize nodes (which are not present in fp32 models) will be mapped to their input
     nodes.
     """
-    assert len(tensors) > 0, '`tensors` cannot be empty'
+    assert len(tensors) > 0, "`tensors` cannot be empty"
 
     PATTERNS = {'quantize': '',
                 '_quantize_output0': '',
@@ -447,7 +529,6 @@ def make_symbol_block(sym_model, ctx, input_desc):
     Returns:
         mx.gluon.SymbolBlock: SymbolBlock model.
     """
-
     assert isinstance(sym_model, tuple) and isinstance(sym_model[0], mx.symbol.Symbol)
 
     symnet, args, auxs = sym_model
@@ -466,6 +547,7 @@ def make_symbol_block(sym_model, ctx, input_desc):
 
 
 def _gluon_forward(net, ctx, dataloader, b_filter, pre_batch=None, post_batch=None):
+    """Gluon forward func."""
     batch_num = 0
     for run, batch in zip(b_filter, dataloader):
         if not run:
@@ -496,7 +578,6 @@ def make_module(sym_model, ctx, input_desc):
     Returns:
         mx.module.Module: Module model.
     """
-
     assert isinstance(sym_model, tuple) and isinstance(sym_model[0], mx.symbol.Symbol)
 
     symnet, args, auxs = sym_model
@@ -510,6 +591,7 @@ def make_module(sym_model, ctx, input_desc):
 
 
 def _module_forward(module, dataloader, b_filter, pre_batch=None, post_batch=None):
+    """Module forward func."""
     batch_num = 0
     for run, batch in zip(b_filter, dataloader):
         if not run:
@@ -579,8 +661,7 @@ def parse_tune_config(tune_cfg, quantizable_nodes):
 
 
 def distribute_calib_tensors(calib_tensors, calib_cfg, tensor_to_node):
-    """Distributes the tensors for calibration, depending on the
-       algorithm set in the configuration of their nodes.
+    """Distributes the tensors for calibration, depending on the algorithm set in the configuration of their nodes.
 
     Args:
         calib_tensors: tensors to distribute.
@@ -618,8 +699,7 @@ def distribute_calib_tensors(calib_tensors, calib_cfg, tensor_to_node):
 
 
 def calib_model(qsym_model, calib_data, calib_cfg):
-    """Calibrate the quantized symbol model using data gathered by
-       the collector.
+    """Calibrate the quantized symbol model using data gathered by the collector.
 
     Args:
         qsym_model (tuple): quantized symbol model (symnet, args, auxs).
@@ -629,7 +709,6 @@ def calib_model(qsym_model, calib_data, calib_cfg):
     Returns:
         tuple: quantized calibrated symbol model (symnet, args, auxs).
     """
-
     assert isinstance(qsym_model, tuple) and isinstance(qsym_model[0], mx.symbol.Symbol)
 
     qsymnet, qargs, auxs = qsym_model
@@ -643,6 +722,7 @@ def calib_model(qsym_model, calib_data, calib_cfg):
 
 
 def amp_convert(sym_model, input_desc, amp_cfg):
+    """Convert model to support amp."""
     assert check_mx_version('2.0.0'), 'AMP is supported since MXNet 2.0. This error is due to ' \
         'an error in the configuration file.'
     from mxnet import amp
@@ -652,53 +732,66 @@ def amp_convert(sym_model, input_desc, amp_cfg):
 
 
 class DataLoaderWrap:
+    """DataLoader Wrap."""
     def __init__(self, dataloader, input_desc):
+        """Initialize."""
         self.dataloader = dataloader
         self.input_desc = input_desc
         self._iter = None
 
     def __iter__(self):
+        """Iter."""
         self._iter = iter(self.dataloader)
         return self
 
     def __next__(self):
+        """Next."""
         return next(self._iter)
 
 
 class DataIterLoader:
+    """DataIterLoader."""
     def __init__(self, data_iter):
+        """Initialize."""
         self.data_iter = data_iter
 
     def __iter__(self):
+        """Iter."""
         self.data_iter.reset()
         return self
 
     def __next__(self):
+        """Next."""
         batch = self.data_iter.__next__()
         return batch.data + (batch.label if batch.label is not None else [])
 
 
 class CollectorBase:
+    """Collector Base class."""
     def collect_gluon(self, name, _, arr):
+        """Collect by gluon api."""
         raise NotImplementedError()
 
     def collect_module(self, name, arr):
+        """Collect by module name."""
         name = mx.base.py_str(name)
         handle = ctypes.cast(arr, mx.base.NDArrayHandle)
         arr = mx.nd.NDArray(handle, writable=False)
         self.collect_gluon(name, '', arr)
 
     def pre_batch(self, m, b):
+        """Function to call prior to batch inference."""
         pass
 
     def post_batch(self, m, b, o):
+        """Function to call after batch inference."""
         pass
 
 
 class CalibCollector(CollectorBase):
-    """ Collects the calibration thresholds depending on the algorithm set """
-
+    """Collect the calibration thresholds depending on the algorithm set."""
     def __init__(self, include_tensors_kl, include_tensors_minmax, num_bins=8001):
+        """Initialize."""
         self.min_max_dict = {}
         self.hist_dict = {}
         self.num_bins = num_bins
@@ -706,6 +799,7 @@ class CalibCollector(CollectorBase):
         self.include_tensors_kl = include_tensors_kl
 
     def collect_gluon(self, name, _, arr):
+        """Collect by gluon api."""
         if name in self.include_tensors_kl:
             alg = 'kl'
         elif name in self.include_tensors_minmax:
@@ -735,6 +829,7 @@ class CalibCollector(CollectorBase):
 
     @staticmethod
     def _combine_histogram(old_hist, arr, new_min, new_max, new_th):
+        """Combine histogram."""
         if check_mx_version('2.0.0'):
             return mx.contrib.quantization._LayerHistogramCollector.combine_histogram(
                 old_hist, arr, new_min, new_max, new_th)
@@ -743,6 +838,7 @@ class CalibCollector(CollectorBase):
                                                              new_max, new_th)
 
     def calc_kl_th_dict(self, quantized_dtype):
+        """Calculation kl thresholds."""
         if len(self.hist_dict) > 0:
             if check_mx_version('2.0.0'):
                 return mx.contrib.quantization._LayerHistogramCollector.get_optimal_thresholds(
@@ -754,18 +850,19 @@ class CalibCollector(CollectorBase):
 
 
 class TensorCollector(CollectorBase):
-    """ Tensors collector. Builds up qtensor_to_tensor mapping """
-
+    """Tensors collector. Builds up qtensor_to_tensor mapping."""
     def __init__(self, include_nodes, qtensor_to_tensor, tensor_to_node):
+        """Initialize."""
         self.tensors_dicts = []
-        self.include_nodes = dict(include_nodes)
+        self.include_nodes = include_nodes
         self.qtensor_to_tensor = qtensor_to_tensor
         self.tensor_to_node = tensor_to_node
 
-        rest = set(self.include_nodes.keys()) - set(self.tensor_to_node.values())
+        rest = set(self.include_nodes) - set(self.tensor_to_node.values())
         assert len(rest) == 0, 'Unexpected tensors set to be collected: {}'.format(rest)
 
     def collect_gluon(self, name, _, arr):
+        """Collect by gluon api."""
         is_quantized = False
         if name not in self.tensor_to_node:
             if name in self.qtensor_to_tensor:
@@ -779,24 +876,28 @@ class TensorCollector(CollectorBase):
 
         node = self.tensor_to_node[name]
         if node in self.include_nodes:
-            op = self.include_nodes[node]
-            key = (node, op)
-            self.tensors_dicts[-1].setdefault(key, {})[name] = (is_quantized, arr.copy())
+            self.tensors_dicts[-1].setdefault(node, {})[name] = (is_quantized, arr.copy())
 
     def pre_batch(self, m, b):
+        """Preprocess."""
         self.tensors_dicts.append({})
 
 
 class NameCollector(CollectorBase):
+    """Name collector."""
     def __init__(self):
+        """Initialize."""
         self.names = []
 
     def collect_gluon(self, name, _, arr):
+        """Collect by gluon api."""
         self.names.append(name)
 
 
 class CalibData:
+    """Calibration data class."""
     def __init__(self, cache_kl={}, cache_minmax={}, tensors_kl=[], tensors_minmax=[]):
+        """Initialize."""
         self.th_dict = {}
         self.th_dict.update({t: cache_kl[t] for t in tensors_kl})
         self.th_dict.update({t: cache_minmax[t] for t in tensors_minmax})
@@ -804,8 +905,10 @@ class CalibData:
     # `min_max_dict` is used as a thresholds dictionary when `calib_mode` == 'naive'
     @property
     def min_max_dict(self):
+        """Return mix-max dict."""
         return self.th_dict
 
     # for mxnet version >= 2.0.0
     def post_collect(self):
+        """Return mix-max dict for mxnet version >= 2.0.0."""
         return self.th_dict
