@@ -41,6 +41,7 @@ logger = logging.getLogger("neural_compressor")
 ONNX18_VERSION = Version("1.8.0")
 ORT112_VERSION = Version("1.12.0")
 
+
 class ONNXRTAugment:
     """augment input model to dump tensor or for calibration."""
 
@@ -67,7 +68,7 @@ class ONNXRTAugment:
         self.model_wrapper = model_wrapper
         self.model = model_wrapper.model
         ai_onnx_domain = [opset for opset in self.model.opset_import \
-            if not opset.domain or opset.domain == "ai.onnx"]
+                          if not opset.domain or opset.domain == "ai.onnx"]
         self.opset_version = ai_onnx_domain[0].version
         self.dataloader = dataloader
         self.dump_op_types = dump_op_types
@@ -98,13 +99,13 @@ class ONNXRTAugment:
         onnx_version = Version(onnx.__version__)
         if onnx_version < ONNX18_VERSION:
             logger.warning("Static quantization for NLP model is supported " \
-                           "at onnx 1.8.0 and newer.")  
+                           "at onnx 1.8.0 and newer.")
         if self.already_quantized and any([i.dims in [1, 2] for i in \
-            self.model_wrapper.initializer() if i.name.endswith('_scale')]):
+                                           self.model_wrapper.initializer() if i.name.endswith('_scale')]):
             if self.opset_version < 13 and self.ort_version >= ORT112_VERSION:
                 logger.warning("Please use onnxruntime < 1.12.0 or upgrade model opset " \
-                    "version to 13 or higher to inspect per-channel quantized weight") 
- 
+                               "version to 13 or higher to inspect per-channel quantized weight")
+
         model = copy.deepcopy(self.model)
         model_nodes_names = [node.name for node in model.graph.node]
 
@@ -113,7 +114,7 @@ class ONNXRTAugment:
         tensors_to_dump = set()
 
         for augment_node_type in self.augment_nodes:
-            if augment_node_type not in ['DequantizeLinear']: # pragma: no cover
+            if augment_node_type not in ['DequantizeLinear']:  # pragma: no cover
                 raise ValueError("Unexpected augment_node {} only DequantizeLinear is " \
                                  "supported".format(augment_node_type))
 
@@ -129,11 +130,11 @@ class ONNXRTAugment:
 
         initializers = {i.name: i.data_type for i in model.graph.initializer}
         node_outputs = []
-        for node in model.graph.node: # pylint: disable=no-member
+        for node in model.graph.node:  # pylint: disable=no-member
             node_outputs.extend(node.output)
             should_be_dump = ((node.op_type in self.dump_op_types) and
-                                   (node.name not in self.black_nodes)) or \
-                                   (node.name in self.white_nodes)
+                              (node.name not in self.black_nodes)) or \
+                             (node.name in self.white_nodes)
             if should_be_dump:
                 if not weight_only and not activation_only:
                     tensors_to_dump.update(node.input)
@@ -141,7 +142,7 @@ class ONNXRTAugment:
                 elif weight_only:
                     for input in node.input:
                         if self.already_quantized and \
-                            input.replace('_dequantized', '_quantized') in initializers:
+                                input.replace('_dequantized', '_quantized') in initializers:
                             tensors_to_dump.add(input)
                         elif not self.already_quantized and input in initializers:
                             tensors_to_dump.add(input)
@@ -151,16 +152,16 @@ class ONNXRTAugment:
         model_inputs = [i.name for i in model.graph.input]
         for tensor in tensors_to_dump:
             if tensor not in node_outputs and tensor not in initializers and \
-                tensor not in model_inputs:
+                    tensor not in model_inputs:
                 continue
             if self.augment_nodes:
                 for augment_node_type in self.augment_nodes:
                     if augment_node_type in ['DequantizeLinear']:
                         # insert DequantizeLinear node as output
                         if tensor.endswith('_scale') or tensor.endswith('_zero_point') or \
-                            tensor.endswith('_QuantizeLinear') or \
-                            tensor.endswith('_QuantizeInput_quantized'):
-                            continue 
+                                tensor.endswith('_QuantizeLinear') or \
+                                tensor.endswith('_QuantizeInput_quantized'):
+                            continue
 
                         if not self.dynamically_quantized:
                             tensor = tensor.replace('_QuantizeInput', '_quantized') if \
@@ -177,8 +178,8 @@ class ONNXRTAugment:
                             if output:
                                 added_nodes.extend(nodes)
                                 added_outputs.append(helper.make_tensor_value_info(
-                                                   output, # pylint: disable=no-member
-                                                   TensorProto.FLOAT, ())) # pylint: disable=no-member
+                                    output,  # pylint: disable=no-member
+                                    TensorProto.FLOAT, ()))  # pylint: disable=no-member
                         else:
                             # the tensor is in FP32 dtype
                             if tensor not in [t.name for t in model.graph.output]:
@@ -192,11 +193,11 @@ class ONNXRTAugment:
                     added_outputs.append(added_tensor)
 
         if self.augment_nodes:
-            model.graph.node.extend(added_nodes) # pylint: disable=no-member
-        model.graph.output.extend(added_outputs) # pylint: disable=no-member
+            model.graph.node.extend(added_nodes)  # pylint: disable=no-member
+        model.graph.output.extend(added_outputs)  # pylint: disable=no-member
 
         self.augmented_model = model
-        if self.model_wrapper.large_size: # pragma: no cover
+        if self.model_wrapper.is_large_model:  # pragma: no cover
             onnx.save_model(model,
                             self.model_wrapper.model_path + '_augment.onnx',
                             save_as_external_data=True,
@@ -208,14 +209,14 @@ class ONNXRTAugment:
         """Gather intermediate model outputs after running inference."""
         # conduct inference session and get intermediate outputs
         so = onnxruntime.SessionOptions()
-        if sys.version_info < (3,10) and find_spec('onnxruntime_extensions'): # pragma: no cover
+        if sys.version_info < (3, 10) and find_spec('onnxruntime_extensions'):  # pragma: no cover
             from onnxruntime_extensions import get_library_path
             so.register_custom_ops_library(get_library_path())
 
         session = onnxruntime.InferenceSession(
                     self.augmented_model.SerializeToString(),
                     so,
-                    provider=self.backend) if not self.model_wrapper.large_size else \
+                    provider=self.backend) if not self.model_wrapper.is_large_model else \
                   onnxruntime.InferenceSession(
                     self.model_wrapper.model_path  + '_augment.onnx',
                     so,
@@ -225,9 +226,9 @@ class ONNXRTAugment:
         len_inputs = len(session.get_inputs())
         inputs_names = [session.get_inputs()[i].name for i in range(len_inputs)]
         output_dicts = {}
-        
+
         node_output_names = [output.name if output.name not in self.dequantized_output \
-                             else self.dequantized_output[output.name] \
+                                 else self.dequantized_output[output.name] \
                              for output in session.get_outputs()]
 
         for idx, (inputs, labels) in enumerate(self.dataloader):
@@ -243,7 +244,7 @@ class ONNXRTAugment:
                     ort_inputs.update(inputs)
                 else:
                     for i in range(len_inputs):
-                        if not isinstance(inputs[i], np.ndarray): # pragma: no cover
+                        if not isinstance(inputs[i], np.ndarray):  # pragma: no cover
                             ort_inputs.update({inputs_names[i]: np.array(inputs[i])})
                         else:
                             ort_inputs.update({inputs_names[i]: inputs[i]})
@@ -251,28 +252,28 @@ class ONNXRTAugment:
                 if idx > max(self.iterations):
                     break
                 if idx in self.iterations:
-                    for output_idx, output in enumerate(session.run(None, ort_inputs)): 
+                    for output_idx, output in enumerate(session.run(None, ort_inputs)):
                         if calib_mode == 'naive' and output.size != 0:
                             output_dicts.setdefault(node_output_names[output_idx], \
-                                []).append([output.min(), output.max()])
+                                                    []).append([output.min(), output.max()])
                         elif calib_mode == None:
                             output_dicts.setdefault(node_output_names[output_idx], \
-                                []).append(output)
+                                                    []).append(output)
             else:
-                for output_idx, output in enumerate(session.run(None, ort_inputs)): 
+                for output_idx, output in enumerate(session.run(None, ort_inputs)):
                     if calib_mode == 'naive' and output.size != 0:
                         output_dicts.setdefault(node_output_names[output_idx], \
-                            []).append([output.min(), output.max()])
+                                                []).append([output.min(), output.max()])
                     elif calib_mode == None:
                         output_dicts.setdefault(node_output_names[output_idx], \
-                            []).append(output)
+                                                []).append(output)
 
         return list(output_dicts.keys()), output_dicts
 
     def _dequantize(self, tensor, scale_tensor, zo_tensor):
         """Helper function to dequantize tensor."""
         int_tensor = self.model_wrapper.get_initializer(tensor)
-        if int_tensor: # weight tensor
+        if int_tensor:  # weight tensor
             return self._dequantize_weight(tensor, scale_tensor, zo_tensor)
         else:
             return self._dequantize_activation(tensor, scale_tensor, zo_tensor)
@@ -292,25 +293,25 @@ class ONNXRTAugment:
                          .format(weight_tensor_name))
             if self.opset_version < 13 and self.ort_version >= ORT112_VERSION:
                 logger.warning("Skip dequantizing weight {}, please use onnxruntime < 1.12.0 " \
-                    "or upgrade model opset version to 13 or higher".format(weight_tensor_name))
+                               "or upgrade model opset version to 13 or higher".format(weight_tensor_name))
                 return [], None
             node = self.model_wrapper.input_name_to_nodes[weight_tensor_name][0]
             if 'Conv' in node.op_type or \
-                ('Gemm' in node.op_type and is_B_transposed(node)):
+                    ('Gemm' in node.op_type and is_B_transposed(node)):
                 added_nodes, added_output = self._add_dequantize_transpose_node(
-                                                                 weight_tensor_name,
-                                                                 scale_tensor, zo_tensor,
-                                                                 len(weight_tensor.dims))
+                    weight_tensor_name,
+                    scale_tensor, zo_tensor,
+                    len(weight_tensor.dims))
             else:
                 added_nodes, added_output = self._add_dequantize_node(
-                    weight_tensor_name, 
+                    weight_tensor_name,
                     scale_tensor,
                     zo_tensor,
                     axis=1 if self.opset_version > 12 else None)
         else:
-            added_nodes, added_output = self._add_dequantize_node(weight_tensor_name, 
-                                                                 scale_tensor,\
-                                                                 zo_tensor)
+            added_nodes, added_output = self._add_dequantize_node(weight_tensor_name,
+                                                                  scale_tensor, \
+                                                                  zo_tensor)
         self.dequantized_output[added_output] = weight_tensor_name
         return added_nodes, added_output
 
@@ -318,8 +319,8 @@ class ONNXRTAugment:
         """Helper function to generate dequantize node."""
         dequantize_node = make_dquant_node(tensor_name + '_DequantizeLinear',
                                            [tensor_name,
-                                           scale_tensor.name,
-                                           zo_tensor.name],
+                                            scale_tensor.name,
+                                            zo_tensor.name],
                                            [tensor_name + '_output'],
                                            axis)
         return [dequantize_node], tensor_name + '_output'
@@ -327,24 +328,24 @@ class ONNXRTAugment:
     def _add_dequantize_transpose_node(self, tensor_name, scale_tensor, zo_tensor, dim):
         """Insert Transpose-DequantizelLinear-Transpose pairs."""
         pre_transpose_node = onnx.helper.make_node(
-                                 'Transpose',
-                                 inputs=[tensor_name],
-                                 outputs=[tensor_name + '_transposed'],
-                                 perm=(1,0,2,3) if dim == 4 else (1,0),
-                                 name=tensor_name + '_pre_transpose')
+            'Transpose',
+            inputs=[tensor_name],
+            outputs=[tensor_name + '_transposed'],
+            perm=(1, 0, 2, 3) if dim == 4 else (1, 0),
+            name=tensor_name + '_pre_transpose')
         dequantize_node = make_dquant_node(
-                                 tensor_name + '_DequantizeLinear',
-                                 [tensor_name + '_transposed', 
-                                  scale_tensor.name, 
-                                  zo_tensor.name], 
-                                 [tensor_name + '_DequantizeLinear'], 
-                                 axis=1 if self.opset_version > 12 else None)
+            tensor_name + '_DequantizeLinear',
+            [tensor_name + '_transposed',
+             scale_tensor.name,
+             zo_tensor.name],
+            [tensor_name + '_DequantizeLinear'],
+            axis=1 if self.opset_version > 12 else None)
         post_transpose_node = onnx.helper.make_node(
-                                 'Transpose',
-                                 inputs=[tensor_name + '_DequantizeLinear'],
-                                 outputs=[tensor_name + '_output'],
-                                 perm=(1,0,2,3) if dim == 4 else (1,0),
-                                 name=tensor_name + '_post_transpose')
+            'Transpose',
+            inputs=[tensor_name + '_DequantizeLinear'],
+            outputs=[tensor_name + '_output'],
+            perm=(1, 0, 2, 3) if dim == 4 else (1, 0),
+            name=tensor_name + '_post_transpose')
         added_nodes = [pre_transpose_node, dequantize_node, post_transpose_node]
         return added_nodes, tensor_name + '_output'
 
@@ -380,7 +381,7 @@ class ONNXRTAugment:
         return self._map_calibration(node_output_names, output_dicts,
                                      calib_mode=calib_mode)
 
-    def dump_calibration(self, q_config, calib_mode='naive'):
+    def dump_calibration(self, q_config, calib_mode='naive', min_max=None):
         """Gather calibration params for quantization.
 
         Args:
@@ -388,10 +389,12 @@ class ONNXRTAugment:
             calib_mode (str, optional): type 'naive' gives (Min, Max) pairs
                                         for each intermediate model output across
                                         test data sets, where the first element is
-                                        a minimum of all values and the second element 
+                                        a minimum of all values and the second element
                                         is a maximum of all values. Defaults to 'naive'.
+            min_max (dict, optional): min/max values of tensors
         """
-        return self.calculate_quantization_params(q_config, self.dump_minmax(calib_mode))
+        return self.calculate_quantization_params(q_config, self.dump_minmax(calib_mode)) if min_max is None \
+            else self.calculate_quantization_params(q_config, min_max)
 
     def calculate_quantization_params(self, q_config, quantization_thresholds):
         """Given quantization thresholds, calculate the quantization params.
@@ -422,10 +425,11 @@ class ONNXRTAugment:
                     child = children[0]
             parent = None
             scheme = 'asym'
-            qType = 2 # uint8
+            qType = 2  # uint8
             if tensor_name in output_name_to_nodes:
                 parent = output_name_to_nodes[tensor_name]
-            if parent and parent.name in q_config and q_config[parent.name] not in ['fp32']:
+            if parent and parent.name in q_config and \
+                q_config[parent.name] not in ['fp32', 'fp16']:
                 scheme = q_config[parent.name]['activation']['scheme']
                 qType = q_config[parent.name]['activation']['dtype']
             elif self.backend in ['TensorrtExecutionProvider']:
@@ -433,7 +437,8 @@ class ONNXRTAugment:
                 qType = 3
             node_thresholds = quantization_thresholds[tensor_name]
             node_params = self.calculate_scale_zeropoint(parent, child, node_thresholds[0],
-                node_thresholds[1], scheme, qType, _get_qrange_for_qType(qType, self.reduce_range))
+                                                         node_thresholds[1], scheme, qType,
+                                                         _get_qrange_for_qType(qType, self.reduce_range))
             quantization_params[tensor_name] = node_params
 
         return quantization_params
@@ -441,7 +446,7 @@ class ONNXRTAugment:
     def dump_tensor(self, activation=True, weight=False):
         """Dump activation or weight or both from the model."""
         if "QuantizeLinear" in [node.op_type for node in self.model.graph.node] or \
-            "DynamicQuantizeLinear" in [node.op_type for node in self.model.graph.node]:
+                "DynamicQuantizeLinear" in [node.op_type for node in self.model.graph.node]:
             self.augment_nodes = ["DequantizeLinear"]
             self.already_quantized = True
             self.dynamically_quantized = \
@@ -461,7 +466,7 @@ class ONNXRTAugment:
         for tensor_name, tensors in output_dicts.items():
             if tensor_name.replace('_dequantized', '_quantized') in model_initializer_names:
                 nodes = [node for node in map_input[tensor_name] \
-                    if node.name.replace('_quant', '') in self.white_nodes]
+                         if node.name.replace('_quant', '') in self.white_nodes]
             elif tensor_name.replace('_quantized', '') in model_input_names:
                 continue
             else:
@@ -480,10 +485,10 @@ class ONNXRTAugment:
                 if tensor_name not in model_initializer_names:
                     for i in range(iters):
                         map_node_activation[i][node_name] = \
-                                               {tensor_name.replace('_quantized', ''): tensors[i]}
+                            {tensor_name.replace('_quantized', ''): tensors[i]}
                 else:
                     map_node_weight[node_name].update({tensor_name.replace('_quantized', ''): \
-                                                                                      tensors[0]})
+                                                           tensors[0]})
         dumped_tensors_map = {}
         if weight:
             dumped_tensors_map.update({"weight": map_node_weight})
@@ -502,16 +507,14 @@ class ONNXRTAugment:
             if next_node.op_type == 'Relu':
                 if rmin < 0:
                     rmin = 0
-            elif next_node.op_type == 'Clip' and len(next_node.input) ==3:
-                if rmin < numpy_helper.to_array(
-                        self.model_wrapper.get_initializer(next_node.input[1])):
-                    rmin = numpy_helper.to_array(
-                        self.model_wrapper.get_initializer(next_node.input[1]))
-                if rmax > numpy_helper.to_array(
-                        self.model_wrapper.get_initializer(next_node.input[2])):
-                    rmax = numpy_helper.to_array(
-                        self.model_wrapper.get_initializer(next_node.input[2]))
-    
+            elif next_node.op_type == 'Clip' and len(next_node.input) == 3:
+                clip_min = numpy_helper.to_array(self.model_wrapper.get_initializer(next_node.input[1]))
+                clip_max = numpy_helper.to_array(self.model_wrapper.get_initializer(next_node.input[2]))
+                if rmin < clip_min:
+                    rmin = clip_min.tolist() if not isinstance(clip_min.tolist(), list)  else clip_min.tolist()[0]
+                if rmax > clip_max:
+                    rmax = clip_max.tolist() if not isinstance(clip_max.tolist(), list)  else clip_max.tolist()[0]
+
         if last_node:
             if last_node.op_type in ['Conv', 'FusedConv']:
                 attrs = [attr for attr in last_node.attribute]
@@ -533,5 +536,119 @@ class ONNXRTAugment:
         else:
             zp_and_scale.append(np.int8(zp))
         zp_and_scale.append(np.float32(scale))
-    
+
         return zp_and_scale
+
+    def _check_is_group_conv(self, node, model):
+        """Check the op is group wised or not(depthwise conv is excluded,return false).
+
+        Args:
+            node: The op node
+            model: The onnx model
+
+        Returns:
+            Bool: group wised True, otherwise False, depthwise False
+        """
+        name_to_indices = {}
+        for index, i in enumerate(model.graph.initializer):
+            name_to_indices[i.name] = index
+
+        if node.op_type == "Conv":
+            group = 1
+            for attr in node.attribute:
+                if hasattr(attr, 'name'):
+                    if attr.name == "group":
+                        group = attr.i
+                        break
+            # currently only normal conv and depthwise conv are supported
+            if group > 1:  # group conv, need to check depthwise or not
+                weight_name = node.input[1]
+                weight_shape = numpy_helper.to_array(
+                    model.graph.initializer[name_to_indices[weight_name]]).shape
+                input_channel = weight_shape.shape[1]
+                if input_channel != 1:  # TODO need to double check
+                    return True
+        return False
+
+    def _get_input_tensor_of_ops(self, op_types=['MatMul', 'Linear', 'Conv']):
+        """Traverse the graph and get all the data tensors flowing into layers of {op_types}.
+
+        Group conv is excluded.
+        TODO the tensors could be set/filtered in configuration.
+
+        Args:
+            op_types: The op types whose input tensor will be dumped
+
+        Returns:
+            A set of tensor names 
+        """
+        tensors_to_dump = set()
+        model = self.model
+        initializers = {i.name: i for i in model.graph.initializer}
+
+        for node in model.graph.node:
+            if len(op_types) == 0 or node.op_type in op_types:
+                if node.op_type == "Conv" and self._check_is_group_conv(node, model):
+                    continue
+                # also need to check whether the layer has weight
+                if len(node.input) >= 2 and node.input[1] in initializers.keys():
+                    tensors_to_dump.add(node.input[0])
+        return tensors_to_dump
+
+    def _get_max_per_channel(self, datas: list, percentile):
+        """Get the max values per input channel.
+
+        Args:
+            datas: The tensors
+            percentile: percentile of calibration to remove outliers
+
+        Returns:
+            The max values per input channel
+        """
+        permute_datas = []
+        for data in datas:
+            if len(data.shape) == 3:  # TODO  mammul batchsize*seq*inchannel, conv:batchsize*inchannle*f*f
+                tensor = np.abs(np.reshape(data, (-1, data.shape[-1])))
+                permute_datas.append(tensor)
+            elif len(data.shape) == 4:
+                tensor = np.swapaxes(data, 1, -1)
+                tensor = np.abs(np.reshape(tensor, (-1, tensor.shape[-1])))
+                permute_datas.append(tensor)
+            elif len(data.shape) == 2:
+                permute_datas.append(np.abs(data))
+            else:
+                assert False, "not supported"
+        permute_datas = np.stack(permute_datas, axis=0)
+        permute_datas = permute_datas.reshape(-1, permute_datas.shape[-1])
+        max_per_channels = np.percentile(permute_datas, percentile, axis=0)
+        max_per_channels = max_per_channels.astype(np.single)
+        return max_per_channels
+
+    def calib_smooth(self, percentile, op_types):
+        """Smooth model calibration.
+
+        Mainly get the max info per channel of input tensors.
+
+        Args:
+            percentile:Percentile of calibration to remove outliers
+            op_types: The op types whose input tensor will be dumped
+
+        Returns:
+            max_vals_per_channel: max values per channel of input tensors
+            shape_infos: The shape information of input tensors
+        """
+        # add the input tensors of {op_types} to outputs of the model
+        tensors_to_dump = self._get_input_tensor_of_ops(op_types)
+        self.model_wrapper.add_tensors_to_outputs(tensors_to_dump)
+        self.augmented_model = self.model_wrapper.model
+        _, output_dicts = self.get_intermediate_outputs()
+
+        # remove the input tensors of {op_types} to outputs of the model
+        self.model_wrapper.remove_tensors_from_outputs(tensors_to_dump)
+        max_vals_per_channel = {}
+        shape_infos = {}
+        for key in tensors_to_dump:
+            max_val_per_channel = self._get_max_per_channel(output_dicts[key], percentile=percentile)
+            max_vals_per_channel[key] = max_val_per_channel
+            shape_infos[key] = output_dicts[key][0].shape
+        return max_vals_per_channel, shape_infos
