@@ -20,6 +20,9 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
+"""Quantizer for onnx models."""
+
+import os
 import onnx
 import logging
 import numpy as np
@@ -29,27 +32,44 @@ from onnx import TensorProto
 from onnx import shape_inference
 from onnxruntime import SessionOptions, InferenceSession, GraphOptimizationLevel
 
-from neural_compressor.adaptor.ox_utils.registry import CreateQDQQuantizer, \
-    CreateOpConverter, CreateCaster
 from neural_compressor.adaptor.ox_utils.util import QuantizedValue, QuantizedInitializer, \
     _get_qrange_for_qType, cast_tensor, make_quant_node, make_dquant_node
-from neural_compressor.adaptor.ox_utils.util import QuantizedValueType
+from neural_compressor.adaptor.ox_utils.util import QuantizedValueType, quantize_data_per_channel
 from neural_compressor.adaptor.ox_utils.util import find_by_name, dtype_to_name
 from neural_compressor.adaptor.ox_utils.util import __producer__, __version__
 from neural_compressor.adaptor.ox_utils.util import quantize_data, dtype_mapping, support_pair, ValueInfo
-from neural_compressor import options
-from neural_compressor.utils.utility import CpuInfo
 from neural_compressor.model.onnx_model import ONNXModel
+from neural_compressor.adaptor.ox_utils.operators import OPERATORS
 
-logger = logging.getLogger()
+logger = logging.getLogger("neural_compressor")
 
 class Quantizer:
+    """Quantizer class."""
+
     def __init__(self, model, q_config, mode, static, quantization_params,
-                 op_types_to_quantize, fallback_list=['fp32']):
-        model = onnx.shape_inference.infer_shapes(model)
-        self.model = ONNXModel(model)
+                 op_types_to_quantize, fallback_list=['fp32'], reduce_range=None,
+                 add_qdq_pair_to_weight=False, optypes_to_exclude_output_quant=[],
+                 dedicated_qdq_pair=False):
+        """Initialization.
+
+        Args:
+            model (ModelProto or ONNXModel): onnx model or onnx model wrapper by neural compressor
+            q_config (dict): op-wise quantization config.
+            mode (QuantizationMode): quantizaion mode
+            static (bool): static or not
+            quantization_params (dict): scale and zero point of tensors
+            op_types_to_quantize (list): optypes to quantize
+            fallback_list (list, optional): fallback data type. Defaults to ['fp32'].
+            reduce_range (bool, optional): use 7 bit or not. Defaults to None.
+            add_qdq_pair_to_weight (bool, optional): add QDQ pair to weight or not. Defaults to False.
+            optypes_to_exclude_output_quant (list, optional): optypes to exclude output quantization. Defaults to [].
+            dedicated_qdq_pair (bool, optional): dedicate QDQ pair or not. Defaults to False.
+        """
+        self.model = ONNXModel(model) if not isinstance(model, ONNXModel) else model
+        model = onnx.shape_inference.infer_shapes(self.model.model) if \
+            not self.model.is_large_model else self.model.model
         self.config = q_config
-        self.reduce_range = False if CpuInfo().vnni else True
+        self.reduce_range = reduce_range
         self.mode = mode # QuantizationMode.Value
         self.static = static  # use static quantization for inputs.
         self.fuse_dynamic_quant = False
@@ -80,14 +100,13 @@ class Quantizer:
         if not self.static:
             self.op_types_to_exclude_output_quantization = op_types_to_quantize
         else:
-            self.op_types_to_exclude_output_quantization = [] \
-                if not options.onnxrt.qdq_setting.OpTypesToExcludeOutputQuantizatioin \
-                else options.onnxrt.qdq_setting.OpTypesToExcludeOutputQuantizatioin
+            self.op_types_to_exclude_output_quantization = optypes_to_exclude_output_quant
 
-        self.add_qdq_pair_to_weight = options.onnxrt.qdq_setting.AddQDQPairToWeight
-        self.dedicated_qdq_pair = options.onnxrt.qdq_setting.DedicatedQDQPair
+        self.add_qdq_pair_to_weight = add_qdq_pair_to_weight
+        self.dedicated_qdq_pair = dedicated_qdq_pair
 
     def check_opset_version(self):
+        """Check opset version."""
         ai_onnx_domain = [
             opset for opset in self.model.model.opset_import if not opset.domain \
                                                         or opset.domain == "ai.onnx"]
@@ -112,6 +131,7 @@ class Quantizer:
         return opset_version
 
     def should_quantize(self, node):
+        """Check if node should be quantized."""
         if node.name in self.config and self.config[node.name] not in self.fallback_list:
             return True
         elif node.name.split('_quant')[0] in self.config and \
@@ -121,6 +141,7 @@ class Quantizer:
             return False
 
     def quantize_model(self):
+        """Quantize onnx model."""
         # step 1: insert q-dq, cast-cast pairs
         self.insert_qdq()
  
@@ -128,8 +149,7 @@ class Quantizer:
         self.remove_redundant_pairs()
  
         # step 3: convert q-node-dq to qlinear op if needed
-        if self.mode != 'qdq':
-            self.convert_qdq_to_operator_oriented()
+        self.convert_qdq_to_operator_oriented()
  
         self.merge_dedicated_qdq_pair() 
  
@@ -141,6 +161,7 @@ class Quantizer:
         return self.model.model
 
     def merge_dedicated_qdq_pair(self):
+        """Merge dedicated Q/DQ pairs."""
         self.remove_nodes = []
         self.replace_input = []
         self.new_nodes = []
@@ -199,20 +220,40 @@ class Quantizer:
                 for node, old_input_name, new_input_name in self.replace_input:
                     self.model.replace_node_input(node, old_input_name, new_input_name)
                 self.model.update()
+        
+        if self.mode == 'qdq':
+            for node in self.model.nodes():
+                if node.op_type in ['QuantizeLinear'] and len(self.model.get_parents(node)) > 0:
+                    if 'QuantizeLinear' in [sibling.op_type \
+                                            for sibling in self.model.get_siblings(node)]:
+                        continue
+                    for sibling in self.model.get_siblings(node):
+                        if not self.should_quantize(sibling) and sibling.op_type in OPERATORS:
+                            for inp_idx in range(len(sibling.input)):
+                                if sibling.input[inp_idx] == node.input[0]:
+                                    self.replace_input.append([sibling,
+                                        sibling.input[inp_idx],
+                                        self.model.get_children(node)[0].output[0]])
+            for node, old_input_name, new_input_name in self.replace_input:
+                self.model.replace_node_input(node, old_input_name, new_input_name)
+            self.model.update()
 
     def should_cast(self, node):
+        """Check if node should be casted."""
         if node.name in self.config and self.config[node.name] != 'fp32': # pragma: no cover
             return True
         else:
             return False
 
     def insert_qdq(self):
+        """Insert Q/DQ pairs."""
         for node in self.model.nodes():
             if self.should_quantize(node):
-                op_quantizer = CreateQDQQuantizer(self, node)
-                op_quantizer.quantize()
+                op_quantizer = OPERATORS[node.op_type](self, node)
+                if op_quantizer.quantize_check():
+                    op_quantizer.quantize()
             elif self.should_cast(node): # pragma: no cover
-                op_caster = CreateCaster(self, node)
+                op_caster = OPERATORS[node.op_type](self, node)
                 op_caster.cast()
         self.model.graph().node.extend(self.new_nodes)
         self.model.remove_nodes(self.remove_nodes)
@@ -222,21 +263,26 @@ class Quantizer:
         self.model.update()
  
     def should_convert(self, node):
+        """Check if node should be converted."""
         name = node.name.split('_quant')[0]
-        if name in self.config and self.config[name] not in self.fallback_list:
+        if name in self.config and self.config[name] not in self.fallback_list and \
+            (self.config[name]['activation']['quant_mode'] == 'dynamic' or self.mode != 'qdq'):
             return True
         else:
             return False
 
     def convert_qdq_to_operator_oriented(self):
+        """Convert QDQ to QOperator format."""
         self.new_nodes = []
         self.remove_nodes = []
         self.replace_input = []
         for node in self.model.nodes():
             if node.op_type not in ['QuantizeLinear', 'DequantizeLinear'] and \
                 self.should_convert(node):
-                op_converter = CreateOpConverter(self, node)
-                op_converter.convert()
+                op_converter = OPERATORS[node.op_type](self, node)
+                mode = self.config[node.name.split('_quant')[0]]['activation']['quant_mode']
+                if op_converter.convert_check(mode):
+                    op_converter.convert(mode)
         self.model.graph().node.extend(self.new_nodes)
         self.model.remove_nodes(self.remove_nodes)
         for node, old_input_name, new_input_name in self.replace_input:
@@ -244,6 +290,7 @@ class Quantizer:
         self.model.update()
 
     def remove_redundant_pairs(self):
+        """Remove redudant Q/DQ, Cast/Cast pairs."""
         self.remove_nodes = []
         self.replace_input = []
         pairs = [['QuantizeLinear', 'DequantizeLinear'], 
@@ -290,7 +337,7 @@ class Quantizer:
                         if len(outs) > 0:
                             output_dtype = str(self.new_value_info[outs[0]].new_dtype)
                             break
-                    if len(outs) == 0 or all([not self.should_convert(i) for i in children]):
+                    if len(outs) == 0 or all([not self.should_cast(i) for i in children]):
                         return
                     if input_dtype == str(match_nodes[1].attribute[0].i) and \
                         output_dtype == str(match_nodes[0].attribute[0].i) and \
@@ -324,17 +371,14 @@ class Quantizer:
         self.model.update()
 
     def dtype_cast(self, node, cfg, keep_io_types=True): # pragma: no cover
-        min_positive_val = 1e-7
-        max_finite_val = 1e4
+        """Cast node dtype."""
         for idx, tensor_name in enumerate(node.input):
             initializer = find_by_name(tensor_name, self.model.initializer())
             if initializer is not None:
                 if initializer.data_type != onnx_proto.TensorProto.FLOAT: 
                     continue
-                new_tensor = cast_tensor(initializer, cfg)
-                if new_tensor:
-                    self.model.remove_initializer(initializer)
-                    self.model.add_initializer(new_tensor)
+                do_cast = cast_tensor(initializer, cfg)
+                if do_cast:
                     self.new_value_info[tensor_name] = ValueInfo(tensor_name,
                                                              TensorProto.FLOAT, dtype_mapping[cfg])
             else:
@@ -364,7 +408,8 @@ class Quantizer:
                                                          dtype_mapping[cfg], TensorProto.FLOAT)
 
     def quantize_outputs(self, node, initializer_use_weight_qType=True, direct_int8=False):
-        if not self.static:
+        """Quantize node outputs."""
+        if self.config[node.name]['activation']['quant_mode'] == 'dynamic':
             return
         for idx, tensor_name in enumerate(node.output):
             if tensor_name in self.value_infos and \
@@ -392,8 +437,8 @@ class Quantizer:
             if data_found == False:
                 raise ValueError(
                     "Quantization parameters are not specified for param {}."
-                    "In static mode quantization params for inputs and outputs \
-                    of nodes to be quantized are required.".format(tensor_name))
+                    "In static mode quantization params for inputs and outputs "
+                    "of nodes to be quantized are required.".format(tensor_name))
 
             node.output[idx] = tensor_name + "_QuantizeInput"
             q_input = node.output[idx]
@@ -416,6 +461,7 @@ class Quantizer:
  
     def quantize_inputs(self, node, indices=None, 
             initializer_use_weight_qType=True, direct_int8=False):
+        """Quantize node inputs."""
         # Quantize the input
         for idx, tensor_name in enumerate(node.input):
             if indices and idx not in indices:
@@ -489,15 +535,15 @@ class Quantizer:
                     data_found, scale_name, zp_name, _, _ = \
                         self._get_quantization_params(tensor_name)
  
-                if self.static:
+                if self.config[node.name.split('_quant')[0]]['activation']['quant_mode'] != \
+                    'dynamic':
                     if data_found == False:
                         raise ValueError(
                             "Quantization parameters are not specified for param {}."
-                            "In static mode quantization params for inputs and outputs \
-                            of nodes to be quantized are required.".format(tensor_name))
+                            "In static mode quantization params for inputs and outputs "
+                            "of nodes to be quantized are required.".format(tensor_name))
                     if direct_int8:
-                        parent = self.model.get_parents(node)[0]
-                        if not parent.output[0].endswith('_QuantizeInput'):
+                        if node.input[0] not in self.quantized_value_map:
                             return
                     q_input = tensor_name
                     q_output = tensor_name + "_" + node.name + "_QuantizeLinear" if \
@@ -506,6 +552,8 @@ class Quantizer:
                     dq_output = tensor_name + "_" + node.name + "_dequantized" if \
                         tensor_name not in self.model.input() else tensor_name + "_dequantized"
                     self.replace_input.append([node, tensor_name, dq_output])
+                    if tensor_name in self.model.input() and tensor_name in self.quantized_value_map:
+                        continue
 
                     quant_node_name = tensor_name + "_" + node.name + "_QuantizeLinear"
                     dequant_node_name = tensor_name + "_" + node.name + "_DequantizeLinear"
@@ -525,41 +573,48 @@ class Quantizer:
                                                                 self.new_nodes,
                                                                 self.model.graph())
                     if qlinear_node is None:
-                        if data_found == True:
-                            qlinear_node = make_quant_node(tensor_name + "_QuantizeLinear",
-                                [tensor_name, scale_name, zp_name], [tensor_name + "_quantized"])
+                        if self.fuse_dynamic_quant and \
+                            self.config[node.name]['activation']['dtype'] == \
+                                onnx_proto.TensorProto.UINT8 and \
+                            self.config[node.name]['activation']['scheme'] == 'asym':
+                            scale_name = tensor_name + "_scale"
+                            zeropoint_name = tensor_name + "_zero_point"
+                            if find_by_name(scale_name, self.model.initializer()):
+                                self.model.remove_initializer(
+                                    find_by_name(scale_name, self.model.initializer()))
+                            if find_by_name(zeropoint_name, self.model.initializer()):
+                                self.model.remove_initializer(
+                                    find_by_name(zeropoint_name, self.model.initializer()))
+                            qlinear_node = onnx.helper.make_node("DynamicQuantizeLinear", 
+                                [tensor_name],
+                                [tensor_name + "_quantized", scale_name, zeropoint_name],
+                                tensor_name + "_QuantizeLinear")
                         else:
-                            if self.fuse_dynamic_quant and \
-                                self.config[node.name]['activation']['dtype'] == \
-                                    onnx_proto.TensorProto.UINT8 and \
-                                self.config[node.name]['activation']['scheme'] == 'asym':
-                                scale_name = tensor_name + "_scale"
-                                zeropoint_name = tensor_name + "_zero_point"
-                                qlinear_node = onnx.helper.make_node("DynamicQuantizeLinear", 
-                                    [tensor_name],
-                                    [tensor_name + "_quantized", scale_name, zeropoint_name],
-                                    tensor_name + "_QuantizeLinear")
-                            else:
-                                scale_name, zp_name, _, _ = \
-                                    self._get_dynamic_input_quantization_params(
-                                    tensor_name, self.config[node.name]['activation']['dtype'])
-                                qlinear_node = make_quant_node(tensor_name + "_QuantizeLinear",
-                                                            [tensor_name, scale_name, zp_name], 
-                                                            [tensor_name + "_quantized"])
+                            scale_name, zp_name, _, _ = \
+                                self._get_dynamic_input_quantization_params(
+                                tensor_name, self.config[node.name]['activation']['dtype'])
+                            qlinear_node = make_quant_node(tensor_name + "_QuantizeLinear",
+                                                        [tensor_name, scale_name, zp_name], 
+                                                        [tensor_name + "_quantized"])
                         if qlinear_node not in self.new_nodes:
                             self.new_nodes.append(qlinear_node)
                         self.quantized_value_map[tensor_name] = QuantizedValue(
                             tensor_name, 
-                            tensor_name + "_quantized", 
+                            qlinear_node.output[0],
                             scale_name, 
                             zp_name, 
                             self.config[node.name]['activation']['dtype'])                        
-                    self.replace_input.append([node, tensor_name, tensor_name + "_quantized"])
+                    self.replace_input.append([node, tensor_name, qlinear_node.output[0]])
  
     def quantize_bias_tensor(self, node):
+        """Quantize bias."""
         input_name, weight_name, bias_name = node.input
-        if self.quantization_params is None or input_name not in self.quantization_params and \
-            input_name not in self.quantized_value_map:
+        if self.quantization_params is None or \
+            input_name not in self.quantization_params or \
+            input_name not in self.quantized_value_map or \
+            (input_name in self.quantized_value_map and \
+            find_by_name(self.quantized_value_map[input_name].scale_name, 
+            self.model.initializer()) is None):
             self._dynamic_quantize_bias(input_name, weight_name + '_scale', bias_name,
                 bias_name + "_quantized")
         else:
@@ -584,9 +639,10 @@ class Quantizer:
                 bias_name, bias_name + '_dequantized'])
 
     def quantize_bias(self, bias_name, input_name, weight_name, beta=1.0):
-        '''
-        Quantized the bias. Zero Point == 0 and Scale == Input_Scale * Weight_Scale
-        '''
+        """Quantized the bias.
+        
+        Zero Point == 0 and Scale == Input_Scale * Weight_Scale
+        """
         # get scale for weight
         weight_scale_initializer = find_by_name(weight_name + '_scale', self.model.initializer())
         weight_scale = self.tensor_proto_to_array(weight_scale_initializer)
@@ -651,13 +707,14 @@ class Quantizer:
 
     def _dynamic_quantize_bias(self, input_name, weight_scale_name, \
         bias_name, quantized_bias_name):
-        '''
-        Adds series of nodes required to quantize the bias dynamically.
-            parameter input_name: Input name
-            parameter weight_scale_name: Weight scale.
-            parameter bias_scale_name: Bias to quantize.
-            parameter quantied_bias_name: Output name to use for quantized bias.
-        '''
+        """Adds series of nodes required to quantize the bias dynamically.
+
+        Args:
+            input_name (string): Input name
+            weight_scale_name (string): Weight scale name
+            bias_name (string): bias name
+            quantized_bias_name (string): quantized bias name
+        """
         qType = onnx_proto.TensorProto.INT32
         input_scale_name = input_name + "_scale"
         bias_scale_node = onnx.helper.make_node("Mul",
@@ -684,6 +741,7 @@ class Quantizer:
         return
 
     def quantize_weights_per_channel(self, node, indices, weight_qType, scheme, axis):
+        """Quantize weights per-channel."""
         if self.opset_version < 13 and self.mode == 'qdq':
             self.quantize_inputs(node, indices)
             return
@@ -718,39 +776,18 @@ class Quantizer:
                 self.replace_input.append([node, weight_name, dequant_node.output[0]])
 
     def quantize_weight_per_channel(self, weight_name, weight_qType, scheme, channel_axis):
+        """Quantize weight per-channel."""
         initializer = find_by_name(weight_name, self.model.initializer())
         if initializer is None:
             raise ValueError("{} is not an initializer", weight_name)
 
         if initializer.name not in self.quantized_value_map:
             weights = self.tensor_proto_to_array(initializer)
-            channel_count = weights.shape[channel_axis]
-            rmin_list = []
-            rmax_list = []
-            zero_point_list = []
-            scale_list = []
-            quantized_per_channel_data_list = []
-            for i in range(channel_count):
-                per_channel_data = weights.take(i, channel_axis)
-                rmin, rmax, zero_point, scale, quantized_per_channel_data = quantize_data(
-                    per_channel_data.flatten().tolist(), _get_qrange_for_qType(weight_qType, 
-                    self.reduce_range), weight_qType, scheme)
-                rmin_list.append(rmin)
-                rmax_list.append(rmax)
-                zero_point_list.append(zero_point)
-                scale_list.append(scale)
-                quantized_per_channel_data_list.append(quantized_per_channel_data)
+            rmin, rmax, zero_point, scale, quantized_weights = quantize_data_per_channel(
+                weights, channel_axis, _get_qrange_for_qType(weight_qType,self.reduce_range), weight_qType, scheme)
 
-            # combine per_channel_data into one
-            reshape_dims = list(weights.shape)  # deep copy
-            reshape_dims[channel_axis] = 1  # only one per channel for reshape
-            quantized_weights = np.asarray(quantized_per_channel_data_list[0]).reshape(reshape_dims)
-            for i in range(1, len(quantized_per_channel_data_list)):
-                channel_weights = np.asarray(quantized_per_channel_data_list[i]).reshape(reshape_dims)
-                quantized_weights = np.concatenate((quantized_weights, channel_weights), channel_axis)
-
-            weight = QuantizedInitializer(initializer.name, initializer, rmin_list, rmax_list, 
-                                          zero_point_list, scale_list,
+            weight = QuantizedInitializer(initializer.name, initializer, rmin, rmax,
+                                          zero_point, scale,
                                           weights,
                                           quantized_weights.flatten().tolist(), 
                                           channel_axis, weight_qType)
@@ -767,14 +804,15 @@ class Quantizer:
                 initializer.name + "_scale")
 
     def _update_weight(self, weight):
-        '''
-            Given a weight object, update the graph by doing the following:
-             - remove old initializer, update new initializers for 
-               quantized weight, zero point, and scale
-             - remove old weight input, update with new inputs for 
-               quantized weight, zero point, and scale
-            This function does NOT update the nodes in the graph, just initializers and inputs
-        '''
+        """Update weight.
+
+        Given a weight object, update the graph by doing the following:
+         - remove old initializer, update new initializers for 
+           quantized weight, zero point, and scale
+         - remove old weight input, update with new inputs for 
+           quantized weight, zero point, and scale
+        This function does NOT update the nodes in the graph, just initializers and inputs
+        """
         if weight.name in self.quantized_value_map:
             return
         packed_weight_name = weight.name + "_quantized"
@@ -804,6 +842,7 @@ class Quantizer:
 
     @staticmethod
     def tensor_proto_to_array(initializer):
+        """Convert TensorProto to array."""
         if initializer.data_type == onnx_proto.TensorProto.FLOAT:
             weights = onnx.numpy_helper.to_array(initializer)
         else:
@@ -813,12 +852,14 @@ class Quantizer:
         return weights
 
     def _get_quantization_params(self, param_name):
-        '''
-        Create initializers and inputs in the graph for zero point and scale of output.
+        """Create initializers and inputs in the graph for zero point and scale of output.
+
         Zero point and scale values are obtained from self.quantization_params if specified.
-            parameter param_name: Name of the quantization parameter.
-            return: result, scale_name, zero_point_name, scale_shape, zero_point_shape.
-        '''
+
+        Args:
+            param_name (string): Name of the quantization parameter.
+
+        """
         if self.quantization_params is None or param_name not in self.quantization_params:
             return False, "", "", "", ""
 
@@ -827,12 +868,12 @@ class Quantizer:
             raise ValueError("Quantization parameters should contain zero point and scale. "
                              "Specified values for output {}: {}".format(param_name, params))
 
-        zero_point_values = [params[0].item()]
+        zero_point_values = [params[0]]
         zero_point_shape = []
         zero_point_name = param_name + "_zero_point"
         zero_point_type = onnx.mapping.NP_TYPE_TO_TENSOR_TYPE[params[0].dtype]
 
-        scale_values = [params[1].item()]
+        scale_values = [params[1]]
         scale_shape = []
         scale_name = param_name + "_scale"
 
@@ -847,12 +888,7 @@ class Quantizer:
         return True, scale_name, zero_point_name, scale_shape, zero_point_shape
 
     def _get_quantized_weight(self, initializer, qType, scheme):
-        '''
-            :param initializer: TensorProto initializer
-            :param scheme: sym or asym quantization.
-            :param qType: type to quantize to
-            :return: Weight class with quantization information
-        '''
+        """Get quantized weight."""
         if initializer.name in self.quantized_value_map:
             return self.quantized_value_map[initializer.name]
         weights_data = self.tensor_proto_to_array(initializer)
@@ -869,13 +905,14 @@ class Quantizer:
         return weight
 
     def get_bias_add_nodes(self, node, weight_name, last_output, quantized_bias_name):
-        '''
-        Given a node, this function handles bias add by
-            adding a "reshape" node on bias and an "add" node
-            parameter node: current node (Conv)
-            parameter last_output: output of previous node (input to bias add)
-            return: the name of output
-        '''
+        """Given a node, this function handles bias add by adding a "reshape" node on bias and an "add" node.
+
+        Args:
+            node (NodeProto): current node (Conv)
+            weight_name (string): weight name
+            last_output (_type_): output of previous node (input to bias add)
+            quantized_bias_name (string): bias name
+        """
         # Add tensors for the shape to be reshaped to
         weight = find_by_name(weight_name, self.model.initializer())
         if weight is None:
@@ -906,6 +943,7 @@ class Quantizer:
         return add_node_output
 
     def is_valid_quantize_weight(self, weight_name):
+        """Check weight can be quantized."""
         weight = find_by_name(weight_name, self.model.initializer())
         if weight is not None:
             return weight.data_type == onnx_proto.TensorProto.FLOAT
@@ -913,6 +951,7 @@ class Quantizer:
             return weight_name in self.quantized_value_map
 
     def dequantize_tensor(self, node, value_name):
+        """Dequantize tensor."""
         if value_name in self.quantized_value_map:
             quantized_value = self.quantized_value_map[value_name]
             dqlinear_name = value_name + "_DequantizeLinear"
@@ -931,8 +970,8 @@ class Quantizer:
                if data_found == False:
                    raise ValueError(
                        "Quantization parameters are not specified for param {}."
-                       "In static mode quantization params for inputs and outputs \
-                       of nodes to be quantized are required.".format(value_name))
+                       "In static mode quantization params for inputs and outputs "
+                       "of nodes to be quantized are required.".format(value_name))
             dqlinear_name = value_name + "_DequantizeLinear"
             dqlinear_inputs = [value_name + '_quantized',
                                scale_name,
@@ -945,11 +984,11 @@ class Quantizer:
                 self.new_nodes.append(dequantize_node)
 
     def _get_dynamic_input_quantization_params(self, input_name, qType):
-        """
-        Create nodes for dynamic quantization of input
-            parameter input_name: Name of the input.
-            parameter qType: type to quantize to.
-            return: scale_name, zero_point_name, scale_shape, zero_point_shape.
+        """Create nodes for dynamic quantization of input.
+
+        Args:
+            input_name (string): Name of the input.
+            qType (int): type to quantize to.
         """
         if qType == onnx_proto.TensorProto.INT8:
             return self._get_dynamic_input_quantization_params_int8(input_name)
@@ -957,10 +996,10 @@ class Quantizer:
         return self._get_dynamic_input_quantization_params_uint8(input_name)
 
     def _get_dynamic_input_quantization_params_int8(self, input_name): # pragma: no cover
-        """
-        Create nodes for dynamic quantization of input to int8
-            parameter input_name: Name of the input.
-            return: scale_name, zero_point_name, scale_shape, zero_point_shape.
+        """Create nodes for dynamic quantization of input to int8.
+
+        Args:
+            input_name (string): Name of the input.
         """
         qType = onnx_proto.TensorProto.INT8
 
@@ -1039,10 +1078,10 @@ class Quantizer:
         return input_scale_name, self.fixed_zero_zp_name, [], []
 
     def _get_dynamic_input_quantization_params_uint8(self, input_name):
-        """
-        Create nodes for dynamic quantization of input to uint8
-            parameter input_name: Name of the input.
-            return: scale_name, zero_point_name, scale_shape, zero_point_shape.
+        """Create nodes for dynamic quantization of input to uint8.
+
+        Args:
+            input_name (string): Name of the input.
         """
         qType = onnx_proto.TensorProto.UINT8
         # Reduce min and Reduce max
