@@ -15,6 +15,7 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
+"""Graph Converter Class."""
 
 import copy
 import os
@@ -24,24 +25,21 @@ import tensorflow as tf
 
 from collections import OrderedDict, UserDict
 from tensorflow.core.framework import graph_pb2
-from tensorflow.python.framework import tensor_util
 from tensorflow.python.platform import gfile
 from neural_compressor.utils.utility import get_all_fp32_data
 from neural_compressor.utils.utility import get_tensor_histogram
 from neural_compressor.utils.utility import combine_histogram
-from neural_compressor.utils.utility import CaptureOutputToFile
-from neural_compressor.utils.utility import str2array
-from neural_compressor.utils.utility import Dequantize, DequantizeWeight
+from neural_compressor.utils.utility import CaptureOutputToFile, CpuInfo
 from neural_compressor.conf.dotdict import deep_get
-from neural_compressor.experimental.common import Model
+from neural_compressor.model import Model
 from .transform_graph.insert_logging import InsertLogging
 from .transform_graph.rerange_quantized_concat import RerangeQuantizedConcat
 from .transform_graph.bias_correction import BiasCorrection
-from .util import iterator_sess_run,version1_gt_version2,version1_eq_version2,version1_lt_version2
-from .util import version1_gte_version2,version1_lte_version2
+from .util import generate_feed_dict, iterator_sess_run,version1_gt_version2,version1_eq_version2
+from .util import version1_gte_version2,version1_lte_version2,version1_lt_version2
+from .util import TF_SPR_BASE_VERSIONS
 from .quantize_graph.quantize_graph_for_intel_cpu import QuantizeGraphForIntel
 from .quantize_graph_common import QuantizeGraphHelper
-from .quantize_graph.quantize_graph_conv import FuseNodeStartWithConv2d
 from .quantize_graph.qdq.optimize_qdq import OptimizeQDQGraph
 
 from .graph_util import GraphAnalyzer
@@ -49,6 +47,7 @@ from .graph_rewriter.generic.remove_training_nodes import RemoveTrainingNodesOpt
 from .graph_rewriter.generic.strip_unused_nodes import StripUnusedNodesOptimizer
 from .graph_rewriter.generic.fold_batch_norm import FoldBatchNormNodesOptimizer
 from .graph_rewriter.generic.fuse_pad_with_conv import FusePadWithConv2DOptimizer
+from .graph_rewriter.generic.strip_equivalent_nodes import StripEquivalentNodesOptimizer
 from .graph_rewriter.generic.fuse_pad_with_fp32_conv import FusePadWithFP32Conv2DOptimizer
 
 from .graph_rewriter.int8.freeze_value import FreezeValueTransformer
@@ -58,23 +57,28 @@ from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeTran
 from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeDequantizeTransformer
 from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeNewAPITransformer
 from .graph_rewriter.int8.fuse_matmul_requantize import FuseMatMulRequantizeDequantizeNewAPITransformer
+from .graph_rewriter.int8.fuse_conv_redundant_dequantize import FuseConvRedundantDequantizeTransformer
+from .graph_rewriter.int8.fuse_matmul_redundant_dequantize import FuseMatMulRedundantDequantizeTransformer
 from .graph_rewriter.int8.scale_propagation import ScaleProPagationTransformer
 from .graph_rewriter.bf16.bf16_convert import BF16Convert
 from .graph_rewriter.int8.post_quantized_op_cse import PostCseOptimizer
 from .graph_rewriter.int8.post_hostconst_converter import PostHostConstConverter
 from .graph_rewriter.int8.meta_op_optimizer import MetaInfoChangingMemOpOptimizer
 from .graph_rewriter.qdq.insert_qdq_pattern import GenerateGraphWithQDQPattern
+from .graph_rewriter.qdq.share_qdq_y_pattern import ShareQDQForItexYPatternOptimizer
+from .graph_rewriter.qdq.merge_duplicated_qdq import MergeDuplicatedQDQOptimizer
 from neural_compressor.adaptor.tf_utils.graph_rewriter.generic.insert_print_node import InsertPrintMinMaxNode
 from .graph_util import GraphRewriterHelper as Helper
 
 
-TF_SUPPORTED_MAX_VERSION = '2.10.0'
+TF_SUPPORTED_MAX_VERSION = '2.11.0'
 TF_SUPPORTED_MIN_VERSION = '1.14.0'
 
-logger = logging.getLogger()
+logger = logging.getLogger("neural_compressor")
 debug = bool(logger.level == logging.DEBUG)
 
 class GraphConverter:
+    """Graph Converter Class is used to generate the quantization graph."""
     def __init__(self,
                  model,
                  qt_config={},
@@ -87,11 +91,12 @@ class GraphConverter:
                  itex_mode=False,
                  qdq_enabled=False,
                  new_api=False,
-                 performance_only=False):
+                 performance_only=False,
+                 use_bf16=False):
         """Convert graph.
 
         :param model: input tensorflow model.
-        :param qt_config: quantization configs, including interation and op-wise quant config
+        :param qt_config: quantization configs, including iteration and op-wise quant config
         :param fp32_ops: fall back to fp32 dtype op list
         :param bf16_ops: fall back to bf16 dtype op list
         :param data_loader: for calibration phase used dataloader
@@ -113,12 +118,19 @@ class GraphConverter:
         self.fake_quant = fake_quant
         self.itex_mode = itex_mode
         self.qdq_enabled = qdq_enabled
+        self.performance_only = performance_only
         self.quantized_node_info = []
         self._calibration_data = []
         self._fp32_print_data = []
         self.data_loader = data_loader
         self._check_tf_version()
         self._check_args()
+
+        self._fp32_model = Model(self.model._model, **self.model.kwargs)
+        self._fp32_model.graph_def = self.model.graph_def
+        self._fp32_model.output_tensor_names = self.output_tensor_names
+        self._fp32_model.input_tensor_names = self.input_tensor_names
+
         self._gen_tmp_filenames()
         self._kl_op_dict = {}
         self._kl_keys = []
@@ -133,39 +145,46 @@ class GraphConverter:
         self.scale_info.update({'bf16_ops': self.bf16_ops})
         self.scale_info.update({'fp32_ops': self.fp32_ops})
 
-        self._fp32_model = Model(self.model._model, **self.model.kwargs)
-        self._fp32_model.graph_def = self.model.graph_def
-        self._fp32_model.output_tensor_names = self.output_tensor_names
-        self._fp32_model.input_tensor_names = self.input_tensor_names
-
         self._sampling_model = Model(self.model._model, **self.model.kwargs)
         self._sampling_model.output_tensor_names = self.output_tensor_names
         self._sampling_model.input_tensor_names = self.input_tensor_names
 
-        self._itex_model = Model(self.model._model, **self.model.kwargs)
-        self._itex_model.graph_def = self.model.graph_def
-        self._itex_model.output_tensor_names = self.output_tensor_names
-        self._itex_model.input_tensor_names = self.input_tensor_names
-        self._tmp_graph_def = copy.deepcopy(self.model.graph_def)
+        if self.performance_only:
+            # reuse the fp32 model for performance only mode
+            self._tmp_graph_def = self.model.graph_def
+        else:
+            self._tmp_graph_def = copy.deepcopy(self.model.graph_def)
         self.new_api = new_api #bool(version1_gte_version2(tf.version.VERSION, '2.8.0'))
-        self.performance_only = performance_only
+        self.use_bf16 = use_bf16
         self.exclude_node_names = []
+
     # pylint: disable=no-member
     def _inference(self, model):
-        """Run the calibration on the input graph
+        """Run the calibration on the input graph.
 
         Args:
             model(TensorflowBaseModel): input TensorflowBaseModel
         """
+        # ITEX optimization has broken INC calibration process.
+        # INC needs turn off ITEX optimization pass in calibration stage.
+        # TODO ITEX will provide API to replace setting environment variable.
+        os.environ["ITEX_REMAPPER"] = "0"
+        sess = model.sess
+        iter_op = model.iter_op
         input_tensor = model.input_tensor
         output_tensor = model.output_tensor
         # TF table initialization: https://github.com/tensorflow/tensorflow/issues/8665
-        node_names = [node.name for node in model.sess.graph.as_graph_def().node]
+        node_names = [node.name for node in sess.graph.as_graph_def().node]
         if 'init_all_tables' in node_names:
-            init_table_op = model.sess.graph.get_operation_by_name('init_all_tables')
-            model.sess.run(init_table_op)
+            init_table_op = sess.graph.get_operation_by_name('init_all_tables')
+            sess.run(init_table_op)
 
         logger.info("Start sampling on calibration dataset.")
+        if hasattr(self.data_loader, "__len__") and len(self.data_loader) == 0:
+            feed_dict = {}
+            _ = sess.run(output_tensor, feed_dict) if iter_op==[] \
+                else iterator_sess_run(sess, iter_op, \
+                    feed_dict, output_tensor, self.calib_iteration)
         for idx, (inputs, labels) in enumerate(self.data_loader):
             if len(input_tensor) == 1:
                 feed_dict = {}
@@ -190,7 +209,7 @@ class GraphConverter:
                         for tensor in input_tensor:
                             pos = tensor.name.rfind(":")
                             t_name = tensor.name if pos < 0 else tensor.name[:pos]
-                            if name == t_name:
+                            if name in [tensor.name, t_name]:
                                 feed_dict[tensor] = inputs[name]
                                 break
                 else:
@@ -210,27 +229,30 @@ class GraphConverter:
                         return True
 
                     disorder_tensors = []
-                    disorder_inputs = [] 
+                    disorder_inputs = []
                     for idx, sort_tensor in enumerate(input_tensor):
                         sort_input = inputs[idx] 
                         if check_shape(sort_tensor, sort_input):
-                            feed_dict.update({sort_tensor: sort_input}) 
+                            feed_dict.update({sort_tensor: sort_input})
                         else:
                             disorder_tensors.append(sort_tensor)
                             disorder_inputs.append(sort_input)
                     for i, dis_tensor in enumerate(disorder_tensors):
-                       for j, dis_input in enumerate(disorder_inputs):  
-                           if check_shape(dis_tensor, dis_input):
-                               feed_dict.update({dis_tensor: dis_input})    
-                               break
-            _ = model.sess.run(output_tensor, feed_dict) if model.iter_op==[] \
-                else iterator_sess_run(model.sess, model.iter_op, \
+                        for j, dis_input in enumerate(disorder_inputs):
+                            if check_shape(dis_tensor, dis_input):
+                                feed_dict.update({dis_tensor: dis_input})
+                                break
+            _ = sess.run(output_tensor, feed_dict) if iter_op==[] \
+                else iterator_sess_run(sess, iter_op, \
                     feed_dict, output_tensor, self.calib_iteration)
             if idx + 1 == self.calib_iteration:
                 break
+        os.environ["ITEX_REMAPPER"] = "1"
 
     def _check_tf_version(self):
+        """Check if the installed tensorflow version is supported."""
         is_supported_version = False
+        is_sprbase_version = False
         try:
             from tensorflow import python
             if (hasattr(python, "pywrap_tensorflow")
@@ -248,43 +270,47 @@ class GraphConverter:
 
             if version1_gte_version2(tf.version.VERSION, '2.9.0'):
                 is_supported_version = True
-                 
-            if version1_eq_version2(tf.version.VERSION, '1.15.0-up3'):
+
+            if tf.version.VERSION == '1.15.0-up3':
                 is_supported_version = True
-                
+
+            if tf.version.VERSION in TF_SPR_BASE_VERSIONS:
+                is_supported_version = True
+                is_sprbase_version = True
+
         except Exception as e:
             raise ValueError(e)
         finally:
-            if version1_gt_version2(tf.version.VERSION, TF_SUPPORTED_MAX_VERSION):
+            if version1_gt_version2(tf.version.VERSION, TF_SUPPORTED_MAX_VERSION) and not is_sprbase_version:
                 logger.warning(
-                    str('Please note the {} version of Intel® Optimizations for '
-                        'TensorFlow is not fully verified! '
-                        'Suggest to use the versions '
-                        'between {} and {} if meet problem.').format(tf.version.VERSION,
-                                                                     TF_SUPPORTED_MIN_VERSION,
-                                                                     TF_SUPPORTED_MAX_VERSION))
+                    str('Please note the {} version of TensorFlow is not fully verified! '
+                        'Suggest to use the versions between {} and {} if meet problem.')
+                        .format(tf.version.VERSION, TF_SUPPORTED_MIN_VERSION, TF_SUPPORTED_MAX_VERSION))
+
             if version1_eq_version2(tf.version.VERSION, '2.5.0') and os.getenv('TF_ENABLE_MKL_NATIVE_FORMAT') != '0':
                 logger.fatal("Please set environment variable TF_ENABLE_MKL_NATIVE_FORMAT=0 "
                              "when TensorFlow 2.5.0 installed.")
 
-            if version1_gte_version2(tf.version.VERSION, '2.6.0') and os.getenv('TF_ENABLE_ONEDNN_OPTS') != '1':
+            if version1_gte_version2(tf.version.VERSION, '2.6.0') and \
+               version1_lt_version2(tf.version.VERSION, '2.9.0') and \
+               os.getenv('TF_ENABLE_ONEDNN_OPTS') != '1':
                 logger.fatal("Please set environment variable TF_ENABLE_ONEDNN_OPTS=1 "
                              "when TensorFlow >= 2.6.0 and < 2.9.0 installed.")
 
             if not is_supported_version:
                 raise ValueError(
-                    str('Please install Intel® Optimizations for TensorFlow '
-                        'or MKL enabled TensorFlow from source code '
-                        'within version >={} and <={}.').format(TF_SUPPORTED_MIN_VERSION,
-                                                                TF_SUPPORTED_MAX_VERSION))
+                    str('Please install TensorFlow within version >={} and <={}.')
+                    .format(TF_SUPPORTED_MIN_VERSION, TF_SUPPORTED_MAX_VERSION))
 
     def _check_args(self):
+        """Check model's arguments."""
         if self.model.workspace_path and not os.path.isdir(self.model.workspace_path) \
                 and not os.path.exists(os.path.dirname(self.model.workspace_path)):
             raise ValueError('"output_graph" directory does not exist.')
         self._output_path = self.model.workspace_path
 
     def _gen_tmp_filenames(self):
+        """Generate the temporary file names."""
         self._int8_dynamic_range_model_path = os.path.join(self._output_path, \
                                                       'int8_dynamic_range_graph')
         self._int8_logged_model_path = os.path.join(self._output_path, 'int8_logged_graph')
@@ -295,14 +321,20 @@ class GraphConverter:
                                                         'int8_bf16_mixed_precision_graph')
 
         self.output_graph = os.path.join(self._output_path, 'int8_final_fused_graph')
-        # to keep temp model
-        self._tmp_model = Model(self.model._model, **self.model.kwargs)
-        self._tmp_model.graph_def = self.model.graph_def
-        self._tmp_model.output_tensor_names = self.output_tensor_names
-        self._tmp_model.input_tensor_names = self.input_tensor_names
+        if self.performance_only:
+            # reuse the fp32 model for performance only mode
+            self._tmp_model = self._fp32_model
+        else:
+            # to keep temp model
+            self._tmp_model = Model(self.model._model, **self.model.kwargs)
+            self._tmp_model.graph_def = self.model.graph_def
+            self._tmp_model.output_tensor_names = self.output_tensor_names
+            self._tmp_model.input_tensor_names = self.input_tensor_names
 
     def convert(self):
-        """Do convert, including:
+        """Do convertion.
+
+        Including:
             1) optimize fp32_frozen_graph,
             2) quantize graph,
             3) calibration,
@@ -319,17 +351,26 @@ class GraphConverter:
                 model = self.quantize()
 
         if self.itex_mode:
-            self._itex_model.graph_def = \
-                PostHostConstConverter(self._itex_model.graph_def).do_transformation()
-            self._itex_model.graph_def.library.CopyFrom(self.model.graph_def.library)
-            return self._itex_model
+            host_const_graph_def = \
+                PostHostConstConverter(self._tmp_model.graph_def).do_transformation()
+            host_const_graph_def.library.CopyFrom(self.model.graph_def.library)
+            self._tmp_model.graph_def = host_const_graph_def
+
+            return self._tmp_model
 
         if self.exclude_node_names:
             self.bf16_ops.extend(self.exclude_node_names)
-       
-        if (len(self.bf16_ops) > 0 and self.performance_only) or \
-           (os.getenv('MIX_PRECISION_TEST') == '1'):
+
+        if len(self.bf16_ops) > 0 and (self.use_bf16 or self.performance_only) and \
+           (CpuInfo().bf16 or os.getenv('FORCE_BF16') == '1'):
             model = self.bf16_convert()
+
+        if self.new_api:
+            if self.performance_only:
+                model.graph_def = FuseConvRedundantDequantizeTransformer(model.graph_def).do_transformation()
+            post_optimize_graph_def = FuseMatMulRedundantDequantizeTransformer(model.graph_def).do_transformation()
+            post_optimize_graph_def.library.CopyFrom(self.model.graph_def.library)
+            model.graph_def = post_optimize_graph_def
         post_cse_graph_def = PostCseOptimizer(model.graph_def).do_transformation()
         post_hostconst_graph_def = PostHostConstConverter(post_cse_graph_def).do_transformation()
         post_hostconst_graph_def.library.CopyFrom(self.model.graph_def.library)
@@ -342,6 +383,7 @@ class GraphConverter:
         return model
 
     def _get_fp32_print_node_names(self, specified_op_list):
+        """Get the print node name of the fp32 graph."""
         offset_map = {
             "QuantizedConv2DWithBiasSumAndRelu": 3,
             "QuantizedConv2DWithBiasAndRelu": 2,
@@ -404,13 +446,14 @@ class GraphConverter:
         return self._fp32_model
 
     def _search_y_pattern_for_itex(self):
-        """Search the Y pattern for itex and return the op name.
-        """
+        """Search the Y pattern for itex and return the op name."""
         g = GraphAnalyzer()
         g.graph = self._fp32_model.graph_def
         g.parse_graph()
-        y_pattern = [['Conv2D', 'MatMul'], ['BiasAdd'], ['Add'], ('Relu',)]
+        y_pattern = [['Conv2D', 'MatMul'], ['BiasAdd'], ['Add', 'AddV2', 'AddN'], ('Relu',)]
+        y_pattern_variant = [['MaxPool', 'AvgPool'], ['Add', 'AddV2', 'AddN'], ('Relu',)]
         target_nodes = g.query_fusion_pattern_nodes(y_pattern)
+        target_nodes_variant = g.query_fusion_pattern_nodes(y_pattern_variant)
 
         res = {}
         for i in target_nodes:
@@ -418,10 +461,18 @@ class GraphConverter:
                 res[i[2]] = 1
             else:
                 res[i[2]] += 1
-        return [(i,) for i in res if res[i] == 2]
+        matched_add_nodes = [(i,) for i in res if res[i] == 2]
+        for i in res:
+            if res[i] == 1:
+                for j in target_nodes_variant:
+                    if j[1] == i:
+                        matched_add_nodes.append((i,))
+        return matched_add_nodes
 
     def quantize(self):
-        """Quantize graph only (without optimizing fp32 graph), including:
+        """Quantize graph only (without optimizing fp32 graph).
+
+        Including:
             1) quantize graph,
             2) calibration,
             3) fuse RequantizeOp with fused quantized conv, and so on.
@@ -432,7 +483,7 @@ class GraphConverter:
             self._quantize_graph()
             self.quantized_node_info = [tuple(i) for i in self.quantized_node_info]
 
-            if self.fake_quant:
+            if self.fake_quant: # pragma: no cover
                 self._fuse_requantize_with_fused_quantized_node()
             else:
                 if self._enable_kl_op_names:
@@ -443,19 +494,20 @@ class GraphConverter:
 
                 output_tensor_names = copy.deepcopy(self.model.output_tensor_names)
                 sampling_graph_def = copy.deepcopy(self._fp32_model.graph_def)
+
                 # TODO: this is a workaround to make Min/Max node be completly eliminated in int8 graph 
                 # after enabling pad+conv2d in new API.
-                if self.new_api:
-                    non_pad_ops = list(list(set(self.fp32_ops).union(set(self.bf16_ops))))
-                    sampling_graph_def = FusePadWithFP32Conv2DOptimizer(
-                        sampling_graph_def,
-                        non_pad_ops,
-                        self._tmp_model.input_node_names,
-                        self.op_wise_config).do_transformation()
+                non_pad_ops = list(list(set(self.fp32_ops).union(set(self.bf16_ops))))
+                sampling_graph_def = FusePadWithFP32Conv2DOptimizer(
+                    sampling_graph_def,
+                    non_pad_ops,
+                    self._tmp_model.input_node_names,
+                    self.op_wise_config,
+                    self.new_api).do_transformation()
 
                 for i in self.quantized_node_info:
                     sampling_graph_def, output_names = InsertPrintMinMaxNode(
-                        sampling_graph_def, i[0], i[-1]).do_transformation()
+                        sampling_graph_def, i[0], i[-1], self.new_api).do_transformation()
                     output_tensor_names.extend(output_names)
                 if self.quantized_node_info:
                     sampling_graph_def.library.CopyFrom(self.model.graph_def.library)
@@ -466,6 +518,12 @@ class GraphConverter:
                         self._inference(self._sampling_model)
                     self._calibration_data = Helper.gen_valid_sampling_log(tmp_dump_file)
 
+                del output_tensor_names
+                del sampling_graph_def
+                del self._sampling_model
+                import gc
+                gc.collect()
+
                 if len(self._calibration_data) > 0:
                     self._freeze_requantization_ranges(self._kl_op_dict)
                     self._fuse_requantize_with_fused_quantized_node()
@@ -486,10 +544,9 @@ class GraphConverter:
         return self._tmp_model
 
     def bf16_convert(self):
-        """Convert fp32 nodes in bf16_node to bf16 dtype based on
-           FP32 + INT8 mixed precision graph.
-        """
+        """Convert fp32 nodes in bf16_node to bf16 dtype based on FP32 + INT8 mixed precision graph."""
         try:
+            logger.info("Start BF16 conversion.")
             self._tmp_model.graph_def = BF16Convert(
                 self._tmp_model.graph_def,
                 self.fp32_ops,
@@ -507,8 +564,7 @@ class GraphConverter:
             return self._tmp_model
 
     def _quantize_graph(self):
-        """quantize graph."""
-
+        """Quantize graph."""
         non_pad_ops = list(list(set(self.fp32_ops).union(set(self.bf16_ops))))
 
         self._tmp_graph_def = FusePadWithConv2DOptimizer(
@@ -532,15 +588,16 @@ class GraphConverter:
             self.device,
             self.fake_quant,
             self.new_api,
-            self.performance_only).do_transform()
+            self.performance_only,
+            self.itex_mode).do_transform()
         self.exclude_node_names = exclude_node_names
         self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
-        if debug:
+        if debug and not self.performance_only:
             self._tmp_model.graph_def = self._tmp_graph_def
             self._tmp_model.save(self._int8_dynamic_range_model_path)
 
     def _generate_calibration_data(self, tmp_path, output_data, enable_kl_algo=False):
-
+        """Generate the calibration data."""
         tmp_dump_file = os.path.join(os.path.dirname(self.output_graph), 'requant_min_max.log')
 
         logger.debug("Generate calibration data and save to {}.".format(tmp_dump_file))
@@ -566,6 +623,7 @@ class GraphConverter:
                     self._kl_op_dict[key] = combine_histogram(self._kl_op_dict[key], fp32_data)
 
     def _freeze_requantization_ranges(self, additional_data=None):
+        """Freeze requantization ranges after doing quantization."""
         self._tmp_graph_def, quantizev2_max = FreezeValueTransformer(
             self._tmp_graph_def,
             self._calibration_data,
@@ -597,7 +655,8 @@ class GraphConverter:
             self._tmp_model.save(self._int8_frozen_range_model_path)
 
     def _fuse_requantize_with_fused_quantized_node(self):
-        if self.fake_quant:
+        """Fuse the Requantize/Dequantize with fused quantized Ops."""
+        if self.fake_quant: # pragma: no cover
             self._tmp_graph_def = FreezeFakeQuantOpOptimizer(
                 self._tmp_graph_def).do_transformation()
 
@@ -640,6 +699,9 @@ class GraphConverter:
         self._tmp_graph_def = MetaInfoChangingMemOpOptimizer(
             self._tmp_graph_def).do_transformation()
 
+        self._tmp_graph_def = StripEquivalentNodesOptimizer(
+            self._tmp_graph_def, self._tmp_model.output_node_names).do_transformation()
+
         if self.advance_config is not None and \
            deep_get(self.advance_config, 'bias_correction') is not None:
             self._tmp_graph_def = BiasCorrection(
@@ -663,9 +725,10 @@ class GraphConverter:
             os.remove(self._int8_logged_model_path + '.pb')
 
     def quantize_with_qdq_pattern(self):
-        """ Quantize model by inserting QDQ
-            step 1: insert QDQ pairs and update node info
-            step 2: convert Q-DQ-node-Q-DQ to Q-newAPI node-DQ
+        """Quantize model by inserting QDQ.
+
+        step 1: insert QDQ pairs and update node info
+        step 2: convert Q-DQ-node-Q-DQ to Q-newAPI node-DQ
         """
         try:
             self._insert_qdq_pairs()
@@ -687,6 +750,7 @@ class GraphConverter:
         return self._tmp_model
 
     def _insert_qdq_pairs(self):
+        """Insert QDQ pairs before Conv/MatMul/Pooling Ops."""
         # Fuse Pad into Conv2D, Conv3D, DepthwiseConv2dNative 
         non_pad_ops = list(list(set(self.fp32_ops).union(set(self.bf16_ops))))
         self._tmp_graph_def = FusePadWithConv2DOptimizer(
@@ -714,7 +778,8 @@ class GraphConverter:
                                                self.device,
                                                self.fake_quant,
                                                self.new_api,
-                                               self.performance_only).get_quantized_nodes()
+                                               self.performance_only,
+                                               self.itex_mode).get_quantized_nodes()
 
         if self.itex_mode:
             self.quantized_node_info.extend(self._search_y_pattern_for_itex())
@@ -735,13 +800,16 @@ class GraphConverter:
                                     sampling_graph_def,
                                     non_pad_ops,
                                     self._tmp_model.input_node_names,
-                                    self.op_wise_config).do_transformation()
+                                    self.op_wise_config,
+                                    self.new_api,
+                                    True).do_transformation()
 
         for i in self.quantized_node_info:
             sampling_graph_def, output_names = InsertPrintMinMaxNode(
-                sampling_graph_def, i[0], i[-1]).do_transformation()
+                sampling_graph_def, i[0], i[-1], self.new_api).do_transformation()
             output_tensor_names.extend(output_names)
 
+
         if self.quantized_node_info:
             sampling_graph_def.library.CopyFrom(self.model.graph_def.library)
             self._sampling_model.graph_def = sampling_graph_def
@@ -751,32 +819,38 @@ class GraphConverter:
                 self._inference(self._sampling_model)
             self._calibration_data = Helper.gen_valid_sampling_log(tmp_dump_file)
 
-        self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
-        self._tmp_model.graph_def = self._tmp_graph_def
+        del sampling_graph_def
+        del output_tensor_names
+        del self._sampling_model
+        import gc
+        gc.collect()
 
         # Insert QDQ pattern
         self._tmp_graph_def = GenerateGraphWithQDQPattern(
-              self._tmp_model, self._calibration_data,
-              self.op_wise_config, self.fake_quant, self.fp32_ops, self.bf16_ops, \
-              self.quantized_node_info, self.device, self.performance_only).do_transformation()
+              self._tmp_graph_def, self._calibration_data, self.op_wise_config,
+              self.fake_quant, self.fp32_ops, self.bf16_ops, self.quantized_node_info, 
+              self.device, self.performance_only, self.itex_mode).do_transformation()
 
     def _convert_qdq(self):
+        """Convert Dequantize + Op + QuantizeV2 into QuantizedOps."""
         if self.itex_mode:
             self._tmp_graph_def, quantizev2_max = FreezeValueTransformer(
                 self._tmp_graph_def,
                 self._calibration_data,
-                '__max:').do_transformation()
+                '__max:',
+                self.itex_mode).do_transformation()
             self._tmp_graph_def, quantizev2_min = FreezeValueTransformer(
                 self._tmp_graph_def,
                 self._calibration_data,
-                '__min:').do_transformation()
+                '__min:',
+                self.itex_mode).do_transformation()
             self._tmp_graph_def, requant_min_max= FreezeValueTransformer(
                 self._tmp_graph_def,
                 self._calibration_data,
                 '__requant_min_max',
                 tensor_data= self._kl_op_dict,
                 device=self.device,
-                ).do_transformation()
+                itex_mode=self.itex_mode).do_transformation()
 
             self.scale_info.update(quantizev2_max)
             self.scale_info.update(quantizev2_min)
@@ -787,9 +861,12 @@ class GraphConverter:
                 self._tmp_model.input_node_names,
                 self._tmp_model.output_node_names).do_transformation()
 
+            self._tmp_graph_def = ShareQDQForItexYPatternOptimizer(self._tmp_graph_def).do_transformation()
+            self._tmp_graph_def = MergeDuplicatedQDQOptimizer(self._tmp_graph_def).do_transformation()
+
             self._tmp_graph_def.library.CopyFrom(self.model.graph_def.library)
-            self._itex_model.graph_def = self._tmp_graph_def
-            self._itex_model.graph_def.library.CopyFrom(self.model.graph_def.library)
+            self._tmp_model.graph_def = self._tmp_graph_def
+            self._tmp_model.graph_def.library.CopyFrom(self.model.graph_def.library)
         else:
             self._tmp_graph_def, exclude_node_names = OptimizeQDQGraph(self._tmp_graph_def,
                                                    self._tmp_model.input_node_names,
@@ -799,9 +876,10 @@ class GraphConverter:
                                                    self.device,
                                                    self.fake_quant,
                                                    self.new_api,
-                                                   self.performance_only).do_transform()
+                                                   self.performance_only,
+                                                   self.itex_mode).do_transform()
             self.exclude_node_names=exclude_node_names
+
             if len(self._calibration_data) > 0:
                 self._freeze_requantization_ranges(self._kl_op_dict)
                 self._fuse_requantize_with_fused_quantized_node()
-
