@@ -14,7 +14,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+"""Switch Graph Rewriter."""
 
 
 from ..graph_base import GraphRewriterBase
@@ -24,12 +24,11 @@ from tensorflow.python.framework import tensor_util
 
 
 class SwitchOptimizer(GraphRewriterBase):
-    """ Remove switch op if the input condition is true.
-    """
+    """Remove switch op if the input condition is true."""
 
     @dump_elapsed_time("Pass SwitchOptimizer")
     def do_transformation(self):
-        """ replace all enter ops whose output is matmul with const
+        """Replace all enter ops whose output is matmul with const.
 
         Args:
           input_graph_def (graphdef): graphdef object
@@ -37,8 +36,6 @@ class SwitchOptimizer(GraphRewriterBase):
         Returns:
            [graphdef]: optimized graph
         """
-
-
         cur_graph = GraphAnalyzer()
         cur_graph.graph = self.model
 
@@ -48,8 +45,10 @@ class SwitchOptimizer(GraphRewriterBase):
         for node_combination in target_nodes:
             switch_node = graph_info[node_combination[0]].node
             pred_node =  graph_info[switch_node.input[1]].node
-            if pred_node.op == 'PlaceholderWithDefault' and tensor_util.MakeNdarray(
-                            graph_info[pred_node.input[0]].node.attr['value'].tensor):
+            if (pred_node.op == 'Const' and tensor_util.MakeNdarray( \
+               graph_info[pred_node.name].node.attr['value'].tensor)) or \
+               (pred_node.op == 'PlaceholderWithDefault' and tensor_util.MakeNdarray(
+                graph_info[pred_node.input[0]].node.attr['value'].tensor)):
                 condition = []
                 for output in graph_info[node_combination[0]].outputs:
                     successor_node = graph_info[output].node
