@@ -20,14 +20,13 @@ from schema import Schema, And, Use, Optional, Or, Hook
 from ..adaptor import FRAMEWORKS
 from ..strategy import STRATEGIES
 from ..objective import OBJECTIVES
-from ..pruners import PRUNERS
 from ..utils import logger
 from ..version import __version__
 import re
 import copy
 import itertools
 from collections import OrderedDict
-from .dotdict import DotDict
+from .dotdict import DotDict, deep_set
 import os, datetime
 
 def constructor_register(cls):
@@ -54,7 +53,8 @@ class Pruner():
                  method='per_tensor',
                  prune_type='basic_magnitude',##for pytorch pruning, these values should be None
                  start_step=None, end_step=None, update_frequency_on_step=None, prune_domain=None,
-                 sparsity_decay_type=None, pattern="tile_pattern_1x1", names=None, exclude_names=None, parameters=None):
+                 sparsity_decay_type=None, pattern="tile_pattern_1x1", names=None,
+                 extra_excluded_names=None, parameters=None):
         self.start_epoch = start_epoch
         self.end_epoch = end_epoch
         self.update_frequency = update_frequency
@@ -66,7 +66,7 @@ class Pruner():
         self.update_frequency_on_step = update_frequency_on_step
         self.prune_domain = prune_domain
         self.sparsity_decay_type = sparsity_decay_type
-        self.exclude_names = exclude_names
+        self.extra_excluded_names = extra_excluded_names
         self.pattern = pattern
         ## move this to experimental/pruning to support dynamic pruning
         # assert prune_type.replace('_', '') in [i.lower() for i in PRUNERS], \
@@ -76,6 +76,41 @@ class Pruner():
         self.names= names
         self.parameters = parameters
 
+
+@constructor_register
+class PrunerV2:
+    """
+    similiar to torch optimizer's interface
+    """
+
+    def __init__(self,
+                 target_sparsity=None, pruning_type=None, pattern=None, op_names=None,
+                 excluded_op_names=None,
+                 start_step=None, end_step=None, pruning_scope=None, pruning_frequency=None,
+                 min_sparsity_ratio_per_op=None, max_sparsity_ratio_per_op=None,
+                 sparsity_decay_type=None, pruning_op_types=None, reg_type=None,
+                 criterion_reduce_type=None, parameters=None, resume_from_pruned_checkpoint=None):
+        self.pruner_config = DotDict({
+            'target_sparsity': target_sparsity,
+            'pruning_type': pruning_type,
+            'pattern': pattern,
+            'op_names': op_names,
+            'excluded_op_names': excluded_op_names,  ##global only
+            'start_step': start_step,
+            'end_step': end_step,
+            'pruning_scope': pruning_scope,
+            'pruning_frequency': pruning_frequency,
+            'min_sparsity_ratio_per_op': min_sparsity_ratio_per_op,
+            'max_sparsity_ratio_per_op': max_sparsity_ratio_per_op,
+            'sparsity_decay_type': sparsity_decay_type,
+            'pruning_op_types': pruning_op_types,
+            'reg_type': reg_type,
+            'criterion_reduce_type': criterion_reduce_type,
+            'parameters': parameters,
+            'resume_from_pruned_checkpoint': resume_from_pruned_checkpoint
+        })
+
+
 # Schema library has different loading sequence priorities for different
 # value types.
 # To make sure the fields under dataloader.transform field of yaml file
@@ -508,7 +543,7 @@ dataset_schema = Schema({
             And(str, Use(input_int_to_float))),
         Optional('dtype'): And(Or(str, list), Use(input_to_list)),
     },
- 
+
     Optional('dummy'): {
         'shape': And(Or(str, list), Use(list_to_tuple)),
         Optional('low'): Or(
@@ -622,7 +657,7 @@ optimizer_schema = Schema({
         Optional('beta_2', default=0.999): Use(float),
         Optional('epsilon', default=1e-07): Use(float),
         Optional('amsgrad', default=False): bool
-    }, 
+    },
 })
 
 criterion_schema = Schema({
@@ -645,18 +680,28 @@ criterion_schema = Schema({
     },
     Optional('IntermediateLayersKnowledgeDistillationLoss'): {
         'layer_mappings':
-            And(Or(tuple, list), lambda s: all(len(i) in [2, 4] for i in s)),
+            And(Or(tuple, list), lambda s: all(len(i) in [1, 2] for i in s)),
         Optional('loss_types'):
             And(Or(tuple, list), lambda s: all(i in ['MSE', 'KL', 'L1'] for i in s)),
         Optional('loss_weights'):
             And(Or(tuple, list), lambda s: all(i >= 0 for i in s)),
         Optional('add_origin_loss'): bool,
+    },
+    Optional('SelfKnowledgeDistillationLoss'): {
+        'layer_mappings':
+            And(Or(tuple, list), lambda s: all(len(i) >= 1 for i in s)),
+        Optional('loss_types'):
+            And(Or(tuple, list), lambda s: all(i in ['L2', 'CE', 'KL'] for i in s)),
+        Optional('loss_weights'):
+            And(Or(tuple, list), lambda s: all(i >= 0.0 and i < 1.0 for i in s)),
+        Optional('add_origin_loss'): bool,
+        Optional('temperature'): And(float, lambda s: s > 0),
     }
 })
 
 train_schema = Schema({
-    'optimizer': optimizer_schema,
     'criterion': criterion_schema,
+    Optional('optimizer', default={'SGD': {'learning_rate': 0.001}}): optimizer_schema,
     Optional('dataloader'): dataloader_schema,
     Optional('epoch', default=1): int,
     Optional('start_epoch', default=0): int,
@@ -682,10 +727,10 @@ weight_compression_schema = Schema({
     Optional('end_step', default=0): int,
     Optional('update_frequency', default=1.0): float,
     Optional('update_frequency_on_step', default=1):int,
-    Optional('not_to_prune_names', default=[]):list,
+    Optional('excluded_names', default=[]):list,
     Optional('prune_domain', default="global"): str,
     Optional('names', default=[]): list,
-    Optional('exclude_names', default=None): list,
+    Optional('extra_excluded_names', default=None): list,
     Optional('prune_layer_type', default=None): list,
     Optional('sparsity_decay_type', default="exp"): str,
     Optional('pattern', default="tile_pattern_1x1"): str,
@@ -694,12 +739,36 @@ weight_compression_schema = Schema({
                                lambda s: all(isinstance(i, Pruner) for i in s))
 })
 
+weight_compression_v2_schema = Schema({
+    Optional('target_sparsity', default=0.9): float,
+    Optional('pruning_type', default="snip_momentum"): str,
+    Optional('pattern', default="4x1"): str,
+    Optional('op_names', default=[]): list,
+    Optional('excluded_op_names', default=[]): list,
+    Optional('start_step', default=0): int,
+    Optional('end_step', default=0): int,
+    Optional('pruning_scope', default="global"): str,
+    Optional('pruning_frequency', default=1): int,
+    Optional('min_sparsity_ratio_per_op', default=0.0): float,
+    Optional('max_sparsity_ratio_per_op', default=0.98): float,
+    Optional('sparsity_decay_type', default="exp"): str,
+    Optional('pruning_op_types', default=['Conv', 'Linear']): list,
+    Optional('reg_type', default=None): str,
+    Optional('criterion_reduce_type', default="mean"): str,
+    Optional('parameters', default={"reg_coeff": 0.0}): dict,
+    Optional('resume_from_pruned_checkpoint', default=False): str,
+    Optional('pruners'): And(list, \
+                             lambda s: all(isinstance(i, Pruner) for i in s))
+})
+
+
 # weight_compression_pytorch_schema = Schema({},ignore_extra_keys=True)
 
 approach_schema = Schema({
     Hook('weight_compression', handler=_valid_prune_sparsity): object,
     Hook('weight_compression_pytorch', handler=_valid_prune_sparsity): object,
     Optional('weight_compression'): weight_compression_schema,
+    Optional('weight_compression_v2'): weight_compression_v2_schema,
     Optional('weight_compression_pytorch'): weight_compression_schema,
 })
 
@@ -720,19 +789,20 @@ schema = Schema({
         Optional('inputs', default=[]): And(Or(str, list), Use(input_to_list)),
         Optional('outputs', default=[]): And(Or(str, list), Use(input_to_list)),
     },
-    Optional('version', default=float(__version__.split('.')[0])): And(
-                                          Or(float,
-                                             And(int, Use(input_int_to_float)),
-                                             And(str, Use(input_int_to_float))),
-                                          lambda s: s == float(__version__.split('.')[0])),
+    Optional('version', default=float(__version__.split('.')[0])): Or(float,
+                                                                      And(int, Use(input_int_to_float)),
+                                                                      And(str, Use(input_int_to_float))),
     Optional('device', default='cpu'): And(str, lambda s: s in ['cpu', 'gpu']),
     Optional('quantization', default={'approach': 'post_training_static_quant', \
                                       'calibration': {'sampling_size': [100]}, \
                                       'recipes': {'scale_propagation_max_pooling': True,
                                                       'scale_propagation_concat': True,
-                                                      'first_conv_or_matmul_quantization': True},
+                                                      'first_conv_or_matmul_quantization': True,
+                                                      'last_conv_or_matmul_quantization': True,
+                                                      'pre_post_process_quantization': True},
                                       'model_wise': {'weight': {'bit': [7.0]},
                                                      'activation': {}},
+                                      'quant_level': "auto",
                                       }): {
         Optional('approach', default='post_training_static_quant'): And(
             str,
@@ -740,6 +810,7 @@ schema = Schema({
             # Now only onnruntime and pytorch supoort
             lambda s: s in ['post_training_static_quant',
                             'post_training_dynamic_quant',
+                            'post_training_auto_quant',
                             'quant_aware_training']),
         Optional('train', default=None): train_schema,
         Optional('advance', default=None): {
@@ -751,13 +822,19 @@ schema = Schema({
         },
         Optional('recipes', default={'scale_propagation_max_pooling': True,
                                          'scale_propagation_concat': True,
-                                         'first_conv_or_matmul_quantization': True}): {
+                                         'first_conv_or_matmul_quantization': True,
+                                         'last_conv_or_matmul_quantization': True,
+                                         'pre_post_process_quantization': True}): {
             Optional('scale_propagation_max_pooling', default=True):
                     And(bool, lambda s: s in [True, False]),
             Optional('scale_propagation_concat', default=True):
                     And(bool, lambda s: s in [True, False]),
             Optional('first_conv_or_matmul_quantization', default=True):
                     And(bool, lambda s: s in [True, False]),
+            Optional('last_conv_or_matmul_quantization', default=True):
+                    And(bool, lambda s: s in [True, False]),
+            Optional('pre_post_process_quantization', default=True):
+                    And(bool, lambda s: s in [True, False]),
             Optional('fast_bias_correction', default=False):
                     And(bool, lambda s: s in [True, False]),
             Optional('weight_correction', default=False):
@@ -819,8 +896,9 @@ schema = Schema({
         Optional('op_wise', default=None): {
             str: ops_schema
         },
+        Optional('quant_level', default="auto"): And(Or(str, int), lambda level: level in ["auto", 0, 1]),
     },
-
+    Optional('use_bf16', default=True): bool,
     Optional('graph_optimization'): graph_optimization_schema,
     Optional('mixed_precision'): mixed_precision_schema,
 
@@ -836,11 +914,14 @@ schema = Schema({
         'diagnosis': False,
         }): {
         Optional('strategy', default={'name': 'basic'}): {
-            'name': And(str, lambda s: s in STRATEGIES), Optional('sigopt_api_token'): str,
+            'name': And(str, lambda s: s in STRATEGIES),
+            Optional('sigopt_api_token'): str,
             Optional('sigopt_project_id'): str,
             Optional('sigopt_experiment_name', default='nc-tune'): str,
             Optional('accuracy_weight', default=1.0): float,
-            Optional('latency_weight', default=1.0): float
+            Optional('latency_weight', default=1.0): float,
+            Optional('confidence_batches', default=2): int,
+            Optional('hawq_v2_loss', default=None): object,
         } ,
         Hook('accuracy_criterion', handler=_valid_accuracy_field): object,
         Optional('accuracy_criterion', default={'relative': 0.01}): {
@@ -850,7 +931,7 @@ schema = Schema({
         },
         Optional('objective', default='performance'): And(str, lambda s: s in OBJECTIVES),
         Hook('multi_objectives', handler=_valid_multi_objectives): object,
-        Optional('multi_objectives'):{ 
+        Optional('multi_objectives'):{
             Optional('objective'): And(
                 Or(str, list), Use(input_to_list), lambda s: all(i in OBJECTIVES for i in s)),
             Optional('weight'): And(Or(str, list), Use(input_to_list_float)),
@@ -909,9 +990,9 @@ schema = Schema({
                 Optional('COCOmAPv2'): {
                     Optional('anno_path'): str,
                     Optional('map_key', default='DetectionBoxes_Precision/mAP'): str,
-                    Optional('output_index_mapping', default={'num_detections': -1, 
-                                                      'boxes': 0, 
-                                                      'scores': 1, 
+                    Optional('output_index_mapping', default={'num_detections': -1,
+                                                      'boxes': 0,
+                                                      'scores': 1,
                                                       'classes': 2}): COCOmAP_input_order_schema
                 },
                 Optional('VOCmAP'): {
@@ -941,7 +1022,7 @@ schema = Schema({
                 Optional('ROC'): {
                     Optional('task'): str
                 },
-            }, 
+            },
             Optional('metric', default=None): {
                 Optional('topk'): And(int, lambda s: s in [1, 5]),
                 Optional('mAP'): {
@@ -958,9 +1039,9 @@ schema = Schema({
                 Optional('COCOmAPv2'): {
                     Optional('anno_path'): str,
                     Optional('map_key', default='DetectionBoxes_Precision/mAP'): str,
-                    Optional('output_index_mapping', default={'num_detections': -1, 
-                                                      'boxes': 0, 
-                                                      'scores': 1, 
+                    Optional('output_index_mapping', default={'num_detections': -1,
+                                                      'boxes': 0,
+                                                      'scores': 1,
                                                       'classes': 2}): COCOmAP_input_order_schema
                 },
                 Optional('VOCmAP'): {
@@ -1064,7 +1145,10 @@ schema = Schema({
             Optional("num_evals", default=100000): int,
             Optional("results_csv_path", default=None): str,
             Optional("dataset_path", default=None): str,
+            Optional("supernet_ckpt_path", default=None): str,
             Optional("batch_size", default=64): int,
+            Optional("num_workers", default=20): int,
+            Optional("distributed", default=False): bool,
             },
     },
 
@@ -1084,11 +1168,14 @@ quantization_default_schema = Schema({
                                       'calibration': {'sampling_size': [100]},
                                       'recipes': {'scale_propagation_max_pooling': True,
                                                       'scale_propagation_concat': True,
-                                                      'first_conv_or_matmul_quantization': True},
+                                                      'first_conv_or_matmul_quantization': True,
+                                                      'last_conv_or_matmul_quantization': True,
+                                                      'pre_post_process_quantization': True},
                                       'model_wise': {'weight': {'bit': [7.0]},
                                                      'activation': {}},
+                                      'quant_level': "auto",
                                     }): dict,
-
+    Optional('use_bf16', default=False): bool,
     Optional('tuning', default={
         'strategy': {'name': 'basic'},
         'accuracy_criterion': {'relative': 0.01, 'higher_is_better': True},
@@ -1109,6 +1196,8 @@ pruning_default_schema = Schema({
 
     Optional('device', default='cpu'): str,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('tuning', default={
         'random_seed': 1978, 'tensorboard': False,
         'workspace': {'path': default_workspace}}): dict,
@@ -1129,14 +1218,18 @@ graph_optimization_default_schema = Schema({
 
     Optional('device', default='cpu'): str,
 
-    Optional('quantization', default={'approach': 'post_training_static_quant', 
+    Optional('quantization', default={'approach': 'post_training_static_quant',
                                     'calibration': {'sampling_size': [100]},
                                     'recipes': {'scale_propagation_max_pooling': True,
                                                     'scale_propagation_concat': True,
-                                                    'first_conv_or_matmul_quantization': True},
+                                                    'first_conv_or_matmul_quantization': True,
+                                                    'last_conv_or_matmul_quantization': True,
+                                                    'pre_post_process_quantization': True},
                                     'model_wise': {'weight': {'bit': [7.0]},
                                                     'activation': {}}}): dict,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('tuning', default={
         'strategy': {'name': 'basic'},
         'accuracy_criterion': {'relative': 0.01, 'higher_is_better': True},
@@ -1147,7 +1240,7 @@ graph_optimization_default_schema = Schema({
 
     Optional('evaluation', default={'accuracy': {'metric': {'topk': 1}}}): dict,
 
-    Optional('graph_optimization', default={'precisions': ['bf16, fp32']}): dict 
+    Optional('graph_optimization', default={'precisions': ['bf16, fp32']}): dict
 })
 
 mixed_precision_default_schema = Schema({
@@ -1159,14 +1252,18 @@ mixed_precision_default_schema = Schema({
 
     Optional('device', default='cpu'): str,
 
-    Optional('quantization', default={'approach': 'post_training_static_quant', 
+    Optional('quantization', default={'approach': 'post_training_static_quant',
                                     'calibration': {'sampling_size': [100]},
                                     'recipes': {'scale_propagation_max_pooling': True,
                                                     'scale_propagation_concat': True,
-                                                    'first_conv_or_matmul_quantization': True},
+                                                    'first_conv_or_matmul_quantization': True,
+                                                    'last_conv_or_matmul_quantization': True,
+                                                    'pre_post_process_quantization': True},
                                     'model_wise': {'weight': {'bit': [7.0]},
                                                     'activation': {}}}): dict,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('tuning', default={
         'strategy': {'name': 'basic'},
         'accuracy_criterion': {'relative': 0.01, 'higher_is_better': True},
@@ -1177,7 +1274,7 @@ mixed_precision_default_schema = Schema({
 
     Optional('evaluation', default={'accuracy': {'metric': {'topk': 1}}}): dict,
 
-    Optional('mixed_precision', default={'precisions': ['bf16, fp32']}): dict 
+    Optional('mixed_precision', default={'precisions': ['bf16, fp32']}): dict
 })
 
 benchmark_default_schema = Schema({
@@ -1189,11 +1286,15 @@ benchmark_default_schema = Schema({
 
     Optional('device', default='cpu'): str,
 
-    Optional('quantization', default={'approach': 'post_training_static_quant', 
+    Optional('use_bf16', default=False): bool,
+
+    Optional('quantization', default={'approach': 'post_training_static_quant',
                                     'calibration': {'sampling_size': [100]},
                                     'recipes': {'scale_propagation_max_pooling': True,
                                                     'scale_propagation_concat': True,
-                                                    'first_conv_or_matmul_quantization': True},
+                                                    'first_conv_or_matmul_quantization': True,
+                                                    'last_conv_or_matmul_quantization': True,
+                                                    'pre_post_process_quantization': True},
                                     'model_wise': {'weight': {'bit': [7.0]},
                                                     'activation': {}}}): dict,
 
@@ -1217,20 +1318,22 @@ distillation_default_schema = Schema({
 
     Optional('device', default='cpu'): str,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('tuning', default={
         'random_seed': 1978, 'tensorboard': False,
         'workspace': {'path': default_workspace}}): dict,
 
     Optional('distillation', default={
-        'train': {'start_epoch': 0, 'end_epoch': 10, 
-                  'iteration': 1000, 'frequency': 1, 
-                  'optimizer': {'SGD': {'learning_rate': 0.001}}, 
-                  'criterion': {'KnowledgeDistillationLoss': 
-                                 {'temperature': 1.0, 
-                                  'loss_types': ['CE', 'KL'], 
+        'train': {'start_epoch': 0, 'end_epoch': 10,
+                  'iteration': 1000, 'frequency': 1,
+                  'optimizer': {'SGD': {'learning_rate': 0.001}},
+                  'criterion': {'KnowledgeDistillationLoss':
+                                 {'temperature': 1.0,
+                                  'loss_types': ['CE', 'KL'],
                                   'loss_weights': [0.5, 0.5]}}}}): dict,
 
-    Optional('evaluation', default={'accuracy': {'metric': {'topk': 1}}}):dict
+    Optional('evaluation', default={'accuracy': {'metric': {'topk': 1}}}): dict
  
 })
 
@@ -1271,7 +1374,7 @@ class Conf(object):
                 with open(cfg_fname, 'w') as f:
                     f.write(content)
 
-            return validated_cfg   
+            return validated_cfg
         except FileNotFoundError as f:
             logger.error("{}.".format(f))
             raise RuntimeError(
@@ -1283,6 +1386,121 @@ class Conf(object):
                 "The yaml file format is not correct. Please refer to document."
             )
 
+    def map_pyconfig_to_cfg(self, pythonic_config):
+        mapping = {}
+        if pythonic_config.quantization is not None:
+            mapping.update({
+                'device': pythonic_config.quantization.device,
+                'model.inputs': pythonic_config.quantization.inputs,
+                'model.outputs': pythonic_config.quantization.outputs,
+                'model.backend': pythonic_config.quantization.backend,
+                'model.quant_format': pythonic_config.quantization.quant_format,
+                'model.domain': pythonic_config.quantization.domain,
+                'quantization.recipes': pythonic_config.quantization.recipes,
+                'quantization.approach': pythonic_config.quantization.approach,
+                'quantization.example_inputs': pythonic_config.quantization.example_inputs,
+                'quantization.calibration.sampling_size':
+                    pythonic_config.quantization.calibration_sampling_size,
+                'quantization.optype_wise': pythonic_config.quantization.op_type_dict,
+                'quantization.op_wise': pythonic_config.quantization.op_name_dict,
+                'tuning.strategy.name': pythonic_config.quantization.strategy,
+                'tuning.accuracy_criterion.relative':
+                    pythonic_config.quantization.accuracy_criterion.relative,
+                'tuning.accuracy_criterion.absolute':
+                    pythonic_config.quantization.accuracy_criterion.absolute,
+                'tuning.accuracy_criterion.higher_is_better':
+                    pythonic_config.quantization.accuracy_criterion.higher_is_better,
+                'tuning.objective': pythonic_config.quantization.objective,
+                'tuning.exit_policy.timeout': pythonic_config.quantization.timeout,
+                'tuning.exit_policy.max_trials': pythonic_config.quantization.max_trials,
+                'tuning.exit_policy.performance_only': pythonic_config.quantization.performance_only,
+                'tuning.use_distributed_tuning': pythonic_config.quantization.use_distributed_tuning,
+                'use_bf16': pythonic_config.quantization.use_bf16,
+                'quantization.quant_level': pythonic_config.quantization.quant_level,
+                'reduce_range': pythonic_config.quantization.reduce_range
+            })
+            if pythonic_config.quantization.strategy_kwargs:
+                st_kwargs = pythonic_config.quantization.strategy_kwargs
+                for st_key in ['sigopt_api_token', 'sigopt_project_id', 'sigopt_experiment_name', \
+                    'accuracy_weight', 'latency_weight', 'hawq_v2_loss', 'confidence_batches']:
+                    if st_key in st_kwargs:
+                        st_val =  st_kwargs[st_key]
+                        mapping.update({'tuning.strategy.' + st_key: st_val})
+
+        if pythonic_config.distillation is not None:
+            mapping.update({
+                'distillation.train.criterion': pythonic_config.distillation.criterion,
+                'distillation.train.optimizer': pythonic_config.distillation.optimizer,
+            })
+        if pythonic_config.pruning is not None:
+            mapping.update({
+                'pruning.approach.weight_compression': pythonic_config.pruning.weight_compression,
+            })
+        if pythonic_config.nas is not None:
+            mapping.update({
+                'nas.approach': pythonic_config.nas.approach,
+                'nas.search': pythonic_config.nas.search,
+                'nas.dynas': pythonic_config.nas.dynas,
+            })
+        if pythonic_config.options is not None:
+            mapping.update({
+                'tuning.random_seed': pythonic_config.options.random_seed,
+                'tuning.workspace.path': pythonic_config.options.workspace,
+                'tuning.workspace.resume': pythonic_config.options.resume_from,
+                'tuning.tensorboard': pythonic_config.options.tensorboard,
+            })
+        if pythonic_config.benchmark is not None:
+            if pythonic_config.benchmark.inputs != []:
+                mapping.update({'model.inputs': pythonic_config.benchmark.inputs})
+            if pythonic_config.benchmark.outputs != []:
+                mapping.update({'model.outputs': pythonic_config.benchmark.outputs})
+            mapping.update({
+                'evaluation.performance.warmup': pythonic_config.benchmark.warmup,
+                'evaluation.performance.iteration': pythonic_config.benchmark.iteration,
+                'evaluation.performance.configs.cores_per_instance':
+                    pythonic_config.benchmark.cores_per_instance,
+                'evaluation.performance.configs.num_of_instance':
+                    pythonic_config.benchmark.num_of_instance,
+                'evaluation.performance.configs.inter_num_of_threads':
+                    pythonic_config.benchmark.inter_num_of_threads,
+                'evaluation.performance.configs.intra_num_of_threads':
+                    pythonic_config.benchmark.intra_num_of_threads,
+                'evaluation.accuracy.configs.cores_per_instance':
+                    pythonic_config.benchmark.cores_per_instance,
+                'evaluation.accuracy.configs.num_of_instance':
+                    pythonic_config.benchmark.num_of_instance,
+                'evaluation.accuracy.configs.inter_num_of_threads':
+                    pythonic_config.benchmark.inter_num_of_threads,
+                'evaluation.accuracy.configs.intra_num_of_threads':
+                    pythonic_config.benchmark.intra_num_of_threads,
+            })
+            if "model.backend" not in mapping:
+                mapping.update({
+                    'model.backend': pythonic_config.benchmark.backend,
+                })
+            else:
+                if mapping['model.backend'] == 'default' and \
+                        pythonic_config.benchmark.backend != 'default':
+                    mapping.update({
+                        'model.backend': pythonic_config.benchmark.backend,
+                    })
+
+        if "model.backend" not in mapping:
+            mapping.update({
+                'model.backend': "default",
+            })
+
+        for k, v in mapping.items():
+            if k in ['tuning.accuracy_criterion.relative', 'tuning.accuracy_criterion.absolute']:
+                target_key = str(pythonic_config.quantization.accuracy_criterion)
+                if target_key not in k and 'accuracy_criterion' in self.usr_cfg.tuning:
+                    if target_key in self.usr_cfg.tuning.accuracy_criterion and \
+                                    k.split('.')[-1] in self.usr_cfg.tuning.accuracy_criterion:
+                        self.usr_cfg.tuning.accuracy_criterion.pop(k.split('.')[-1])
+                    continue
+            if v is not None:
+                deep_set(self.usr_cfg, k, v)
+
     def _convert_cfg(self, src, dst):
         """Helper function to merge user defined dict into default dict.
 
@@ -1301,7 +1519,7 @@ class Conf(object):
         for key in src:
             if key in dst:
                 if isinstance(dst[key], dict) and isinstance(src[key], dict):
-                    if key in ['accuracy_criterion', 'metric', 'dataset', 
+                    if key in ['accuracy_criterion', 'metric', 'dataset',
                         'criterion', 'optimizer']:
                         # accuracy_criterion can only have one of absolute and relative
                         # others can only have one item
@@ -1384,139 +1602,6 @@ class Quantization_Conf(Conf):
 
         return self._model_wise_tune_space
 
-    def _weight_compute(self, combined_cfg):
-        temp_set = set()
-        for _, config in combined_cfg.items():
-            temp_str = ''
-            for part, params in config.items():
-                temp_str = temp_str + part
-                for _, param in params.items():
-                    temp_str += str(param)
-                temp_str += '_'
-            temp_set.add(temp_str)
-        return len(temp_set)
-
-    def _sort_cfgs(self, combined_cfgs):
-        cfgs_num = len(combined_cfgs)
-        for i in range(cfgs_num):
-            for j in range(cfgs_num-i-1):
-                weight_a = self._weight_compute(combined_cfgs[j])
-                weight_b = self._weight_compute(combined_cfgs[j+1])
-                if weight_a > weight_b:
-                    temp = combined_cfgs[j]
-                    combined_cfgs[j] = combined_cfgs[j+1]
-                    combined_cfgs[j+1] = temp
-        return combined_cfgs
-
-    def _combine_optype_quant_cfgs(self, model_wise_quant_cfgs):
-        if len(model_wise_quant_cfgs) == 0:
-            return []
-        temp_cfgs = OrderedDict()
-
-        for optype, cfgs in model_wise_quant_cfgs.items():
-            if len(cfgs) > 0:
-                temp_cfgs[optype] = copy.deepcopy(cfgs)
-
-        if not bool(temp_cfgs):
-            return []
-
-        keys, values = zip(*temp_cfgs.items())
-
-        return self._sort_cfgs([dict(zip(keys, v)) for v in itertools.product(*values)])
-
-    def opwise_tune_space(self, opwise_quant):
-        def _is_regex(pattern):
-            if re.match("^[A-Za-z0-9.][A-Za-z0-9_.\\-/]*$", pattern):
-                return False
-            return True
-
-        opwise = copy.deepcopy(opwise_quant)
-
-        cfg = self.usr_cfg
-        if cfg.quantization.op_wise:
-            for k, v in cfg.quantization.op_wise.items():
-                is_regex = _is_regex(k)
-                for k_op, _ in opwise.items():
-                    if (not is_regex and k == k_op[0]) or (is_regex and re.match(k, k_op[0])):
-                        opwise[k_op] = self._merge_dicts(v, opwise[k_op])
-
-        for k, v in opwise.items():
-            opwise[k] = self._merge_dicts(self._model_wise_tune_space[k[1]], opwise[k])
-
-        self._opwise_tune_space = opwise
-        return self._opwise_tune_space
-
-    def expand_tune_cfgs(self, tune_space, framework):
-        """generate all possible tuning combinations for each op or model wise tuning.
-
-        Args:
-            tune_space (dict): The tuning space to be expanded.
-
-        Returns:
-            dict: The expanded tuning configs
-        """
-        cfg_lists = self._expand_tune_cfgs_recursively(tune_space)
-
-        # remove unreasonable tuning combinations
-        valid_cfgs = []
-        quant_dtype = ['int8', 'uint8', 'int4', 'uint4']
-
-        for cfg in cfg_lists:
-            cfg = DotDict(cfg)
-            dtype = cfg.activation.dtype
-
-            if dtype not in quant_dtype and "pytorch" not in framework:
-                cfg.activation.clear()
-                cfg.activation.dtype = dtype
-
-            if 'weight' in cfg:
-                dtype = cfg.weight.dtype
-                if dtype not in quant_dtype:
-                    cfg.weight.clear()
-                    cfg.weight.dtype = dtype
-                if (cfg.weight.dtype != cfg.activation.dtype and
-                    cfg.weight.dtype not in quant_dtype and
-                    cfg.activation.dtype not in quant_dtype) or \
-                   (cfg.weight.dtype != cfg.activation.dtype and
-                    cfg.weight.dtype in quant_dtype and
-                    cfg.activation.dtype not in quant_dtype and
-                    ("pytorch" not in framework or
-                     len(tune_space['activation']['dtype']) > 1)) or \
-                   (cfg.weight.dtype != cfg.activation.dtype and
-                    cfg.weight.dtype not in quant_dtype and
-                    cfg.activation.dtype in quant_dtype):
-                    continue
-
-            valid_cfgs.append(cfg)
-
-        # remove duplicated configurations
-        valid_cfgs = [cfg[0] for cfg in itertools.groupby(valid_cfgs)]
-        return valid_cfgs
-
-    def _expand_tune_cfgs_recursively(self, cfg_dict):
-        """Helper function of recursively generating all combinations.
-
-        Args:
-            cfg_dict (dict): The dict of conf space.
-
-        Returns:
-            list: List containing all combinations
-        """
-        assert isinstance(cfg_dict, dict)
-        combinations = OrderedDict()
-        for key in cfg_dict:
-            if isinstance(cfg_dict[key], dict):
-                lists = self._expand_tune_cfgs_recursively(cfg_dict[key])
-                combinations[key] = lists
-
-        if len(combinations) != 0:
-            return self._expand_tune_cfgs_recursively(combinations)
-
-        keys, values = zip(*cfg_dict.items())
-        values = list(filter(None, values))
-        lists = [dict(zip(keys, v)) for v in itertools.product(*values)]
-        return lists
-
 class Pruning_Conf(Conf):
     """config parser.
 
@@ -1630,7 +1715,7 @@ class NASConfig(Conf):
 
     def validate(self):
         self.usr_cfg = schema.validate(self.usr_cfg)
-        
+
     @property
     def nas(self):
         return self.usr_cfg.nas
@@ -1645,7 +1730,6 @@ class DefaultConf(DotDict):
     __getattr__ = __getitem__
 
 conf = DefaultConf({})
-
 QuantConf = Quantization_Conf
 PruningConf = Pruning_Conf
 GraphOptConf = Graph_Optimization_Conf
