@@ -15,6 +15,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Built-in datasets class for multiple framework backends."""
+
+from .coco_dataset import COCORecordDataset
+from .dataset import Datasets, Dataset, IterableDataset, dataset_registry, TensorflowImageRecord
 from os.path import dirname, basename, isfile, join
 import glob
 
@@ -24,3 +28,6 @@ for f in modules:
     if isfile(f) and not f.startswith('__') and not f.endswith('__init__.py'):
         __import__(basename(f)[:-3], globals(), locals(), level=1)
 
+
+__all__ = ["Datasets", "Dataset", "IterableDataset", "dataset_registry", "TensorflowImageRecord",
+            "COCORecordDataset"]
