@@ -17,8 +17,12 @@
 # ==============================================================================
 """Neural Compressor Built-in transforms for multiple framework backends."""
 
-from .transform import TRANSFORMS, BaseTransform, transform_registry
+from .transform import TRANSFORMS, BaseTransform, ComposeTransform, transform_registry, \
+ResizeTFTransform, TensorflowResizeWithRatio, RescaleTFTransform, NormalizeTFTransform
+from .transform import TFSquadV1PostTransform, TFSquadV1ModelZooPostTransform
+from .coco_transform import ParseDecodeCocoTransform
 from .postprocess import Postprocess
+from .imagenet_transform import LabelShift, BilinearImagenetTransform, TensorflowResizeCropImagenetTransform
 from os.path import dirname, basename, isfile, join
 import glob
 
@@ -29,4 +33,7 @@ for f in modules:
         __import__(basename(f)[:-3], globals(), locals(), level=1)
 
 
-__all__ = ["TRANSFORMS", "BaseTransform", "transform_registry", "Postprocess"]
+__all__ = ["TRANSFORMS", "BaseTransform", "ComposeTransform", "transform_registry", "ResizeTFTransform",
+           "Postprocess", "LabelShift", "BilinearImagenetTransform", "TensorflowResizeCropImagenetTransform",
+           "RescaleTFTransform", "NormalizeTFTransform", "ParseDecodeCocoTransform",
+           "TensorflowResizeWithRatio", "TFSquadV1PostTransform", "TFSquadV1ModelZooPostTransform"]
