@@ -20,11 +20,9 @@
 The Component class will be inherited by the class 'Quantization', 'Pruning' and 'Distillation'.
 """
 
-from ..conf.config import Conf
 from ..utils import logger
 from ..utils.utility import required_libs
 from ..utils.create_obj_from_config import create_dataloader, create_train_func, create_eval_func
-from ..model import BaseModel
 from .common import Model
 from ..adaptor import FRAMEWORKS
 from ..model.model import get_model_fwk_name
@@ -87,6 +85,7 @@ class Component(object):
             'on_after_eval': []
         }
         if conf_fname_or_obj is not None:  # pragma: no cover
+            from ..conf.config import Conf
             if isinstance(conf_fname_or_obj, str):
                 self.conf = Conf(conf_fname_or_obj)
             elif isinstance(conf_fname_or_obj, Conf):
@@ -102,6 +101,8 @@ class Component(object):
         self.cfg = self.conf.usr_cfg
         if self.cfg.model.framework != 'NA':
             self.framework = self.cfg.model.framework.lower()
+            from neural_compressor.experimental.common.model import set_backend
+            set_backend(self.framework)
             if self.framework in required_libs:
                 for lib in required_libs[self.framework]:
                     try:
@@ -205,7 +206,7 @@ class Component(object):
                                                self.adaptor,
                                                metric,
                                                self.cfg.evaluation.accuracy.postprocess,
-                                               fp32_baseline = False)
+                                               fp32_baseline=False)
 
         self.prepare()
         # strategy will be considered in future
@@ -491,6 +492,7 @@ class Component(object):
                         make sure the name is in supported slim model list.
 
         """
+        from ..model import BaseModel
         if self.cfg.model.framework == 'NA':
             assert not isinstance(user_model, BaseModel), \
                 "Please pass an original framework model but not neural compressor model!"
@@ -508,7 +510,10 @@ class Component(object):
 
         if not isinstance(user_model, BaseModel):
             logger.warning("Force convert framework model to neural_compressor model.")
-            self._model = Model(user_model, framework=self.framework)
+            if "tensorflow" in self.framework or self.framework == "keras":
+                self._model = Model(user_model, framework=self.framework, device=self.cfg.device)
+            else:
+                self._model = Model(user_model, framework=self.framework)
         else:
             # It is config of neural_compressor version < 2.0, no need in 2.0
             if self.cfg.model.framework == "pytorch_ipex":
