@@ -1,3 +1,5 @@
+"""Distillation class."""
+
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
@@ -24,21 +26,35 @@ from .common import Model
 from ..adaptor import FRAMEWORKS
 from neural_compressor.experimental.common import Criterions, Optimizers
 from ..conf.config import DistillationConf
+from ..conf.pythonic_config import Config
 
 class Distillation(Component):
-    """
-
+    """Distillation class derived from Component class.
+    
+    Distillation class abstracted the pipeline of knowledge distillation, 
+    transfer the knowledge of the teacher model to the student model.
+       
     Args:
         conf_fname_or_obj (string or obj): The path to the YAML configuration file or
-            DistillationConf containing accuracy goal, distillation objective and related
+            Distillation_Conf containing accuracy goal, distillation objective and related
             dataloaders etc.
-
+            
+    Attributes:
+        _epoch_ran: A integer indicating how much epochs ran.
+        eval_frequency: The frequency for doing evaluation of the student model 
+            in terms of epoch.
+        best_score: The best metric of the student model in the training.
+        best_model: The best student model found in the training.
     """
 
     def __init__(self, conf_fname_or_obj=None):
+        """Initialize the attributes."""
         super(Distillation, self).__init__()
         if isinstance(conf_fname_or_obj, DistillationConf):
             self.conf = conf_fname_or_obj
+        elif isinstance(conf_fname_or_obj, Config):
+            self.conf = DistillationConf()
+            self.conf.map_pyconfig_to_cfg(conf_fname_or_obj)
         else:
             self.conf = DistillationConf(conf_fname_or_obj)
         self._init_with_conf()
@@ -46,14 +62,17 @@ class Distillation(Component):
         self._teacher_model = None
         self._criterion = None
         self._optimizer = None
-        self._epoch_runned = 0
+        self._epoch_ran = 0
         self.eval_frequency = 1
         self.best_score = 0
         self.best_model = None
         self._train_cfg = None
 
     def _on_train_begin(self, dataloader=None):
-        """ called before training """
+        """Operations called on the begining of the training.
+
+        Called before training, evaluate the teacher model and the student model. 
+        """
         assert self._model, 'student_model must be set.'
         if self._eval_func is not None:
             if self.teacher_model:
@@ -75,12 +94,22 @@ class Distillation(Component):
                     self.best_model = self._model
 
     def _on_step_begin(self, batch_id):
-        """ called on the beginning of batches"""
+        """Operations called on the beginning of batches."""
         if self.criterion is not None and hasattr(self.criterion, 'clear_features'):
             self.criterion.clear_features()
 
     def _on_after_compute_loss(self, input, student_output, student_loss, teacher_output=None):
-        """ called after model forward """
+        """Set or compute output of teacher model.
+
+        Called after student model forward, calculate the output of the teacher model
+        with the same input of the student model.
+
+        Args:
+            input (tensor or list or dict): The input of the student model.
+            student_output (tensor): The output logits of the student model.
+            student_loss (tensor or float): The original loss of the student model.
+            teacher_output (tensor, optional): The output logits of the teacher model.
+        """
         if self.criterion is None:
             self.create_criterion()
         assert self.criterion, \
@@ -93,18 +122,26 @@ class Distillation(Component):
         return self.criterion.loss_cal_sloss(student_output, teacher_output, student_loss)
 
     def on_post_forward(self, input, teacher_output=None):  # pragma: no cover
+        """Set or compute output of teacher model.
+
+        Deprecated.
+        """
         assert False, "This method is deprecated. please use `on_after_compute_loss` instead." \
                       "on_after_compute_loss(input, student_output, student_loss, teacher_output=None)"
 
     def _on_epoch_end(self):
-        """ called on the end of epochs"""
-        self._epoch_runned += 1
+        """Operations called on the end of every epochs.
+
+        Called on the end of every epochs, evaluate the student model 
+        and record the best one regularly.
+        """
+        self._epoch_ran += 1
         if self._eval_func is not None and self.eval_frequency > 0 and \
-           self._epoch_runned % self.eval_frequency == 0:
+           self._epoch_ran % self.eval_frequency == 0:
             score = self._eval_func(
                 self._model if getattr(self._eval_func, 'builtin', None) else self._model.model
             )
-            logger.info("model score of epoch {} is {}.".format(self._epoch_runned, str(score)))
+            logger.info("model score of epoch {} is {}.".format(self._epoch_ran, str(score)))
             if (isinstance(score, list) and all([s > b_s for s, b_s in
                 zip(score, self.best_score)])) or score > self.best_score:
                 self.best_score = score
@@ -114,6 +151,7 @@ class Distillation(Component):
                     self.best_model = self._model
 
     def init_train_cfg(self):
+        """Initialize the training configuration."""
         if self._train_cfg is None:
             # train section of distillation section in yaml file should be configured.
             self._train_cfg = self.cfg.distillation.train
@@ -121,6 +159,7 @@ class Distillation(Component):
                                 "be configured for distillation if train_func is NOT set."
 
     def create_criterion(self):
+        """Create the criterion for training."""
         self.init_train_cfg()
         if self.criterion is None:
             assert 'criterion' in self._train_cfg.keys(), \
@@ -150,6 +189,7 @@ class Distillation(Component):
         self._train_cfg.criterion = self.criterion
 
     def create_optimizer(self):
+        """Create the optimizer for training."""
         self.init_train_cfg()
         if self.optimizer is None:
             assert 'optimizer' in self._train_cfg.keys(), \
@@ -174,11 +214,22 @@ class Distillation(Component):
 
         self._train_cfg.optimizer = self.optimizer
 
+    def prepare(self):
+        """Prepare hooks."""
+        self.generate_hooks()
+        self.create_criterion()
+
     def pre_process(self):
+        """Preprocessing before the disillation pipeline.
+        
+        Initialize necessary parts for distillation pipeline. 
+        """
         framework_specific_info = {'device': self.cfg.device,
                                    'random_seed': self.cfg.tuning.random_seed,
                                    'workspace_path': self.cfg.tuning.workspace.path,
-                                   'q_dataloader': None}
+                                   'q_dataloader': None,
+                                   'format': 'default',
+                                   'backend': 'default'}
 
         if self.framework == 'tensorflow':
             framework_specific_info.update(
@@ -189,11 +240,9 @@ class Distillation(Component):
         self.generate_hooks()
         assert isinstance(self._model, BaseModel), 'need set neural_compressor Model for distillation....'
 
-        if self._train_dataloader is None and self._train_func is None:
+        if self._train_dataloader is None and self._train_func is None and \
+            self.cfg.distillation.train.dataloader is not None:
             train_dataloader_cfg = self.cfg.distillation.train.dataloader
-            assert train_dataloader_cfg is not None, \
-                   'dataloader field of train field of distillation section ' \
-                   'in yaml file should be configured as train_dataloader property is NOT set!'
 
             self._train_dataloader = create_dataloader(self.framework, train_dataloader_cfg)
 
@@ -208,13 +257,15 @@ class Distillation(Component):
             self._eval_dataloader = create_dataloader(self.framework, eval_dataloader_cfg)
 
         if self._train_func is None:
-            self.create_criterion()
+            if self.criterion is None:
+                self.create_criterion()
             self.create_optimizer()
-            self._train_func = create_train_func(self.framework, \
-                                                 self.train_dataloader, \
-                                                 self.adaptor, \
-                                                 self._train_cfg, \
-                                                 hooks=self.hooks)
+            if self._train_dataloader is not None:
+                self._train_func = create_train_func(self.framework, \
+                                                     self.train_dataloader, \
+                                                     self.adaptor, \
+                                                     self._train_cfg, \
+                                                     hooks=self.hooks)
         if self.cfg.evaluation and self.eval_dataloader and self._eval_func is None:
             # eval section in yaml file should be configured.
             eval_cfg = self.cfg.evaluation
@@ -228,6 +279,14 @@ class Distillation(Component):
                                                fp32_baseline=False)
 
     def execute(self):
+        """Do distillation pipeline.
+
+        First train the student model with the teacher model, after training, 
+        evaluating the best student model if any.
+
+        Returns:
+            Best distilled model found.
+        """
         self._train_func(
             self._model if getattr(self._train_func, 'builtin', None) else self._model.model
         )
@@ -245,14 +304,17 @@ class Distillation(Component):
         return self._model
 
     def generate_hooks(self):
-        # register hooks for distillation
+        """Register hooks for distillation.
+
+        Register necessary hooks for distillation pipeline.
+        """
         self.register_hook('on_train_begin', self._on_train_begin)
         self.register_hook('on_step_begin', self._on_step_begin)
         self.register_hook('on_after_compute_loss', self._on_after_compute_loss)
         self.register_hook('on_epoch_end', self._on_epoch_end)
 
     def __call__(self):
-        """The main entry point of distillation.
+        """Do distillation workflow.
 
            This interface currently only works on pytorch
            and provides three usages:
@@ -296,28 +358,58 @@ class Distillation(Component):
 
     @property
     def criterion(self):
+        """Getter of criterion.
+
+        Returns:
+            The criterion used in the distillation process.
+        """
         return self._criterion
 
     @criterion.setter
     def criterion(self, user_criterion):
+        """Setter of criterion used in the distillation process.
+        
+        Set the user defined criterion. When using built-in train_func, user can 
+        specify the customized criterion through this setter.
+
+        Args:
+            user_criterion (criterion object): User defined criterion.
+        """
         self._criterion = user_criterion
 
     @property
     def optimizer(self):
+        """Getter of optimizer.
+
+        Returns:
+            The optimizer used in the distillation process.
+        """
         return self._optimizer
 
     @optimizer.setter
     def optimizer(self, user_optimizer):
+        """Setter of optimizer used in the distillation process.
+        
+        Set the user defined optimizer. When using built-in train_func, user can 
+        specify the customized optimizer through this setter.
+
+        Args:
+            user_optimizer (criterion object): User defined optimizer.
+        """
         self._optimizer = user_optimizer
 
     @property
     def teacher_model(self):
-        """ Getter of model in neural_compressor.model  """
+        """Getter of the teacher model.
+
+        Returns:
+            The teacher model used in the distillation process.
+        """
         return self._teacher_model
 
     @teacher_model.setter
     def teacher_model(self, user_model):
-        """Set the user model and dispatch to framework specific internal model object
+        """Set the user model and dispatch to framework specific internal model object.
 
         Args:
            user_model: user are supported to set model from original framework model format
@@ -341,12 +433,16 @@ class Distillation(Component):
 
     @property
     def student_model(self):
-        """ Getter of model in neural_compressor.model  """
+        """Getter of the student model.
+
+        Returns:
+            The student model used in the distillation process.
+        """
         return self._model
 
     @student_model.setter
     def student_model(self, user_model):
-        """Set the user model and dispatch to framework specific internal model object
+        """Set the user model and dispatch to framework specific internal model object.
 
         Args:
            user_model: user are supported to set model from original framework model format
@@ -370,26 +466,33 @@ class Distillation(Component):
 
     @property
     def train_cfg(self):
-        """ Getter of model in neural_compressor.model  """
+        """Getter of the train configuration.
+
+        Returns:
+            The train configuration used in the distillation process.
+        """
         return self._train_cfg
 
     @property
     def evaluation_distributed(self):
-        """ Getter to know whether need distributed evaluation dataloader"""
+        """Getter to know whether need distributed evaluation dataloader."""
         return self._evaluation_distributed
 
     @evaluation_distributed.setter
     def evaluation_distributed(self, distributed):
+        """Setter to know whether need distributed evaluation dataloader."""
         self._evaluation_distributed = distributed
 
     @property
     def train_distributed(self):
-        """ Getter to know whether need distributed training dataloader"""
+        """Getter to know whether need distributed training dataloader."""
         return self._train_distributed
 
     @train_distributed.setter
     def train_distributed(self, distributed):
+        """Setter to know whether need distributed training dataloader."""
         self._train_distributed = distributed
 
     def __repr__(self):
+        """Class representation."""
         return 'Distillation'
