@@ -50,7 +50,7 @@ def get_model_type(model):
         if (model.endswith('.h5') and os.path.isfile(model)) or \
           is_saved_model_format(os.path.dirname(model)) or \
           (os.path.isdir(model) and is_saved_model_format(model)):
-            if version1_lt_version2(tf.version.VERSION, '2.10.0'):
+            if version1_lt_version2(tf.version.VERSION, '2.10.0'): # pragma: no cover
                 logger.warn("keras model running on tensorflow 2.10.0 and"
                             " lower not support intel ITEX.")
             try:
@@ -172,10 +172,36 @@ def graph_def_session(model, input_tensor_names, output_tensor_names, **kwargs):
         input_tensor_names (list of string): validated input_tensor_names
         output_tensor_names (list of string): validated output_tensor_names
     """
+    device = kwargs.get('device')
     graph = tf.Graph()
+    if version1_lt_version2(tf.version.VERSION, '2.0.0'): # pragma: no cover
+        from tensorflow._api.v1.config import experimental
+        list_physical_devices = experimental.list_physical_devices
+    else:
+        list_physical_devices = tf.config.list_physical_devices
+
     try:
         with graph.as_default():
-            tf.import_graph_def(model, name='')
+            if device == "cpu":
+                cpus = list_physical_devices("CPU")
+                node_device = cpus[0].name.replace('physical_device:', '')
+                with graph.device(node_device):
+                    tf.import_graph_def(model, name='')
+            else: # pragma: no cover
+                found_device = False
+                gpus = list_physical_devices("GPU")
+                for gpu in gpus:
+                    if gpu.name.replace('physical_device:', '') == device:
+                        found_device = True
+                xpus = list_physical_devices("XPU")
+                for xpu in xpus:
+                    if xpu.name.replace('physical_device:', '') == device:
+                        found_device = True
+                if found_device:
+                    with graph.device(device):
+                        tf.import_graph_def(model, name='')
+                else:
+                    tf.import_graph_def(model, name='')
     except:
         input_tensor_names, output_tensor_names = validate_and_inference_input_output(\
             model, input_tensor_names, output_tensor_names)
@@ -186,7 +212,26 @@ def graph_def_session(model, input_tensor_names, output_tensor_names, **kwargs):
         output_node_names = tensor_to_node(output_tensor_names)
         model = strip_unused_nodes(model, input_node_names, output_node_names)
         with graph.as_default():
-            tf.import_graph_def(model, name='')
+            if device == "cpu":
+                cpus = list_physical_devices("CPU")
+                node_device = cpus[0].name.replace('physical_device:', '')
+                with graph.device(node_device):
+                    tf.import_graph_def(model, name='')
+            else: # pragma: no cover
+                found_device = False
+                gpus = list_physical_devices("GPU")
+                for gpu in gpus:
+                    if gpu.name.replace('physical_device:', '') == device:
+                        found_device = True
+                xpus = list_physical_devices("XPU")
+                for xpu in xpus:
+                    if xpu.name.replace('physical_device:', '') == device:
+                        found_device = True
+                if found_device:
+                    with graph.device(device):
+                        tf.import_graph_def(model, name='')
+                else:
+                    tf.import_graph_def(model, name='')
 
     return graph_session(graph, input_tensor_names, output_tensor_names, **kwargs)
 
@@ -455,7 +500,7 @@ def keras_session(model, input_tensor_names, output_tensor_names, **kwargs):
     return graph_def_session(graph_def, input_names, output_names, **kwargs)
 
 
-def slim_session(model, input_tensor_names, output_tensor_names, **kwargs):
+def slim_session(model, input_tensor_names, output_tensor_names, **kwargs): # pragma: no cover
     """Build session with slim model.
 
     Args:
@@ -502,7 +547,7 @@ def slim_session(model, input_tensor_names, output_tensor_names, **kwargs):
                 clear_devices=True,
                 initializer_nodes='')
 
-    return graph_def_session(graph_def, ['input'], output_tensor_names)
+    return graph_def_session(graph_def, ['input'], output_tensor_names, **kwargs)
 
 def checkpoint_session(model, input_tensor_names, output_tensor_names, **kwargs):
     """Build session with ckpt model.
@@ -528,9 +573,37 @@ def checkpoint_session(model, input_tensor_names, output_tensor_names, **kwargs)
     config.inter_op_parallelism_threads = 1
     graph = tf.Graph()
     sess = tf.compat.v1.Session(graph=graph, config=config)
+    if version1_lt_version2(tf.version.VERSION, '2.0.0'): # pragma: no cover
+        from tensorflow._api.v1.config import experimental
+        list_physical_devices = experimental.list_physical_devices
+    else:
+        list_physical_devices = tf.config.list_physical_devices
+
     with graph.as_default():
-        saver = tf.compat.v1.train.import_meta_graph(\
-            os.path.join(model, ckpt_prefix + '.meta'), clear_devices=True)
+        device = kwargs.get('device')
+        if device == "cpu":
+            cpus = list_physical_devices("CPU")
+            node_device = cpus[0].name.replace('physical_device:', '')
+            with graph.device(node_device):
+                saver = tf.compat.v1.train.import_meta_graph(\
+                    os.path.join(model, ckpt_prefix + '.meta'), clear_devices=True)
+        else: # pragma: no cover
+            found_device = False
+            gpus = list_physical_devices("GPU")
+            for gpu in gpus:
+                if gpu.name.replace('physical_device:', '') == device:
+                    found_device = True
+            xpus = list_physical_devices("XPU")
+            for xpu in xpus:
+                if xpu.name.replace('physical_device:', '') == device:
+                    found_device = True
+            if found_device:
+                with graph.device(device):
+                    saver = tf.compat.v1.train.import_meta_graph(\
+                        os.path.join(model, ckpt_prefix + '.meta'), clear_devices=True)
+            else:
+                saver = tf.compat.v1.train.import_meta_graph(\
+                    os.path.join(model, ckpt_prefix + '.meta'), clear_devices=True)
 
         sess.run(tf.compat.v1.global_variables_initializer())
         saver.restore(sess, os.path.join(model, ckpt_prefix))
@@ -585,7 +658,7 @@ def estimator_session(model, input_tensor_names, output_tensor_names, **kwargs):
             graph_def = tf.compat.v1.graph_util.convert_variables_to_constants(sess,
               g.as_graph_def(), output_nodes)
 
-        return graph_def_session(graph_def, input_tensor_names, outputs)
+        return graph_def_session(graph_def, input_tensor_names, outputs, **kwargs)
 
 def saved_model_session(model, input_tensor_names, output_tensor_names, **kwargs):
     """Build session with saved model.
@@ -732,9 +805,10 @@ class TensorflowBaseModel(BaseModel):
         """Set graph defination."""
         if self._sess is not None:
             self._sess.close()
-        output_sess =  SESSIONS['graph_def'](graph_def,\
+        output_sess = SESSIONS['graph_def'](graph_def,\
                                              self._input_tensor_names, \
-                                             self._output_tensor_names)
+                                             self._output_tensor_names, \
+                                             **self.kwargs)
 
         self._sess = output_sess[0]
         self._input_tensor_names = output_sess[1]
@@ -847,6 +921,42 @@ class TensorflowBaseModel(BaseModel):
         f.write(self.graph_def.SerializeToString())
         logger.info("Save quantized model to {}.".format(pb_file))
 
+    def export(self, save_path, conf):
+        """Export the Tensorflow fp32/int8 model to ONNX fp32/int8 model."""
+        from neural_compressor.config import TF2ONNXConfig
+
+        if isinstance(conf, TF2ONNXConfig):
+            if conf.quant_format == 'QDQ' and conf.opset_version < 13:   # pragma: no cover
+                conf.opset_version = 13
+                logger.warning("QDQ format requires opset_version >= 13, " +
+                                "we reset opset_version={} here".format(conf.opset_version))
+
+            from neural_compressor.experimental.export import tf_to_fp32_onnx, tf_to_int8_onnx
+            inputs_as_nchw = conf.kwargs.get("inputs_as_nchw", None)
+            if conf.dtype == 'int8':
+                tf_to_int8_onnx(
+                    self.graph_def,
+                    save_path,
+                    opset_version=conf.opset_version,
+                    input_names=conf.input_names if conf.input_names else self.input_tensor_names,
+                    output_names=conf.output_names if conf.output_names else self.output_tensor_names,
+                    inputs_as_nchw=inputs_as_nchw
+                )
+            elif conf.dtype == 'fp32':
+                tf_to_fp32_onnx(
+                    self.graph_def,
+                    save_path,
+                    opset_version=conf.opset_version,
+                    input_names=conf.input_names if conf.input_names else self.input_tensor_names,
+                    output_names=conf.output_names if conf.output_names else self.output_tensor_names,
+                    inputs_as_nchw=inputs_as_nchw
+                )
+            else:   # pragma: no cover
+                assert False, "Not allowed dtype: {}, pleas use 'fp32' or 'int8'.".format(conf.dtype)
+        else:
+            logger.warning("Unsupported config for export, only TF2ONNXConfig is supported!")
+            exit(0)
+
 
 class TensorflowSavedModelModel(TensorflowBaseModel):
     """Build Tensorflow saved model."""
@@ -904,8 +1014,7 @@ class TensorflowSavedModelModel(TensorflowBaseModel):
         import pandas as pd
         import tensorflow as tf
         import numpy as np
-        df = pd.DataFrame(columns=['Name', 'Shape', 'NNZ (dense)', 'NNZ (sparse)', "Sparsity(%)",
-                                   'Std', 'Mean', 'Abs-Mean'])
+        df = pd.DataFrame(columns=['Name', 'Shape', 'NNZ (dense)', 'NNZ (sparse)', "Sparsity(%)"])
         pd.set_option('display.precision', 2)
         param_dims = [2, 4]
         params_size = 0
@@ -928,20 +1037,16 @@ class TensorflowSavedModelModel(TensorflowBaseModel):
                     dense_param_size,
                     sparse_param_size,
                     (1 - density) * 100,
-                    np.std(weights),
-                    np.mean(weights),
-                    np.mean(np.abs(weights))
                 ])
 
         total_sparsity = sparse_params_size / params_size * 100
 
         df.loc[len(df.index)] = ([
             'Total sparsity:',
-            params_size,
             "-",
-            int(sparse_params_size),
-            total_sparsity,
-            0, 0, 0])
+            params_size,
+            sparse_params_size,
+            total_sparsity,])
 
         return df, total_sparsity
 
@@ -998,8 +1103,10 @@ class TensorflowQATModel(TensorflowSavedModelModel):
         """Initialize a Tensorflow QAT model.
 
         Args:
-            model (string or tensorflow model object): model path or model object.
+            model (string or  tf.keras.Model object): model path or model object.
         """
+        assert isinstance(model, tf.keras.Model) or isinstance(model, str), \
+        "The TensorflowQATModel should be initialized either by a string or a tf.keras.Model."
         super(TensorflowQATModel, self).__init__(model)
         self.keras_model = None
         self.model_type = 'keras'
@@ -1008,7 +1115,11 @@ class TensorflowQATModel(TensorflowSavedModelModel):
     def model(self):
         """Return model itself."""
         if self.keras_model == None:
-            self.keras_model = tf.keras.models.load_model(self._model)
+            if isinstance(self._model, tf.keras.Model):
+                self.keras_model = self._model
+            else:
+                self.keras_model = tf.keras.models.load_model(self._model)
+
         return self.keras_model
 
     @model.setter
@@ -1016,18 +1127,30 @@ class TensorflowQATModel(TensorflowSavedModelModel):
         """Set model itself."""
         self.keras_model = q_model
 
+    @property
+    def frozen_graph_def(self):
+        """Get frozen graph_def."""
+        graph_def = tf.compat.v1.graph_util.convert_variables_to_constants(
+            self.sess, self.sess.graph_def, self.output_node_names) 
+        return graph_def
+
     def save(self, root=None):
         """Save Tensorflow QAT model."""
         if not root:
             root = cfg.default_workspace + '/saved_model'
         root = os.path.abspath(os.path.expanduser(root))
-        # if not have suffix, default append .pb
         os.makedirs(os.path.dirname(root), exist_ok=True)
-        q_aware_model = self.keras_model
-        q_aware_model.save(root)
-        saved_format = 'saved_model'
-        if root.endswith('.h5'):
-            saved_format = 'h5 file'
+        if root.endswith('.pb'):
+            saved_format = 'pb file'
+            graph_def = self.frozen_graph_def
+            f=tf.io.gfile.GFile(root,'wb')
+            f.write(graph_def.SerializeToString()) 
+        else:
+            q_aware_model = self.keras_model
+            q_aware_model.save(root)
+            saved_format = 'saved_model'
+            if root.endswith('.h5'):
+                saved_format = 'h5 file'
         logger.info("Save quantized model to {}.".format(saved_format))
         return root
 
@@ -1055,12 +1178,18 @@ class TensorflowCheckpointModel(TensorflowBaseModel):
             self._sess.close()
         output_sess = SESSIONS['graph_def'](graph_def,
                                             self._input_tensor_names, \
-                                            self._output_tensor_names)
+                                            self._output_tensor_names, \
+                                            **self.kwargs)
         self._sess = output_sess[0]
         self._input_tensor_names = output_sess[1]
         self._output_tensor_names = output_sess[2]
         self.model_type = 'graph_def'
 
+    @property
+    def model(self):
+        """Return the model itself to avoid the initialization issue."""
+        return self
+
 
 TENSORFLOW_MODELS = {'frozen_pb': TensorflowBaseModel,
                      'graph_def': TensorflowBaseModel,
