@@ -15,9 +15,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from neural_compressor.experimental.metric import METRICS
-from neural_compressor.experimental.data import DATASETS, TRANSFORMS, FILTERS, DATALOADERS
-from neural_compressor.experimental.common import Optimizers, Criterions
+"""Utility methods to create corresponding objects from configuration."""
+
+from neural_compressor.metric import METRICS
+from neural_compressor.data import Datasets, TRANSFORMS, FILTERS, DATALOADERS
 from collections import OrderedDict
 import copy
 import gc
@@ -26,6 +27,7 @@ DEFAULT_BATCH_SIZE = 64
 
 
 def get_func_from_config(func_dict, cfg, compose=True):
+    """Get the function or the composed function from configuration."""
     func_list = []
     for func_name, func_value in OrderedDict(cfg).items():
         func_kwargs = {}
@@ -42,22 +44,36 @@ def get_func_from_config(func_dict, cfg, compose=True):
 
 
 def get_preprocess(preprocesses, cfg, compose=True):
+    """Get the preprocess function from configuration."""
     return get_func_from_config(preprocesses, cfg, compose)
 
 
 def get_metrics(metrics, cfg, compose=True):
+    """Get the metrics function from configuration."""
     return get_func_from_config(metrics, cfg, compose)
 
 def get_postprocess(postprocesses, cfg, compose=True):
+    """Get the postprocess function from configuration."""
     return get_func_from_config(postprocesses, cfg, compose)
 
 def get_algorithm(algorithms, cfg, compose=False):
+    """Get the algorithms from configuration.
+    
+    Args:
+        algorithms: the algorithm management.
+        cfg: a dict contain the algo name and use it or not. 
+        compose: compose all algo or not. Defaults to False.
+
+    Returns:
+        All open algos.
+    """
     # recipes contains quantization part, only use algorithms in that
     algo_conf = algorithms.support_algorithms().intersection(set(cfg.keys()))
     #(TODO) only support open/close according to cfg
-    return [algorithms()[algo]() for algo in algo_conf if cfg[algo]]
+    return [algorithms()[algo] for algo in algo_conf if cfg[algo]]
 
 def create_dataset(framework, data_source, cfg_preprocess, cfg_filter):
+    """Create the dataset from the data source."""
     transform_list = []
     # generate framework specific transforms
     preprocess = None
@@ -65,7 +81,7 @@ def create_dataset(framework, data_source, cfg_preprocess, cfg_filter):
         preprocesses = TRANSFORMS(framework, 'preprocess')
         preprocess = get_preprocess(preprocesses, cfg_preprocess)
     # even we can unify transform, how can we handle the IO, or we do the transform here
-    datasets = DATASETS(framework)
+    datasets = Datasets(framework)
     dataset_type = list(data_source.keys())[0]
     # generate framework and dataset specific filters
     filter = None
@@ -81,6 +97,7 @@ def create_dataset(framework, data_source, cfg_preprocess, cfg_filter):
 
 
 def create_dataloader(framework, dataloader_cfg):
+    """Create the dataloader according to the framework."""
     batch_size = int(dataloader_cfg['batch_size']) \
         if dataloader_cfg.get('batch_size') is not None else DEFAULT_BATCH_SIZE
     last_batch = dataloader_cfg['last_batch'] \
@@ -109,12 +126,18 @@ def create_eval_func(framework, dataloader, adaptor,
     """The interface to create evaluate function from config.
 
     Args:
-        model (object): The model to be evaluated.
+        framework (str): The string of framework.
+        dataloader (common.DataLoader): The object of common.DataLoader.
+        adaptor (obj): The object of adaptor.
+        metric: The evaluation metric.
+        postprocess_cfg: The postprocess configuration.
+        iteration: The number of iterations to evaluate.
+        tensorboard: Whether to use tensorboard.
+        fp32_baseline: The fp32 baseline score.
 
     Returns:
-        Objective: The objective value evaluated
+        The constructed evaluation function
     """
-
     # eval_func being None means user will provide dataloader and metric info
     # in config yaml file
     assert dataloader, "dataloader should NOT be empty when eval_func is None"
@@ -154,7 +177,7 @@ def create_train_func(framework, dataloader, adaptor, train_cfg, hooks=None, cal
         adaptor (obj): The object of adaptor.
         train_cfg (dict): The dict of training related config.
         hooks (dict): The dict of training hooks, supported keys are:
-                      on_epoch_begin, on_epoch_end, on_batch_begin, on_batch_end.
+                      on_epoch_begin, on_epoch_end, on_step_begin, on_step_end.
                       Their values are functions to be executed in adaptor layer.
 
     Returns:
@@ -164,6 +187,8 @@ def create_train_func(framework, dataloader, adaptor, train_cfg, hooks=None, cal
     # in config yaml file
     assert dataloader, "dataloader should NOT be empty when train_func is None"
     assert adaptor, "adaptor should NOT be empty"
+
+    from neural_compressor.experimental.common import Optimizers, Criterions
     postprocess_cfg = train_cfg.postprocess
     if postprocess_cfg is not None:
         postprocesses = TRANSFORMS(framework, "postprocess")
