@@ -23,17 +23,44 @@ from ..adaptor.pytorch import PyTorch_FXAdaptor
 from ..adaptor.torch_utils import util
 from . import logger
 from packaging.version import Version
-from torch.quantization import add_observer_, convert
+from torch.quantization import convert
 import torch
 import torch.quantization as tq
 import yaml
 import os
-import copy
+
 
 yaml.SafeLoader.add_constructor('tag:yaml.org,2002:python/tuple',
                                  lambda loader, node: tuple(loader.construct_sequence(node)))
 
 
+def is_int8_model(model):
+    """Check whether the input model is a int8 model.
+
+    Args:
+        model (torch.nn.Module): input model
+
+    Returns:
+        result(bool): Return True if the input model is a int8 model.
+    """
+    def _is_int8_value(value):
+        """Check whether the input tensor is a int8 tensor."""
+        if hasattr(value, 'dtype') and 'int8' in str(value.dtype):
+            return True
+        else:
+            return False
+
+    stat_dict = dict(model.state_dict())
+    for name, value in stat_dict.items():
+        if _is_int8_value(value):
+            return True
+        # value maybe a tuple, such as 'linear._packed_params._packed_params'
+        if isinstance(value, tuple):
+            for v in value:
+                if _is_int8_value(v):
+                    return True
+    return False
+
 def _set_sub_module_scale_zeropoint(model, tune_cfg, prefix=''):
     """Set activation scale and zero_point for converted sub modules recursively.
 
@@ -108,7 +135,7 @@ def _load_int8_orchestration(model, tune_cfg, stat_dict, example_inputs, **kwarg
         from torch.quantization.quantize_fx import prepare_qat_fx, convert_fx
         quantized_ops = {op[0]: q_cfgs for op in tune_cfg['quantizable_ops']}
         version = get_torch_version()
-        if version < Version("1.11.0-rc1"):
+        if version.release < Version("1.11.0").release:
             quantized_ops["default_qconfig"] = None
         else:
             from torch.ao.quantization import default_embedding_qat_qconfig
@@ -201,7 +228,7 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
                 stat_dict['best_configure'] = tune_cfg
         else:
             logger.error("Unexpected checkpoint type:{}. \
-              Only file dir/path or state_dict is acceptable")
+                         Only file dir/path or state_dict is acceptable")
 
         if not isinstance(stat_dict, torch.jit._script.RecursiveScriptModule):
             assert 'best_configure' in stat_dict, \
@@ -217,40 +244,16 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
     version = get_torch_version()
     # Get example inputs for ipex and torch>=1.13.
     example_inputs = None
-    if isinstance(stat_dict, torch.jit._script.RecursiveScriptModule) or \
-      (version.release >= Version("1.13.0").release and \
-      tune_cfg['approach'] != "post_training_dynamic_quant"):
-        if "example_inputs" in kwargs:
-            example_inputs = kwargs["example_inputs"]
-        elif "dataloader" in kwargs:
-            from ..adaptor.pytorch import get_example_inputs
-            example_inputs = get_example_inputs(model, kwargs["dataloader"])
-        else:
-            logger.warning("Please provide the example_inputs or a dataloader" +
-                            " to get example_inputs for quantized model.")
 
     if isinstance(stat_dict, torch.jit._script.RecursiveScriptModule):
         q_model = torch.jit.freeze(stat_dict.eval())
-        if isinstance(example_inputs, torch.Tensor):
-            q_model(example_inputs)
-            q_model(example_inputs)
-        else:
-            q_model(*example_inputs)
-            q_model(*example_inputs)
         logger.info("Finish load the model quantized by INC IPEX backend.")
         return q_model
 
-    try:
-        q_model = copy.deepcopy(model)
-    except Exception as e:                                           # pragma: no cover
-        logger.warning("Fail to deep copy the model due to {}, inplace is used now.".
-                       format(repr(e)))
-        q_model = model
-
     if 'is_oneshot' in tune_cfg and tune_cfg['is_oneshot']:
-        return _load_int8_orchestration(q_model, tune_cfg, stat_dict, example_inputs, **kwargs)
+        return _load_int8_orchestration(model, tune_cfg, stat_dict, example_inputs, **kwargs)
 
-    q_model.eval()
+    model.eval()
     approach_quant_mode = None
     if tune_cfg['approach'] == "post_training_dynamic_quant":
         approach_quant_mode = 'dynamic'
@@ -262,19 +265,19 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
             op_cfg['activation']['quant_mode'] = approach_quant_mode
 
     if tune_cfg['approach'] != "post_training_dynamic_quant":
-        if version < Version("1.7.0-rc1"):   # pragma: no cover
+        if version.release < Version("1.7.0").release:   # pragma: no cover
             q_mapping = tq.default_mappings.DEFAULT_MODULE_MAPPING
-        elif version < Version("1.8.0-rc1"):   # pragma: no cover
+        elif version.release < Version("1.8.0").release:   # pragma: no cover
             q_mapping = \
                 tq.quantization_mappings.get_static_quant_module_mappings()
         else:
             q_mapping = \
                 tq.quantization_mappings.get_default_static_quant_module_mappings()
     else:
-        if version < Version("1.7.0-rc1"):   # pragma: no cover
+        if version.release < Version("1.7.0").release:   # pragma: no cover
             q_mapping = \
                 tq.default_mappings.DEFAULT_DYNAMIC_MODULE_MAPPING
-        elif version < Version("1.8.0-rc1"):   # pragma: no cover
+        elif version.release < Version("1.8.0").release:   # pragma: no cover
             q_mapping = \
                 tq.quantization_mappings.get_dynamic_quant_module_mappings()
         else:
@@ -283,7 +286,7 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
 
     if tune_cfg['framework'] == "pytorch_fx":             # pragma: no cover
         # For torch.fx approach
-        assert version >= Version("1.8.0-rc1"), \
+        assert version.release >= Version("1.8.0").release, \
                       "Please use PyTroch 1.8 or higher version with pytorch_fx backend"
         from torch.quantization.quantize_fx import prepare_fx, convert_fx, prepare_qat_fx
         if kwargs is None:
@@ -296,79 +299,91 @@ def load(checkpoint_dir=None, model=None, history_cfg=None, **kwargs):
         op_cfgs = _cfg_to_qconfig(tune_cfg, tune_cfg['approach'])
         fx_op_cfgs = _cfgs_to_fx_cfgs(op_cfgs, tune_cfg['approach'])
         if not tune_cfg['fx_sub_module_list']:
-            tmp_model = q_model
+            tmp_model = model
             if tune_cfg['approach'] == "quant_aware_training":
-                q_model.train()
-                if version > Version("1.12.1"):  # pragma: no cover
+                model.train()
+                if version.release > Version("1.12.1").release:  # pragma: no cover
                     # pylint: disable=E1123
-                    q_model = prepare_qat_fx(q_model,
-                                             fx_op_cfgs,
-                                             prepare_custom_config=prepare_custom_config_dict,
-                                             example_inputs=example_inputs)
+                    model = prepare_qat_fx(model,
+                                           fx_op_cfgs,
+                                           prepare_custom_config=prepare_custom_config_dict,
+                                           example_inputs=example_inputs)
                 else:
-                    q_model = prepare_qat_fx(q_model,
-                                             fx_op_cfgs,
-                                             prepare_custom_config_dict=prepare_custom_config_dict)
+                    model = prepare_qat_fx(model,
+                                           fx_op_cfgs,
+                                           prepare_custom_config_dict=prepare_custom_config_dict)
             else:
-                if version > Version("1.12.1"):  # pragma: no cover
+                if version.release > Version("1.12.1").release:  # pragma: no cover
                     # pylint: disable=E1123
-                    q_model = prepare_fx(q_model,
-                                         fx_op_cfgs,
-                                         prepare_custom_config=prepare_custom_config_dict,
-                                         example_inputs=example_inputs)
+                    model = prepare_fx(model,
+                                       fx_op_cfgs,
+                                       prepare_custom_config=prepare_custom_config_dict,
+                                       example_inputs=example_inputs)
                 else:
-                    q_model = prepare_fx(q_model,
-                                         fx_op_cfgs,
-                                         prepare_custom_config_dict=prepare_custom_config_dict)
-            if version > Version("1.12.1"):  # pragma: no cover
+                    model = prepare_fx(model,
+                                       fx_op_cfgs,
+                                       prepare_custom_config_dict=prepare_custom_config_dict)
+            if version.release > Version("1.12.1").release:  # pragma: no cover
                 # pylint: disable=E1123
-                q_model = convert_fx(q_model,
+                model = convert_fx(model,
                   convert_custom_config=convert_custom_config_dict)
             else:
-                q_model = convert_fx(q_model,
+                model = convert_fx(model,
                   convert_custom_config_dict=convert_custom_config_dict)
-            util.append_attr(q_model, tmp_model)
+            util.append_attr(model, tmp_model)
             del tmp_model
         else:
             sub_module_list = tune_cfg['fx_sub_module_list']
             if tune_cfg['approach'] == "quant_aware_training":
-                q_model.train()
+                model.train()
                 PyTorch_FXAdaptor.prepare_sub_graph(sub_module_list,
                                                     fx_op_cfgs,
-                                                    q_model,
+                                                    model,
                                                     prefix='',
                                                     is_qat=True,
                                                     example_inputs=example_inputs)
             else:
                 PyTorch_FXAdaptor.prepare_sub_graph(sub_module_list,
                                                     fx_op_cfgs,
-                                                    q_model,
+                                                    model,
                                                     prefix='',
                                                     example_inputs=example_inputs)
-            PyTorch_FXAdaptor.convert_sub_graph(sub_module_list, q_model, prefix='')
+            PyTorch_FXAdaptor.convert_sub_graph(sub_module_list, model, prefix='')
     else:
         if tune_cfg['approach'] == "post_training_dynamic_quant":
             op_cfgs = _cfg_to_qconfig(tune_cfg, tune_cfg['approach'])
         else:
             op_cfgs = _cfg_to_qconfig(tune_cfg)
 
-        _propagate_qconfig(q_model, op_cfgs, approach=tune_cfg['approach'])
+        _propagate_qconfig(model, op_cfgs, approach=tune_cfg['approach'])
         # sanity check common API misusage
-        if not any(hasattr(m, 'qconfig') and m.qconfig for m in q_model.modules()):
+        if not any(hasattr(m, 'qconfig') and m.qconfig for m in model.modules()):
             logger.warn("None of the submodule got qconfig applied. Make sure you "
                         "passed correct configuration through `qconfig_dict` or "
                         "by assigning the `.qconfig` attribute directly on submodules")
         if tune_cfg['approach'] != "post_training_dynamic_quant":
-            add_observer_(q_model)
-        q_model = convert(q_model, mapping=q_mapping, inplace=True)
+            if version.release < Version("2.0.0").release:
+                from torch.quantization.quantize import add_observer_
+            else:
+                from torch.quantization.quantize import _add_observer_ as add_observer_
+            add_observer_(model)
+        model = convert(model, mapping=q_mapping, inplace=True)
 
     bf16_ops_list = tune_cfg['bf16_ops_list'] if 'bf16_ops_list' in tune_cfg.keys() else []
     if len(bf16_ops_list) > 0 and (version >= Version("1.11.0-rc1")):
         from ..adaptor.torch_utils.bf16_convert import Convert
-        q_model = Convert(q_model, tune_cfg)
+        model = Convert(model, tune_cfg)
+    if not is_int8_model(model):   # pragma: no cover
+        logger.warning("The loaded model is not a int8 model.")
     if checkpoint_dir is None and history_cfg is not None:
-        _set_activation_scale_zeropoint(q_model, history_cfg)
+        _set_activation_scale_zeropoint(model, history_cfg)
     else:
-        q_model.load_state_dict(stat_dict)
-    util.get_embedding_contiguous(q_model)
-    return q_model
+        try:
+            model.load_state_dict(stat_dict)
+        except:
+            # set strict=False to avoid loading linked tensors, ignore missing_keys.
+            mismatch_log = model.load_state_dict(stat_dict, strict=False)
+            assert len(mismatch_log.unexpected_keys) == 0, \
+              "Loading state_dict failed: {}".format(mismatch_log)
+    util.get_embedding_contiguous(model)
+    return model
