@@ -50,7 +50,7 @@ required_libs = {
     'pytorch_ipex': ['torch', 'intel_extension_for_pytorch'],
     'onnxrt_qlinearops': ['onnx', 'onnxruntime'],
     'onnxrt_integerops': ['onnx', 'onnxruntime'],
-    'onnxrt_qoperator': ['onnx', 'onnxruntime'],
+    'onnxruntime': ['onnx', 'onnxruntime'],
     'mxnet': ['mxnet'],
 }
 
@@ -80,7 +80,7 @@ class LazyImport(object):
 
     def __init__(self, module_name):
         """Init LazyImport object.
-        
+
         Args:
            module_name (string): The name of module imported later
         """
@@ -373,17 +373,17 @@ def recover(fp32_model, tuning_history_path, num, **kwargs):
     from neural_compressor.adaptor import FRAMEWORKS
     adaptor = FRAMEWORKS[framework](q_config['framework_specific_info'])
     if 'onnx' in framework:
-        from neural_compressor.model.model import Model
+        from neural_compressor.model import Model
         ox_fp32_model = Model(fp32_model)
         tune_index_qmodel = adaptor.recover(ox_fp32_model, q_config)
         return tune_index_qmodel
     elif 'tensorflow' in framework:
-        from neural_compressor.model.model import Model
+        from neural_compressor.model import Model
         tf_fp32_model = Model(fp32_model)
         tune_index_qmodel = adaptor.recover_tuned_model(tf_fp32_model, q_config)
         return tune_index_qmodel
     elif 'mxnet' in framework:
-        from neural_compressor.model.model import Model
+        from neural_compressor.model import Model
         mx_fp32_model = Model(fp32_model)
         tune_index_qmodel = adaptor.recover_tuned_model(mx_fp32_model, q_config)
         return tune_index_qmodel
@@ -553,3 +553,12 @@ def set_tensorboard(tensorboard: bool):
     """Set the tensorboard in config."""
     from neural_compressor.config import options
     options.tensorboard = tensorboard
+
+def show_memory_info(hint):
+    """Show process full memory."""
+    pid = os.getpid()
+    p = psutil.Process(pid)
+
+    info = p.memory_full_info()
+    memory = info.uss / 1024. / 1024
+    print('{} memory used: {} MB'.format(hint, memory))
\ No newline at end of file
